/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse stellt die interne Datenstruktur eines Zeitfilters dar und kann
 * zur Parametrierung und als Rckgabewert des Zeitfilters benutzt werden.
 * Erfasst werden unter anderem die selektierten Monate, Tage und Wochentage
 * sowie eine Liste der zugehrigen Zeitgruppen.
 * 
 * @author HofmannJ
 * 
 */
public class KalenderZeitFilterDatenStruktur implements Cloneable {

	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	private static int anzahlMonate = 12;

	private static int anzahlTage = 31;

	private static int anzahlWochentage = 7;

	/**
	 * Array ber 12 Monate von Januar - Dezember true == Monat ist gewhlt,
	 * false == Monat ist nicht gewhlt.
	 */
	private final boolean[] auswahlMonate = new boolean[KalenderZeitFilterDatenStruktur.anzahlMonate];

	/**
	 * Array ber 31 Tage von 1 - 31 true == Tag ist gewhlt, false == Tag ist
	 * nicht gewhlt.
	 */
	private final boolean[] auswahlTage = new boolean[KalenderZeitFilterDatenStruktur.anzahlTage];

	/**
	 * Array ber 7 Wochentage, Montag bis Sonntag true == Wochentag ist
	 * gewhlt, false == Wochentag ist nicht gewhlt.
	 */
	private final boolean[] auswahlWochentage = new boolean[KalenderZeitFilterDatenStruktur.anzahlWochentage];

	private List<KalenderZeitFilterDatenStrukturZeitGruppe> listeZeitGruppen;

	/**
	 * Basiskonstruktor dieser Klasse, startet die basisInitialisierung.
	 * 
	 */
	public KalenderZeitFilterDatenStruktur() {
		basisInitialisierung();
	}

	/**
	 * Basiskonstruktor dieser Klasse, startet die basisInitialisierung.
	 * 
	 * @param String
	 *            Reprsentation des Filters.
	 */
	public KalenderZeitFilterDatenStruktur(final String filter) {
		basisInitialisierung();
		erzeugeAusFilterString(filter);
	}

	/**
	 * Fhrt eine Basisinitialisierung der Felder Monate, Tage, Wochentage mit
	 * false durch.
	 * 
	 */
	private void basisInitialisierung() {
		final GregorianCalendar ende = new GregorianCalendar(1970, 1, 2, 0, 0,
				0);
		ende.add(Calendar.MILLISECOND, -1);

		// Monate auf False setzen
		for (int i = 0; i < KalenderZeitFilterDatenStruktur.anzahlMonate; i++) {
			auswahlMonate[i] = false;
		}

		// Tage aus False setzen
		for (int i = 0; i < KalenderZeitFilterDatenStruktur.anzahlTage; i++) {
			auswahlTage[i] = false;
		}

		// Wochentage aus False setzen
		for (int i = 0; i < KalenderZeitFilterDatenStruktur.anzahlWochentage; i++) {
			auswahlWochentage[i] = false;
		}

		// Zeitgruppenliste initialisieren
		listeZeitGruppen = new LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe>();
	}

	/**
	 * Liefert ein Array mit den Nummern der selektierten Monate.
	 * 
	 * @return Array der selektierten Monate
	 */
	public int[] getSelektierteMonate() {

		int[] returnWert = null;
		final List<Integer> monate = new LinkedList<Integer>();

		for (int i = 0; i < auswahlMonate.length; i++) {
			if (auswahlMonate[i]) {
				monate.add(Integer.valueOf(i));
			}
		}

		returnWert = new int[monate.size()];
		int j = 0;

		final ListIterator<Integer> li = monate.listIterator();
		while (li.hasNext()) {
			returnWert[j] = li.next().intValue();
			j++;
		}

		return returnWert;
	}

	/**
	 * Liefert ein Array mit den Nummern der selektierten Tage.
	 * 
	 * @return Array mit den Nummern der selektierten Tage
	 */
	public int[] getSelektierteTage() {

		int[] returnWert = null;
		final List<Integer> tage = new LinkedList<Integer>();

		for (int i = 0; i < auswahlTage.length; i++) {
			if (auswahlTage[i]) {
				tage.add(Integer.valueOf(i));
			}
		}

		returnWert = new int[tage.size()];
		int j = 0;

		final ListIterator<Integer> li = tage.listIterator();
		while (li.hasNext()) {
			returnWert[j] = li.next().intValue();
			j++;
		}

		return returnWert;
	}

	/**
	 * Liefert ein Array mit den Nummern der selektierten Wochentage.
	 * 
	 * @return Array mit den Nummern der Selektierten Wochentage.
	 */
	public int[] getSelektierteWochentage() {

		int[] returnWert = null;
		final List<Integer> wochentage = new LinkedList<Integer>();

		for (int i = 0; i < auswahlWochentage.length; i++) {
			if (auswahlWochentage[i]) {
				wochentage.add(Integer.valueOf(i));
			}
		}

		// returnWert = new int[wochentage.size()];
		// int j = 0;

		// ListIterator<Integer> li = wochentage.listIterator();
		// while (li.hasNext())
		// {
		// returnWert[j] = li.next().intValue();
		// j++;
		// }

		final List<Integer> tempWochentage = new LinkedList<Integer>();
		int i = -1;
		for (final Boolean bool : auswahlWochentage) {
			i++;
			if (bool) {
				tempWochentage.add(i);
			}
		}

		returnWert = new int[tempWochentage.size()];
		final Iterator<Integer> it = tempWochentage.iterator();
		int j = -1;
		while (it.hasNext()) {
			j++;
			returnWert[j] = it.next();
		}

		return returnWert;
	}

	/**
	 * TextFilter.
	 */
	public String erzeugeFilterString() {
		String returnString = "";

		String monate = "";
		String tage = "";
		String wochentage = "";
		String zeitgruppen = "";

		/*
		 * Monate
		 */
		final int[] selektierteMonate = getSelektierteMonate();
		if ((selektierteMonate.length == 0)
				|| (selektierteMonate.length >= KalenderZeitFilterDatenStruktur.anzahlMonate)) {
			monate = "* ";
		} else {
			for (final int i : selektierteMonate) {
				final int wert = i + 1;
				monate = monate + wert + ",";
			}
			monate = monate.substring(0, monate.length() - 1) + " ";
		}

		/*
		 * Tage
		 */
		final int[] selektierteTage = getSelektierteTage();
		if ((selektierteTage.length == 0)
				|| (selektierteTage.length >= KalenderZeitFilterDatenStruktur.anzahlTage)) {
			tage = "* ";
		} else {
			for (final int i : selektierteTage) {
				int wert = i;
				wert = wert + 1;
				tage = tage + wert + ",";
			}
			tage = tage.substring(0, tage.length() - 1) + " ";
		}

		/*
		 * Wochentage
		 */
		final int[] selektierteWochentage = getSelektierteWochentage();
		if ((selektierteWochentage.length == 0)
				|| (selektierteWochentage.length >= KalenderZeitFilterDatenStruktur.anzahlWochentage)) {
			wochentage = "* ";
		} else {
			for (final int i : selektierteWochentage) {
				switch (i) {
				case 0:
					wochentage = wochentage + "Mo,";
					break;
				case 1:
					wochentage = wochentage + "Di,";
					break;
				case 2:
					wochentage = wochentage + "Mi,";
					break;

				case 3:
					wochentage = wochentage + "Do,";
					break;
				case 4:
					wochentage = wochentage + "Fr,";
					break;
				case 5:
					wochentage = wochentage + "Sa,";
					break;
				case 6:
					wochentage = wochentage + "So,";
					break;
				default:
					break;
				}
			}
			wochentage = wochentage.substring(0, wochentage.length() - 1) + " ";
		}

		/*
		 * Zeitgruppen
		 */
		final List<KalenderZeitFilterDatenStrukturZeitGruppe> gruppen = getListeZeitGruppen();
		if (gruppen.size() == 0) {
			zeitgruppen = "*";
		} else {
			final Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = gruppen
					.iterator();
			/*
			 * DateFormat df = new SimpleDateFormat("hh:mm:s.S"); String ausgabe
			 * = df.format(date);
			 */
			while (it.hasNext()) {
				final KalenderZeitFilterDatenStrukturZeitGruppe zeitGruppe = it
						.next();
				final long startzeitpunkt = zeitGruppe.getStartzeitpunkt();
				final long endzeitpunkt = zeitGruppe.getEndzeitpunkt();

				final DateFormat df = new SimpleDateFormat("HH:mm:s.S");
				final String startString = df.format(new Date(startzeitpunkt));
				final String endeString = df.format(new Date(endzeitpunkt));
				zeitgruppen = zeitgruppen + startString + "-" + endeString
						+ ",";
			}
			zeitgruppen = zeitgruppen.substring(0, zeitgruppen.length() - 1);
		}

		returnString = returnString + monate + tage + wochentage + zeitgruppen;

		return returnString;
	}

	public final void erzeugeAusFilterString(final String eingang) {
		// String eingang =
		// "* 2,4,5 Di 00:00:0.0-00:00:12.0,00:13:0.0-00:44:0.0";
		final String[] splitLeer = eingang.split(" ");

		final String temp = "===" + getClass().getName()
				+ "::erzeugeAusFilterString===";

		String temp2 = "";
		for (int i = 0; i < temp.length(); i++) {
			temp2 = temp2 + "=";
		}

		final String monat = splitLeer[0];
		final String tage = splitLeer[1];
		final String wochentage = splitLeer[2];
		final String zeiten = splitLeer[3];

		/*
		 * Monate
		 */
		if (monat.contains("*")) {
			for (int i = 0; i < auswahlMonate.length; i++) {
				auswahlMonate[i] = true;
			}

		} else {
			// Splitten nach dem ,
			final String[] monateZahlen = monat.split(",");

			for (final String stringMonat : monateZahlen) {
				if (stringMonat.contains("-")) {

					final String[] startEndeMonat = stringMonat.split("-");
					final int start = Integer.parseInt(startEndeMonat[0]);
					final int ende = Integer.parseInt(startEndeMonat[1]);

					if ((start > 0)
							&& (ende <= KalenderZeitFilterDatenStruktur.anzahlMonate)) {
						for (int i = start - 1; i < ende; i++) {
							selektiereMonat(i);
						}
					} else {
						KalenderZeitFilterDatenStruktur.LOGGER
								.error(getClass().getName()
										+ "::erzeugeAusFilterString \n\t=>Der Angegebene Monatsbereich ist ungltig!");
					}

				} else {
					final int tempMonat = Integer.parseInt(stringMonat);
					if (tempMonat <= 12) {
						if (tempMonat > 0) {
							selektiereMonat(tempMonat - 1);
						} else {
							KalenderZeitFilterDatenStruktur.LOGGER
									.error(getClass().getName()
											+ "::erzeugeAusFilterString \n\t=>Es wurde ein Monat kleiner gleich 0 angegeben. Dieser existiert nicht!");
						}

					} else {
						KalenderZeitFilterDatenStruktur.LOGGER
								.error(getClass().getName()
										+ "::erzeugeAusFilterString \n\t=>Es wurde ein Monat grer 12 angegeben. Dieser existiert nicht!");
					}
				}
			}

			/*
			 * monateRueckgabe = new int[monateZahlen.length]; int i = -1; for
			 * (String string : monateZahlen) { i++; monateRueckgabe[i] =
			 * Integer.parseInt(string); }
			 * 
			 * leseSelektierteMonateEin(monateRueckgabe);
			 */
		}

		/*
		 * Tage
		 */
		if (tage.contains("*")) {
			for (int i = 0; i < auswahlTage.length; i++) {
				auswahlTage[i] = true;
			}

		} else {
			final String[] stringsTage = tage.split(",");
			for (final String stringTag : stringsTage) {
				if (stringTag.contains("-")) {
					final String[] startEndeTage = stringTag.split("-");
					final int start = Integer.parseInt(startEndeTage[0]);
					final int ende = Integer.parseInt(startEndeTage[1]);
					if ((start > 0)
							&& (ende <= KalenderZeitFilterDatenStruktur.anzahlTage)) {
						for (int i = start - 1; i < ende; i++) {
							selektiereTag(i);
						}
					}
				} else {
					int tag = Integer.parseInt(stringTag);
					tag = tag - 1;
					selektiereTag(tag);
				}
			}

		}

		/*
		 * Wochentage
		 */
		if (wochentage.contains("*")) {

			for (int i = 0; i < auswahlWochentage.length; i++) {
				auswahlWochentage[i] = true;
			}

		} else {
			final String[] stringsWochentage = wochentage.split(",");

			for (final String string : stringsWochentage) {

				if ("Mo".equals(string)) {
					selektiereWochentag(0);
				} else if ("Di".equals(string)) {
					selektiereWochentag(1);
				} else if ("Mi".equals(string)) {
					selektiereWochentag(2);
				} else if ("Do".equals(string)) {
					selektiereWochentag(3);
				} else if ("Fr".equals(string)) {
					selektiereWochentag(4);
				} else if ("Sa".equals(string)) {
					selektiereWochentag(5);
				} else if ("So".equals(string)) {
					selektiereWochentag(6);
				}

			}

		}

		/*
		 * Zeiten
		 */
		if (zeiten.contains("*")) {

			fuegeZeitGruppeHinzu(2674800000L, 2761199999L);
		} else {
			// Splitten nach ,
			final String[] stringsZeiten = zeiten.split(",");

			for (final String stringZeitKomplett : stringsZeiten) {
				if (stringZeitKomplett.contains("-")) {
					final String[] stringsZeitenBis = stringZeitKomplett
							.split("-");

					final String zeitVon = stringsZeitenBis[0];
					final String zeitBis = stringsZeitenBis[1];

					long zeitVonLong = -3600000L;
					long zeitBisLong = -3600000L;

					// Bestimmen Anzahl : in zeitVon
					final int anzDoppelpkteZeitVon = zeitVon.split(":").length - 1;
					final int anzSemikolaZeitVon = zeitVon.split("\\.").length - 1;
					final int anzDoppelpkteZeitBis = zeitBis.split(":").length - 1;
					final int anzSemikolaZeitBis = zeitBis.split("\\.").length - 1;

					// VonZeit
					if (anzDoppelpkteZeitVon == 0) {
						final DateFormat df = new SimpleDateFormat("HH");
						try {
							zeitVonLong = df.parse(zeitVon).getTime();
						} catch (final ParseException e) {
							e.printStackTrace();
						}
					} else if (anzDoppelpkteZeitVon == 1) {
						final DateFormat df = new SimpleDateFormat("HH:mm");
						try {
							zeitVonLong = df.parse(zeitVon).getTime();
						} catch (final ParseException e) {
							e.printStackTrace();
						}
					} else if (anzDoppelpkteZeitVon == 2) {
						if (anzSemikolaZeitVon == 1) {
							final DateFormat df = new SimpleDateFormat(
									"HH:mm:s.S");
							try {
								zeitVonLong = df.parse(zeitVon).getTime();
							} catch (final ParseException e) {
								e.printStackTrace();
							}
						} else {

							final DateFormat df = new SimpleDateFormat(
									"HH:mm:s");
							try {
								zeitVonLong = df.parse(zeitVon).getTime();
							} catch (final ParseException e) {
								e.printStackTrace();
							}
						}
					}

					// Bis-Zeit
					if (anzDoppelpkteZeitBis == 0) {
						final DateFormat df = new SimpleDateFormat("HH");
						try {
							zeitBisLong = df.parse(zeitBis).getTime();
						} catch (final ParseException e) {
							e.printStackTrace();
						}
					} else if (anzDoppelpkteZeitBis == 1) {
						final DateFormat df = new SimpleDateFormat("HH:mm");
						try {
							zeitBisLong = df.parse(zeitBis).getTime();
						} catch (final ParseException e) {
							e.printStackTrace();
						}
					} else if (anzDoppelpkteZeitBis == 2) {
						if (anzSemikolaZeitBis == 1) {
							final DateFormat df = new SimpleDateFormat(
									"HH:mm:s.S");
							try {
								zeitBisLong = df.parse(zeitBis).getTime();
							} catch (final ParseException e) {
								e.printStackTrace();
							}
						} else {

							final DateFormat df = new SimpleDateFormat(
									"HH:mm:s");
							try {
								zeitBisLong = df.parse(zeitBis).getTime();
							} catch (final ParseException e) {
								e.printStackTrace();
							}
						}
					}

					if ((zeitVonLong > -3600001) && (zeitBisLong > -3600001)) {
						fuegeZeitGruppeHinzu(zeitVonLong, zeitBisLong);
					}

				} else {
					// einzelne Zahl, steht fr ganze Stunde
					final String zeitVon = stringZeitKomplett;
					long zeitVonLong = -1L;
					long zeitBisLong = -1L;
					final DateFormat df = new SimpleDateFormat("HH");
					try {
						zeitVonLong = df.parse(zeitVon).getTime();
						zeitBisLong = (zeitVonLong + (1000 * 60 * 60)) - 1;
						fuegeZeitGruppeHinzu(zeitVonLong, zeitBisLong);
					} catch (final ParseException e) {
						KalenderZeitFilterDatenStruktur.LOGGER
								.error(getClass().getName()
										+ "::erzeugeAusFilterString \n\t=>VonZeit bei einzeln angegebener Stunde konnte nicht bestimmt werden.");
						e.printStackTrace();
					}
				}

			}

		}

	}

	/*
	 * Selektoren/Deselektoren fr Monate, Tage und Wochentage
	 */

	/**
	 * Selektiert einen Monat und setzt ihn auf true.
	 * 
	 * @param i
	 *            Nummer des Monats, der selektiert werden soll
	 */
	public void selektiereMonat(final int i) {
		auswahlMonate[i] = true;
	}

	/**
	 * Deselektiert einen Monat und setzt ihn auf false.
	 * 
	 * @param i
	 *            Nummer des Monats, der deselektiert soll
	 */
	public void deSelektiereMonat(final int i) {
		auswahlMonate[i] = false;
	}

	/**
	 * Selektiert einen Tag und setzt ihn auf true.
	 * 
	 * @param i
	 *            Nummer des Tages, der selektiert werden soll
	 */
	public void selektiereTag(final int i) {
		auswahlTage[i] = true;
	}

	/**
	 * Deselektiert einen Tag und setzt ihn auf false.
	 * 
	 * @param i
	 *            Nummer des tages, der selektiert werden soll
	 * 
	 */
	public void deSelektiereTag(final int i) {
		auswahlTage[i] = false;
	}

	/**
	 * Selektiert einen Wochentag und setzt ihn auf true.
	 * 
	 * @param i
	 *            Nummer des Wochentags, der selektiert werden soll
	 */
	public void selektiereWochentag(final int i) {
		auswahlWochentage[i] = true;
	}

	/**
	 * Deselektiert einen Wochentag und setzt ihn auf false.
	 * 
	 * @param i
	 *            Nummer des Wochentags, der deselektiert werden soll
	 */
	public void deSelektiereWochentag(final int i) {
		auswahlWochentage[i] = false;
	}

	/**
	 * Fgt eine Zeitgruppe zur Liste der Zeitgruppen hinzu.
	 * 
	 * @param startzeitpunkt
	 *            Startzeitpunkt als long (Zeit in MS seit 1.1.1970)
	 * @param endzeitpunkt
	 *            Enzeitpunkt als long (Zeit in MS seit 1.1.1970)
	 */
	public void fuegeZeitGruppeHinzu(final long startzeitpunkt,
			final long endzeitpunkt) {
		final KalenderZeitFilterDatenStrukturZeitGruppe zg = new KalenderZeitFilterDatenStrukturZeitGruppe(
				startzeitpunkt, endzeitpunkt);
		listeZeitGruppen.add(zg);
	}

	/**
	 * Fgt eine Zeitgruppe zur Liste der Zeitgruppen hinzu.
	 * 
	 * @param zg
	 *            Zeitgruppe, die hinzugefgt werden soll.
	 */
	public void fuegeZeitGruppeHinzu(
			final KalenderZeitFilterDatenStrukturZeitGruppe zg) {
		if (!listeZeitGruppen.contains(zg)) {
			listeZeitGruppen.add(zg);
		} else {
			System.err
					.println(getClass().getName()
							+ "::fuegeZeitGruppeHinzu \n\t=>Die bergebene Zeitgruppe ist bereits in der Liste der Zeitgruppen vorhanden.");
		}
	}

	/**
	 * Entfernt eine Zeitgruppe aus der Liste der Zeitgruppen.
	 * 
	 * @param zg
	 *            Zeitgruppe, die entfernt werden soll.
	 */
	public void entferneZeitGruppe(
			final KalenderZeitFilterDatenStrukturZeitGruppe zg) {
		listeZeitGruppen.remove(zg);
	}

	/**
	 * Liefert die Liste der ZeitGruppen zurck.
	 * 
	 * @return Liste der Zeitgruppen
	 */
	public List<KalenderZeitFilterDatenStrukturZeitGruppe> getListeZeitGruppen() {
		return listeZeitGruppen;
	}

	/**
	 * Liefert einen Clone dieser Klasse. Funktioniert intern ber
	 * erzeugeFilterString und erzeugeAusFilterString Methode dieser Klasse.
	 */
	@Override
	public KalenderZeitFilterDatenStruktur clone() {

		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new IllegalStateException(e);
		}

		KalenderZeitFilterDatenStruktur rueckgabe = null;
		final String filterString = this.erzeugeFilterString();
		rueckgabe = new KalenderZeitFilterDatenStruktur(filterString);
		return rueckgabe;
	}

}
