/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liefert die Reprsentation eines Ereigniskalender Eintrages.
 * 
 * Der Eintrag enthlt nur die Daten, die fr die Verwendung in den
 * Zeitauswahldialogen erfordrlich sind!
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ(Dambach)
 * @version $Id: EintragEreigniskalender.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class EintragEreigniskalender implements
		Comparable<EintragEreigniskalender> {

	/** Pid des Ereigniskalendereintrags. */
	private final String pid;

	/** Name des Ereigniskalendereintrags. */
	private final String name;

	/**
	 * Speichert den Anfangszeitpunkt in Millisekunden seit dem 1.1.1970 bei der
	 * Zeitbereichsangabe.
	 */
	private final long anfangsZeitpunkt;

	/**
	 * Speichert den Endzeitpunkt in Millisekunden seit dem 1.1.1970 bei der
	 * Zeitbereichsangabe.
	 */
	private final long endZeitpunkt;

	/**
	 * Speichert das Datum in Millisekunden seit dem 1.1.1970 bei der
	 * Datumseingabe.
	 */
	private final SystemObject systemKalenderEintrag;

	/**
	 * erzeugt eine Instanz eines Ereigniskalender-Eintrags, welcher im
	 * Datenverteiler durch das bergebene Systemobjekt reprsentiert wird.
	 * 
	 * Es werden die bergebenen Parameter ("atg.ereignisParameter") und die
	 * Konfigurationsdaten des Objekts ausgewertet.
	 * 
	 * @param objekt
	 *            das Systemobjekt
	 * @param parameterData
	 *            die bergebenen Parameterdaten
	 */
	public EintragEreigniskalender(final SystemObject objekt,
			final Data parameterData) {
		pid = objekt.getPid();
		name = objekt.getName();

		final AttributeGroup atg = objekt.getDataModel().getAttributeGroup(
				"atg.ereignisEigenschaften");
		final Data configData = objekt.getConfigurationData(atg);
		configData.getItem("Ereignisbeschreibung").asTextValue().getText();
		final SystemObject systemObject = parameterData
				.getItem("SystemKalenderEintragReferenz").asReferenceValue()
				.getSystemObject();

		final long anfang = parameterData.getItem("BeginnZeitlicheGltigkeit")
				.asTimeValue().getMillis();
		final long ende = parameterData.getItem("EndeZeitlicheGltigkeit")
				.asTimeValue().getMillis();

		anfangsZeitpunkt = anfang;
		endZeitpunkt = ende;
		systemKalenderEintrag = systemObject;
	}

	/**
	 * Liefert den Namen des Ereigniskalender Datensatzes.
	 * 
	 * @return Name des Ereigniskalender Datensatzes.
	 */
	public String getName() {
		return name;
	}

	/**
	 * liefert den Anfangszeitpunkt bei Zeitbereichseingabe.
	 * 
	 * @return Anfangszeitpunkt in Millisekunden seit dem 1.1.1970.
	 */
	long getAnfangsZeitpunkt() {
		return anfangsZeitpunkt;
	}

	/**
	 * liefert den Endzeitpunkt bei Zeitbereichseingabe.
	 * 
	 * @return Endzeitpunkt in Millisekunden seit dem 1.1.1970.
	 */
	long getEndZeitpunkt() {
		return endZeitpunkt;
	}

	/**
	 * Liefert die Pid des Ereigniskalender.
	 * 
	 * @return Pid des Ereigniskalender.
	 */
	String getPid() {
		return pid;
	}

	/**
	 * liefert das Systemobjekt, welches den Kalendereintrag reprsentiert.
	 * 
	 * @return das Systemobjekt
	 */
	SystemObject getSystemObject() {
		return systemKalenderEintrag;
	}

	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer(100);
		buffer.append(getName());
		buffer.append(" (Pid: ");
		buffer.append(getPid());
		buffer.append(')');
		return buffer.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + ((pid == null) ? 0 : pid.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final EintragEreigniskalender other = (EintragEreigniskalender) obj;
		if (pid == null) {
			if (other.pid != null) {
				return false;
			}
		} else if (!pid.equals(other.pid)) {
			return false;
		}
		return true;
	}

	@Override
	public int compareTo(final EintragEreigniskalender o) {
		final String pidA = getPid();
		final String pidB = o.getPid();
		return pidA.compareTo(pidB);
	}

}
