/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.legende;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.part.WorkbenchPart;

import de.bsvrz.buv.rw.basislib.legende.ILegende.Corner;

/**
 * Ein Toolfenster, welches ber einem Workbench Part schwebt. Das Toolfenster
 * in einer der Ecken des Workbench Parts gelegt werden und folgt auch dem
 * Workbench Part, wenn dieser verschoben oder in der Gre gendert wird.
 * 
 * <p>
 * <em>Hinweis fr abgeleitete Klasse:</em>
 * 
 * <p>
 * Es muss {@link #createContents(Composite)} berschrieben werden, um den
 * Inhalt des Fensters anzulegen. Der Fenstertitel und die Fenstergre sollte
 * in {@link #configureShell(Shell)} festgelegt werden.
 * 
 * <p>
 * ber {@link WorkbenchPart#getAdapter(Class)} knnen Objekte zur Kommunikation
 * mit dem Workbench Part bezogen werden.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id: ToolWindow.java 47112 2013-11-12 15:25:14Z peuker $
 */
abstract class ToolWindow extends Window {

	/**
	 * Richtet eine Shell in einer Ecke eines Controls aus.
	 * 
	 * @param shell
	 *            die auszurichtende Shell.
	 * @param control
	 *            das Control in dem die Shell ausgerichtet werden soll.
	 */
	private static void relocateTopLeft(final Shell shell, final Control control) {
		if (control == null) {
			return;
		}

		shell.setLocation(control.toDisplay(0, 0));
	}

	/**
	 * Richtet eine Shell in einer Ecke eines Controls aus.
	 * 
	 * @param shell
	 *            die auszurichtende Shell.
	 * @param control
	 *            das Control in dem die Shell ausgerichtet werden soll.
	 */
	private static void relocateTopRight(final Shell shell,
			final Control control) {
		if (control == null) {
			return;
		}

		final int windowWidth = shell.getSize().x;
		final int partControlWidth = ((Composite) control).getClientArea().width;
		final int x = partControlWidth - windowWidth - 25;

		shell.setLocation(control.toDisplay(x, 0));
	}

	/**
	 * Richtet eine Shell in einer Ecke eines Controls aus.
	 * 
	 * @param shell
	 *            die auszurichtende Shell.
	 * @param control
	 *            das Control in dem die Shell ausgerichtet werden soll.
	 */
	private static void relocateBottomLeft(final Shell shell,
			final Control control) {
		if (control == null) {
			return;
		}

		final int windowHeight = shell.getSize().y;
		final int partControlHeight = ((Composite) control).getClientArea().height;
		final int y = partControlHeight - windowHeight - 25;

		shell.setLocation(control.toDisplay(0, y));
	}

	/**
	 * Richtet eine Shell in einer Ecke eines Controls aus.
	 * 
	 * @param shell
	 *            die auszurichtende Shell.
	 * @param control
	 *            das Control in dem die Shell ausgerichtet werden soll.
	 */
	private static void relocateBottomRight(final Shell shell,
			final Control control) {
		if (control == null) {
			return;
		}

		final int windowWidth = shell.getSize().x;
		final int partControlWidth = ((Composite) control).getClientArea().width;
		final int x = partControlWidth - windowWidth - 25;

		final int windowHeight = shell.getSize().y;
		final int partControlHeight = ((Composite) control).getClientArea().height;
		final int y = partControlHeight - windowHeight - 25;

		shell.setLocation(control.toDisplay(x, y));
	}

	private final IWorkbenchPart part;
	private final Control partControl;
	private Corner corner;

	{
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.TOOL);
	}

	/**
	 * Initialisiert das Toolfenster.
	 * 
	 * @param part
	 *            der Workbench Part zu dem das Toolfenster gehrt.
	 * @param partControl
	 *            das Control des Workbench Parts bzw. das Control an dessen
	 *            Ecken das Toolfenster ausgerichtet wird.
	 */
	public ToolWindow(final IWorkbenchPart part, final Control partControl) {
		super(part.getSite());

		Assert.isNotNull(part, "Der Workbench Part darf nicht null sein.");
		this.part = part;
		this.partControl = partControl;
	}

	/**
	 * Gibt die Ecke des Workbench Parts zurck, in dem ds Fenster angeordnet
	 * werden soll.
	 * 
	 * <p>
	 * Wurde keine Ecke festgelegt, wird {@link Corner#TopLeft} zurckgegeben.
	 * 
	 * @return die Ecke des Workbench Parts.
	 */
	public Corner getCorner() {
		if (corner == null) {
			return Corner.TopLeft;
		}
		return corner;
	}

	/**
	 * Legt die Ecke des Workbench Parts fest, in dem ds Fenster angeordnet
	 * werden soll.
	 * 
	 * @param corner
	 *            die Ecke des Workbench Parts
	 */
	public void setCorner(final Corner corner) {
		if (corner == null) {
			this.corner = Corner.TopLeft;
		} else {
			this.corner = corner;
		}
	}

	protected final void relocate(final Shell shell) {
		switch (getCorner()) {
		case TopLeft:
			ToolWindow.relocateTopLeft(shell, getPartControl());
			break;
		case TopRight:
			ToolWindow.relocateTopRight(shell, getPartControl());
			break;
		case BottomLeft:
			ToolWindow.relocateBottomLeft(shell, getPartControl());
			break;
		case BottomRight:
		default:
			ToolWindow.relocateBottomRight(shell, getPartControl());
			break;
		}
	}

	/**
	 * Gibt den verbundenen Workbench Part zurck.
	 * 
	 * @return der Workbench Part.
	 */
	public IWorkbenchPart getWorkbenchPart() {
		return part;
	}

	/**
	 * Gibt das Control zurck an dem das Toolfenster ausgerichtet wird.
	 * 
	 * @return das Control an dem ausgerichtet wird.
	 */
	public Control getPartControl() {
		return partControl;
	}

	@Override
	public void create() {
		super.create();
		relocate(getShell());
	}

	/**
	 * Registriert die notwendigen Listener um die Position des Toolfensters mit
	 * dem Workbench Part zu synchronisieren.
	 */
	@Override
	protected void configureShell(final Shell newShell) {
		super.configureShell(newShell);

		final ControlListener controlListener;
		if (getPartControl() == null) {
			controlListener = null;
		} else {
			// Window an mit der Position des Workbench Parts synchronisieren
			controlListener = new ControlListener() {

				@Override
				public void controlMoved(final ControlEvent e) {
					if ((newShell != null) && (!newShell.isDisposed())) {
						relocate(newShell);
					}
				}

				@Override
				public void controlResized(final ControlEvent e) {
					if ((newShell != null) && (!newShell.isDisposed())) {
						relocate(newShell);
					}
				}

			};

			getPartControl().addControlListener(controlListener);
			getWorkbenchPart().getSite().getShell()
					.addControlListener(controlListener);
		}

		// Window mit Sichtbarkeit des Workbench Parts synchronisieren
		final IPartListener2 partListener = new IPartListener2() {

			@Override
			public void partVisible(final IWorkbenchPartReference partRef) {
				if (part == partRef.getPart(false)) {
					getShell().setVisible(true);
				}
			}

			@Override
			public void partOpened(final IWorkbenchPartReference partRef) {
				// tut nix
			}

			@Override
			public void partInputChanged(final IWorkbenchPartReference partRef) {
				// tut nix
			}

			@Override
			public void partHidden(final IWorkbenchPartReference partRef) {
				if (part == partRef.getPart(false)) {
					getShell().setVisible(false);
				}
			}

			@Override
			public void partDeactivated(final IWorkbenchPartReference partRef) {
				// tut nix
			}

			/**
			 * Schliet die Legende, wenn der Workbench Part geschlossen wird.
			 */
			@Override
			public void partClosed(final IWorkbenchPartReference partRef) {
				if (part == partRef.getPart(false)) {
					close();
				}
			}

			@Override
			public void partBroughtToTop(final IWorkbenchPartReference partRef) {
				// tut nix
			}

			@Override
			public void partActivated(final IWorkbenchPartReference partRef) {
				// tut nix
			}
		};
		part.getSite().getPage().addPartListener(partListener);

		newShell.addDisposeListener(new DisposeListener() {

			@Override
			public void widgetDisposed(final DisposeEvent e) {
				part.getSite().getPage().removePartListener(partListener);

				if (controlListener != null) {
					getPartControl().removeControlListener(controlListener);
				}
			}

		});
	}

	/**
	 * Legt das Layout der Shell auf {@link FillLayout} fest. Kann in
	 * abgeleiteten Klassen angepasst werden.
	 */
	@Override
	protected Layout getLayout() {
		return new FillLayout();
	}

}
