/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hilfsklasse, realisiert einen Baum zur Aufnahme der Zellenstruktur der
 * Kopfzeilen. Jeder {@link Knoten} verfgt ber nur einen Vorgnger und
 * beliebig viele Nachfolger.
 * 
 * @author HofmannJ
 * 
 */
public class Knoten {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Speichert den Vorgnger Knoten. Wenn null, dann ist auch die Ebene 0 und
	 * es gibt keinen Vorgnger.
	 */
	private Knoten vorgaenger;

	/**
	 * Liste der Nachfolger dieses Knotens. Ist die Liste leer, so hat dieser
	 * Knoten keinen Nachfolgeknoten!
	 */
	private final LinkedList<Knoten> nachfolger = new LinkedList<Knoten>();

	/**
	 * Wert des Knotens.
	 */
	private Object wert;

	private int vonWert;

	private int bisWert;

	/**
	 * Start X der Zelle (oben Links).
	 */
	private int vonX = -1;

	/**
	 * Start Y der Zelle (oben Links).
	 */
	private int vonY = -1;

	/**
	 * End X der Zelle (unten Rechts).
	 */
	private int bisX = -1;

	/**
	 * End Y der Zelle (unten Rechts).
	 */
	private int bisY = -1;

	/**
	 * Konstruktor. bernimmt den Wert des Knotens.
	 * 
	 * @param wert
	 *            Wert des Knotens.
	 */
	public Knoten(final Object wert) {
		this.wert = wert;
	}

	/**
	 * Setzt den Vorgngerknoten.
	 * 
	 * @param vorgaenger
	 *            Vorgngerknoten.
	 */
	public void setVorgaenger(final Knoten vorgaenger) {
		this.vorgaenger = vorgaenger;
	}

	/**
	 * Fgt einen Nachfolgerknoten zur Liste der Nachfolger hinzu. In dem
	 * Nachfolgeknoten wird dieser Knoten als Vorgnger gesetzt. <br>
	 * <br>
	 * <b>Achtung:</b> Jeder Knoten kann nur einen Vorgnger haben und jeder
	 * Knoten kann nur ein mal in der Liste der Nachfolger auftauchen. Die
	 * Prfung, ob ein Knoten in der Hierarchie schon einmal verwendet wurde,
	 * bleibt dem Entwickler vorbehalten.
	 * 
	 * @param nachfolgeKnoten
	 *            Hinzuzufgender Nachfolgeknoten.
	 */
	public void addNachfolger(final Knoten nachfolgeKnoten) {
		if (!nachfolger.contains(nachfolgeKnoten)) {
			nachfolgeKnoten.setVorgaenger(this);
			nachfolger.add(nachfolgeKnoten);
		}
	}

	/**
	 * Entfernt einen nachfolgenden Knoten von der Liste der Nachfolger. Der
	 * Vorgnger des nachfolgenden Knotens wird auf <code>null</code> gesetzt.
	 * 
	 * @param nachfolgeKnoten
	 *            Zu entfernender, nachfolgender Knoten.
	 */
	public void removeNachfolger(final Knoten nachfolgeKnoten) {
		if (nachfolger.contains(nachfolgeKnoten)) {
			nachfolgeKnoten.setVorgaenger(null);
			nachfolger.remove(nachfolgeKnoten);
		}
	}

	/**
	 * Prft, ob der Knoten Nachfolger hat.
	 * 
	 * @return Liefert <code>true</code>, wenn der Knoten Nachfolger hat, sonst
	 *         <code>false</code>.
	 */
	public boolean hatNachfolger() {
		if (nachfolger.size() == 0) {
			return false;
		}
		return true;
	}

	/**
	 * Liefert die Liste der Nachfolger dieses Knoten zurck.
	 * 
	 * @return Liste der Nachfolger dieses Knotens.
	 */
	public LinkedList<Knoten> getNachfolger() {
		return nachfolger;
	}

	/**
	 * Setzt den Wert des Knotens.
	 * 
	 * @param wert
	 *            Zu setzender Wert.
	 */
	public void setWert(final Object wert) {
		this.wert = wert;
	}

	/**
	 * Liefert den Wert des Knotens.
	 * 
	 * @return Wert des Knotens.
	 */
	public Object getWert() {
		return wert;
	}

	/**
	 * Prft, ob der Knoten einen Vorgngerknoten hat.
	 * 
	 * @return Liefert <code>true</code>, wenn dieser Knoten einen
	 *         Vorgngerknoten hat, sonst <code>false</code>.
	 */
	public boolean hatVorgaenger() {
		if (vorgaenger != null) {
			return true;
		}
		return false;
	}

	/**
	 * Liefert den Elternknoten dieses Knotens.
	 * 
	 * @return ElternKnoten
	 */
	public Knoten getParent() {
		return vorgaenger;
	}

	/**
	 * Bestimmt rekursiv eine Liste der Vorgnger dieses Knotens. Der erste
	 * Knoten ohne Vorgnger befindet sich am Anfang dieser Liste, dieser
	 * aktuelle Knoten befindet sich am Ende der Liste.
	 * 
	 * @return Liste der Vorgnger mit dem aktuellen Knoten am Ende
	 */
	public LinkedList<Knoten> getVorgaenger() {
		final LinkedList<Knoten> re = new LinkedList<Knoten>();
		re.add(this);
		if (hatVorgaenger()) {
			re.addAll(0, vorgaenger.getVorgaenger());
		}
		return re;
	}

	/**
	 * Liefert die Tiefe des Knotens zurck. Wird rekursiv berechnet.
	 * 
	 * @return Tiefe des Knotens.
	 */
	public int getTiefe() {
		int re = 0;

		if (hatVorgaenger()) {
			re++;

			final int tiefe = vorgaenger.getTiefe();
			re = re + tiefe;
		}

		return re;
	}

	/**
	 * Liefert den Pfad, auf dem dieser {@link Knoten} erreichbar ist. Der erste
	 * {@link Knoten} in der Liste entspricht dem Wurzelknoten und verfgt ber
	 * keinen weiteren Vorgnger.
	 * 
	 * @return Pfad vom Wurzelknoten zu diesem {@link Knoten}.
	 */
	public LinkedList<Knoten> getPfad() {
		final LinkedList<Knoten> pfad = new LinkedList<Knoten>();
		pfad.add(this);

		if (hatVorgaenger()) {
			final LinkedList<Knoten> vorgaengerPfad = vorgaenger.getPfad();
			pfad.addAll(0, vorgaengerPfad);
		}

		return pfad;
	}

	/**
	 * Liefert den Wurzelknoten des Baumes.
	 * 
	 * @return Wurzelknoten des Baumes.
	 */
	public Knoten getWurzelKnoten() {
		return getPfad().get(0);
	}

	/**
	 * Bestimmt die Breite der breitesten Ebene.
	 * 
	 * @return Breite der breitesten Ebene.
	 */
	public int bestimmeBreitesteEbene() {
		int breitesteEbene = 0;
		final Knoten wurzelKnoten = getWurzelKnoten();
		final int tiefsteEbene = wurzelKnoten.bestimmeTiefsteEbene();

		for (int i = 0; i <= tiefsteEbene; i++) {
			final LinkedList<Knoten> ebene = wurzelKnoten
					.bestimmeKnotenAufEbene(i);
			if (ebene.size() > breitesteEbene) {
				breitesteEbene = ebene.size();
			}

			final Iterator<Knoten> it = ebene.iterator();
			while (it.hasNext()) {
				final Knoten knoten = it.next();
				Knoten.LOGGER.fine("Knoten auf Ebene " + i + ": "
						+ knoten.getSpaltenBezeichnung());

			}
		}

		return breitesteEbene;
	}

	/**
	 * Bestimmt die Knoten, die sich auf der gleichen Ebene befinden.
	 * 
	 * @param ebene
	 *            Ebene
	 * @return Knoten, die auf der angegebenen ebene liegen.
	 */
	public LinkedList<Knoten> bestimmeKnotenAufEbene(final int ebene) {
		LinkedList<Knoten> knotenListe = null;

		/*
		 * Wurzelknoten bestimmen
		 */
		final Knoten wurzelKnoten = this.getWurzelKnoten();

		/*
		 * Knoten auf gleicher Ebene bestimmen.
		 */
		knotenListe = bestimmtKnotenAufEbeneRekursiv(wurzelKnoten, ebene);

		return knotenListe;
	}

	/**
	 * Bestimmt die Knoten, die sich auf der gleichen Ebene befinden.
	 * DFS-Traversierung.
	 * 
	 * @param knoten
	 *            Hierbei sollte es sich initial um den Wurzelknoten handeln.
	 * @param ebene
	 *            Ebene
	 * @return Liste der Knoten auf der angegebenen Ebene.
	 */
	private LinkedList<Knoten> bestimmtKnotenAufEbeneRekursiv(
			final Knoten knoten, final int ebene) {
		final LinkedList<Knoten> knotenListe = new LinkedList<Knoten>();

		final int tiefeKnoten = knoten.getTiefe();

		if (tiefeKnoten == ebene) {
			knotenListe.add(knoten);
		} else if (tiefeKnoten < ebene) {
			final Iterator<Knoten> it = knoten.getNachfolger().iterator();
			while (it.hasNext()) {
				final Knoten nachfolgeKnoten = it.next();
				knotenListe
						.addAll(nachfolgeKnoten.bestimmtKnotenAufEbeneRekursiv(
								nachfolgeKnoten, ebene));
			}
		}

		return knotenListe;
	}

	/**
	 * Bestimmte die tiefste Ebene im Baum, ausgehend vom Wurzelknoten.
	 * 
	 * @return Liefert die tiefste Ebene im Baum vom Wurzelknoten ausgehend.
	 */
	public int bestimmeTiefsteEbene() {
		final Knoten wurzelKnoten = this.getWurzelKnoten();
		return bestimmeTiefsteEbeneRekursiv(wurzelKnoten);
	}

	/**
	 * Bestimmt rekursiv die tiefste Ebene des Baumes.
	 * 
	 * @param knoten
	 *            Wurzeknoten, von dem ausgehend die tiefste Ebene bestimmt
	 *            werden soll.
	 * @return Liefert die tiefste Ebene innerhalb der jeweiligen
	 *         Rekursionsstufe zurck.
	 */
	public int bestimmeTiefsteEbeneRekursiv(final Knoten knoten) {
		int tiefeKnoten = knoten.getTiefe();

		if (knoten.getNachfolger().size() > 0) {
			final Iterator<Knoten> it = knoten.getNachfolger().iterator();
			while (it.hasNext()) {
				final Knoten nachfolgeKnoten = it.next();
				final int tiefeNachfolger = nachfolgeKnoten
						.bestimmeTiefsteEbeneRekursiv(nachfolgeKnoten);
				if (tiefeNachfolger > tiefeKnoten) {
					tiefeKnoten = tiefeNachfolger;
				}
			}
		}

		return tiefeKnoten;
	}

	/**
	 * Bestimmt die Knoten auf der tiefsten Ebene, welche ber keinen Nachfolger
	 * verfgen.
	 * 
	 * @return Knoten auf der tiefsten Ebene ohne Nachfogler
	 */
	public LinkedList<Knoten> bestimmeKnotenOhneNachfolger() {
		final Knoten wurzelKnoten = this.getWurzelKnoten();
		return bestimmeKnotenOhneNachfolgerRek(wurzelKnoten);
	}

	/**
	 * Bestimmt rekursiv die Knoten auf der tiefsten Ebene, welche ber keinen
	 * Nachfolger verfgen.
	 * 
	 * @param knoten
	 *            Ausgangsknoten, von dem aus der Algorithmus los rennt.
	 * @return Liste von Knoten auf der tiefsten Ebene.
	 */
	public LinkedList<Knoten> bestimmeKnotenOhneNachfolgerRek(
			final Knoten knoten) {
		final LinkedList<Knoten> re = new LinkedList<Knoten>();

		if (knoten.getNachfolger().size() == 0) {
			re.add(knoten);
		} else {
			final Iterator<Knoten> it = knoten.getNachfolger().iterator();
			while (it.hasNext()) {
				final Knoten nachfolgeKnoten = it.next();
				re.addAll(nachfolgeKnoten
						.bestimmeKnotenOhneNachfolgerRek(nachfolgeKnoten));
			}
		}

		return re;
	}

	/**
	 * Liefert die Bezeichnung der Spalte.
	 * 
	 * @return Bezeichnung der Spalte
	 */
	public String getSpaltenBezeichnung() {
		String re = null;

		if (wert != null) {
			if (wert instanceof AttributeGroup) {
				final AttributeGroup atgWert = (AttributeGroup) wert;
				re = atgWert.getName();
			} else if (wert instanceof Attribute) {
				final Attribute att = (Attribute) wert;
				re = att.getName();

				/*
				 * Wenn es sich um ein Array handelt, dann [..] an die
				 * Spaltenbezeichnung anhngen.
				 */
				if (att.isArray()) {
					re = re + " [..]";
				}

			} else if (wert instanceof AttributeListDefinition) {
				final AttributeListDefinition att = (AttributeListDefinition) wert;
				re = att.getName();

				re += " Liste ";

				/*
				 * Wenn es sich um ein Array handelt, dann [..] an die
				 * Spaltenbezeichnung anhngen.
				 */
				// if (att.isArray())
				// {
				// re = re + " [..]";
				// }
			} else if (wert instanceof Data) {
				re = ((Data) wert).getName();
			} else {
				re = "Unbekannt";
			}
		}

		return re;
	}

	/**
	 * Prft, ob es sich bei diesem Knoten um ein Array handelt.
	 * 
	 * @return Liefert <code>true</code>, wenn es sich um ein Array handelt,
	 *         sonst <code>false</code>.
	 */
	protected boolean isArray() {
		boolean re = false;

		if (wert != null) {
			if (wert instanceof Attribute) {
				final Attribute att = (Attribute) wert;

				/*
				 * Wenn es sich um ein Array handelt, dann [..] an die
				 * Spaltenbezeichnung anhngen.
				 */
				if (att.isArray()) {
					re = true;
				}

			}
		}

		return re;
	}

	@Override
	public String toString() {
		String re = null;

		/*
		 * Attributgruppe
		 */
		if (wert instanceof AttributeGroup) {
			final AttributeGroup atgWert = (AttributeGroup) wert;
			re = "Attributgruppe" + atgWert.getName();
		} else if (wert instanceof Attribute) {
			/* Attribut */
			final Attribute att = (Attribute) wert;
			/*
			 * Normales Attribut oder AttributeListDefinition
			 */
			if (att.getType() instanceof AttributeListDefinition) {
				re = "Attribut (ATL): " + att.getName();
			} else {
				re = "Attribut: " + att.getName();
			}
		} else {
			re = "Knoten (" + wert.toString() + ")";
		}

		return re;
	}

	public int getVon() {
		return vonWert;
	}

	public void setVon(final int von) {
		vonWert = von;
	}

	public int getBis() {
		return bisWert;
	}

	public void setBis(final int bis) {
		bisWert = bis;
	}

	public boolean istInBereich(final int pruefWert) {

		if ((pruefWert >= vonWert) && (pruefWert <= bisWert)) {
			return true;
		}
		return false;
	}

	public int getVonX() {
		return vonX;
	}

	public void setVonX(final int vonX) {
		this.vonX = vonX;
	}

	public int getVonY() {
		return vonY;
	}

	public void setVonY(final int vonY) {
		this.vonY = vonY;
	}

	public int getBisX() {
		return bisX;
	}

	public void setBisX(final int bisX) {
		this.bisX = bisX;
	}

	public int getBisY() {
		return bisY;
	}

	public void setBisY(final int bisY) {
		this.bisY = bisY;
	}

	/**
	 * Prft, ob sich eine Zelle im Bereich eines Knotens befindet.
	 * 
	 * @param zeile
	 *            Zeile der Zelle.
	 * @param spalte
	 *            Spalte der Zelle.
	 * @return Liefert <code>true</code>, wenn sich die Zelle im Bereich des
	 *         Knotens befindet, sonst <code>false</code>.
	 */
	public boolean istInBereich(final int zeile, final int spalte) {
		boolean re = false;

		if ((spalte >= vonX) && (spalte <= bisX)) {
			if ((zeile >= vonY) && (zeile <= bisY)) {
				re = true;
			}
		}
		return re;
	}

	public LinkedList<Knoten> getListeAllerKnotenDfs() {
		final Knoten wurzelKnoten = getWurzelKnoten();
		final LinkedList<Knoten> re = getListeAllerKnotenDfsRekursiv(wurzelKnoten);

		return re;
	}

	public LinkedList<Knoten> getListeAllerKnotenDfsRekursiv(final Knoten knoten) {
		final LinkedList<Knoten> re = new LinkedList<Knoten>();

		re.add(knoten);
		final Iterator<Knoten> itNachfolger = knoten.getNachfolger().iterator();
		while (itNachfolger.hasNext()) {
			final Knoten nachfolgeKnoten = itNachfolger.next();
			re.addAll(getListeAllerKnotenDfsRekursiv(nachfolgeKnoten));
		}

		return re;
	}

}
