/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.Iterator;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Bestimmt die Kopfzeilen und legt diese in einer {@link Knoten}-Struktur ab.
 * Der Wurzel{@link Knoten} ist mit
 * {@link ListenDarstellungBestimmeKopfzeilen#getWurzelKnoten()} abfragbar.
 * 
 * @author HofmannJ
 * 
 */
public class ListenDarstellungBestimmeKopfzeilen {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * DatenModell, bestimmt sich aus der Connection.
	 */
	private final DataModel dataModel;

	/**
	 * Attributgruppe.
	 */
	private String atg;

	/**
	 * Speichert den Wurzel{@link Knoten} der Struktur.
	 */
	private Knoten wurzelKnoten;

	/**
	 * Konstruktor, bernimmt die Verbindung zum DaV und holt dann das
	 * DataModel.
	 * 
	 * @param connection
	 *            Verbindung zum DaV.
	 */
	protected ListenDarstellungBestimmeKopfzeilen(
			final ClientDavInterface connection) {
		this.dataModel = connection.getDataModel();
	}

	/**
	 * Setzt die Parameter, aus denen sich die verfgbaren Kopfzeilen bestimmen.
	 * 
	 * @param objektTyp
	 *            Objekttyp.
	 * @param attributGruppe
	 *            AttributGruppe.
	 * @param aspekt
	 *            Aspekt.
	 */
	protected void setDaten(final String objektTyp,
			final String attributGruppe, final String aspekt) {
		atg = attributGruppe;
	}

	/**
	 * Bestimmt die Attribute einer Attributgruppe. <br>
	 * Typ = typ.messQuerschnitt, ATG = atg.anteilLkwFuzzy, ASP =
	 * asp.parameterDefault liefert:<br>
	 * <code>Urlasser             | ALkW           |Minimum|Maximum|</code><br>
	 * <code>Ursache | Veranlasse |Name|t1|t2|t3|t4|       |       |</code>
	 * 
	 */
	protected void bestimmeAttributeEinerAttributGruppe() {
		ListenDarstellungBestimmeKopfzeilen.LOGGER
				.fine("Bestimme die Attribute einer AttributGruppe");

		/*
		 * ATG als WurzelKnoten
		 */
		final AttributeGroup attributeGroup = dataModel.getAttributeGroup(atg);

		wurzelKnoten = new Knoten(attributeGroup);
		final List<Attribute> lAttributes = attributeGroup.getAttributes();
		final Iterator<Attribute> itAtts = lAttributes.iterator();
		while (itAtts.hasNext()) {
			final Attribute attribute = itAtts.next();
			final Knoten subKnoten = new Knoten(attribute);
			wurzelKnoten.addNachfolger(subKnoten);
			bestimmeSubKnotenRekursiv(subKnoten);
		}
	}

	/**
	 * Bestimmt rekursiv alle SubKnoten.
	 * 
	 * @param knoten
	 *            {@link Knoten}, dessen SubKnoten bestimmt werden sollen.
	 */
	private void bestimmeSubKnotenRekursiv(final Knoten knoten) {
		if (knoten.getWert() instanceof Attribute) {
			final Attribute att = (Attribute) knoten.getWert();
			final AttributeType attType = att.getAttributeType();

			if (attType instanceof AttributeListDefinition) {
				final AttributeListDefinition atlDef = (AttributeListDefinition) attType;
				final Iterator<Attribute> itAtts = atlDef.getAttributes()
						.iterator();
				while (itAtts.hasNext()) {
					final Attribute subAtt = itAtts.next();
					final Knoten subKnoten = new Knoten(subAtt);
					knoten.addNachfolger(subKnoten);
					bestimmeSubKnotenRekursiv(subKnoten);
				}
			}
		}
	}

	/**
	 * Liefert den Wurzel{@link Knoten} der Header-Struktur zurck.
	 * 
	 * @return Wurzel{@link Knoten}
	 */
	protected Knoten getWurzelKnoten() {
		return wurzelKnoten;
	}

	/**
	 * Bestimmt REKURSIV die TabellenSpaltenberschriften zu einem Attribut.
	 * Bsp.:
	 * 
	 * Typ = typ.messQuerschnitt, ATG = typ.messQuerschnitt, ASP =
	 * asp.parameterDefault liefert:
	 * <code>| ALkW           |Minimum|Maximum|</code><br>
	 * <code>|Name|t1|t2|t3|t4|       |       |</code>
	 * 
	 * 
	 * 
	 * @param attribut
	 *            Attribut, zu dem die TabellenSpaltenberschriften bestimmt
	 *            werden sollen.
	 * @param tiefe
	 *            Parameter zum feststellen der Rekursionstiefe. Bei Bedarf kann
	 *            ber diesen eine Einschrnkung in der Tiefe der Rekursion
	 *            vorgenommen werden.
	 */

	protected void analyseAtg(final AttributeGroup analyseAtg,
			final String pfad, final int anzahlSpalten) {
		final List<Attribute> att = analyseAtg.getAttributes();

		Object[] spalten;

		for (int i = 0; i < att.size(); i++) {
			String pfad1 = pfad;

			final Attribute att2 = att.get(i);

			// System.out.print("" + att2);

			final AttributeType att3 = att2.getAttributeType();

			if ("typ.attributListenDefinition".equals(att3.getType().getPid())) {

				pfad1 += "2" + att2.getName();

				if (attributWertVorhanden(att3.getPid())) {
					// Attributliste hat
					// Attribut "Wert"

					int j = 0;

					spalten = new Object[1 + anzahlSpalten];

					if (att2.isArray()) {
						spalten[j++] = att2.getName() + "[]";
					} else {
						spalten[j++] = att2.getName();
					}

					for (int k = 0; k < anzahlSpalten; k++) {
						spalten[j++] = "-";
					}
				} else {
					int j = 0;

					spalten = new Object[1 + anzahlSpalten];

					if (att2.isArray()) {
						spalten[j++] = att2.getName() + "[]";
					} else {
						spalten[j++] = att2.getName();
					}

					for (int k = 0; k < anzahlSpalten; k++) {
						spalten[j++] = "";
					}
				}

				rekursivAtl("  ", att3.getPid(), pfad1, anzahlSpalten);
			} else {

				if (!att2.isArray()) {

					int j = 0;

					spalten = new Object[1 + anzahlSpalten];
					spalten[j++] = att2.getName();

					for (int k = 0; k < anzahlSpalten; k++) {
						spalten[j++] = "-";
					}
				}
			}
		}
	}

	protected void rekursivAtl(final String rand, final String pid,
			final String pfad, final int anzahlSpalten) {
		final AttributeListDefinition atl = (AttributeListDefinition) dataModel
				.getObject(pid);

		final List<Attribute> att = atl.getAttributes();

		Object[] spalten;

		for (int i = 0; i < att.size(); i++) {
			String pfad1 = pfad;

			final Attribute att2 = att.get(i);

			final AttributeType att3 = att2.getAttributeType();

			if ("typ.attributListenDefinition".equals(att3.getType().getPid())) {

				pfad1 += ":2" + att2.getName();

				if (attributWertVorhanden(att3.getPid())) {
					// Attributliste hat
					// Attribut "Wert"

					int j = 0;

					spalten = new Object[1 + anzahlSpalten];

					if (att2.isArray()) {
						spalten[j++] = att2.getName() + "[]";
					} else {
						spalten[j++] = att2.getName();
					}

					for (int k = 0; k < anzahlSpalten; k++) {
						spalten[j++] = "-";
					}
				} else {
					int j = 0;

					spalten = new Object[1 + anzahlSpalten];
					spalten[j++] = att2.getName();

					for (int k = 0; k < anzahlSpalten; k++) {
						spalten[j++] = "";
					}
				}

				rekursivAtl(rand + "  ", att3.getPid(), pfad1, anzahlSpalten);
			} else {
				// System.out.print(" [" + att2.getAttributeType() + "]");

				if (att2.isArray()) {
					if (att2.isCountLimited()) {
						// System.out.println("");
						pfad1 += ":2" + att2.getName() + " ["
								+ att2.getAttributeType() + "]";
						System.out.println("Pfad: " + pfad1);
						// System.out.println(" Array mit maximaler Gre (" +
						// att2.getMaxCount() + ")");
					} else {
						// System.out.println("");
						pfad1 += ":3" + att2.getName() + " ["
								+ att2.getAttributeType() + "]";
						System.out.println("Pfad: " + pfad1);
						// System.out.println(" Array mit variabler Gre");
					}
				} else {
					// System.out.println("");

					pfad1 += ":1" + att2.getName() + " ["
							+ att2.getAttributeType() + "]";

					System.out.println("Pfad: " + pfad1);

					// String text = new String(pfad1);

					int j = 0;

					spalten = new Object[1 + anzahlSpalten];
					spalten[j++] = att2.getName();

					for (int k = 0; k < anzahlSpalten; k++) {
						spalten[j++] = "-";
					}
				}
			}
		}
	}

	protected boolean attributWertVorhanden(final String pid) {
		boolean vorhanden = false;

		final AttributeListDefinition atl = (AttributeListDefinition) dataModel
				.getObject(pid);

		final List<Attribute> att = atl.getAttributes();

		if (att.size() > 0) {
			final Attribute att2 = att.get(0);

			if ("Wert".equals(att2.getName())) {
				vorhanden = true;
			}
		}

		return vorhanden;
	}
}
