/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.sys.funclib.debug.Debug;

public class Zelle {
	/**
	 * Wert der Zelle.
	 */
	private Object value;

	private boolean isPlain;

	private boolean isListe;

	private boolean isArray;

	/**
	 * Vorgngerzelle.
	 */
	private Zelle parent;

	/**
	 * Liste der {@link Zelle}n in die Breite.
	 */
	private final LinkedList<Zelle> liste = new LinkedList<Zelle>();

	/**
	 * Liste der {@link Zelle}n in die Tiefe.
	 */
	private final LinkedList<Zelle> array = new LinkedList<Zelle>();

	private int breite = 1;

	private int startX = -1;

	private int startY = -1;

	private int endX = -1;

	private int endY = -1;

	private static final Debug LOGGER = Debug.getLogger();

	public Zelle(final Object value) {
		this.value = value;
	}

	/**
	 * Liefert die Wurzel.
	 * 
	 * @param zelle
	 *            Zelle, von der die Wurzel bestimmt werden soll.
	 * @return Wurzel der Zelle.
	 */
	public Zelle getWurzel(final Zelle zelle) {
		Zelle re = null;

		if (parent == null) {
			re = this;
		} else {
			re = parent.getWurzel(parent);
		}

		return re;
	}

	/**
	 * Liefert die Eltern-{@link Zelle} dieser {@link Zelle}.
	 * 
	 * @return Eltern-{@link Zelle} dieser {@link Zelle} oder <code>null</code>,
	 *         wenn diese {@link Zelle} keine Eltern-{@link Zelle} hat und damit
	 *         die Wurzel ist.
	 */
	public Zelle getParent() {
		return parent;
	}

	/**
	 * Bestimmt die Liste der Vorgngerzellen. Die Letzte Zelle in der Liste ist
	 * die aktuell Zelle.
	 * 
	 * @return Liste der Vorgnger
	 */
	public LinkedList<Zelle> getVorgaenger() {
		final LinkedList<Zelle> re = new LinkedList<Zelle>();
		re.add(this);
		if (getParent() != null) {
			// System.err.println("Aktuell " + ((Data)getWert()).getName() +
			// " Array "
			// + ((Data)getWert()).isArray() + " Gre " +
			// ((((Data)getWert()).isArray()) ?
			// ((Data)getWert()).asArray().getLength() : -1));

			re.addAll(0, getParent().getVorgaenger());
		}
		return re;
	}

	/**
	 * Liefert die Liste der {@link Zelle}n in die Breite.
	 * 
	 * @return Liste der {@link Zelle}n in die Breite
	 */
	public LinkedList<Zelle> getListe() {
		return liste;
	}

	/**
	 * Liefert die Liste der {@link Zelle}n in die Tiefe.
	 * 
	 * @return Liste der {@link Zelle}n in die Tiefe
	 */
	public LinkedList<Zelle> getArray() {
		return array;
	}

	/**
	 * Fgt einen {@link Zelle} zur Liste hinzu, wenn es sich um eine Liste
	 * handelt und die Zelle in der Liste noch nicht vorhanden ist.
	 * 
	 * @param zelle
	 *            Hinzuzufgende {@link Zelle}.
	 */
	public void addToList(final Zelle zelle) {
		if ((isListe) && (!liste.contains(zelle))) {
			liste.add(zelle);
			zelle.parent = this;
		}
	}

	/**
	 * Fgt eine {@link Zelle} zum Array hinzu, falls es sich um ein Array
	 * handelt und diese {@link Zelle} noch nicht im Array vorhanden ist.
	 * 
	 * @param zelle
	 *            Hinzuzufgende Zelle
	 */
	public void addToArray(final Zelle zelle) {
		if ((isArray) && (!array.contains(zelle))) {
			array.add(zelle);
			zelle.parent = this;
		}
	}

	/**
	 * Liefert eine Liste aller Plain-{@link Zelle}n.
	 * 
	 * @return Liste aller Plain-{@link Zelle}n
	 */
	public LinkedList<Zelle> bestimmeAllePlainZellen() {
		final LinkedList<Zelle> listePlain = new LinkedList<Zelle>();

		final Iterator<Zelle> it = bestimmeAlleZellen().iterator();
		while (it.hasNext()) {
			final Zelle zelle = it.next();
			if (zelle.isPlain()) {
				listePlain.add(zelle);
			}
		}

		return listePlain;
	}

	/**
	 * Liefert eine Liste aller Array-{@link Zelle}n zurck.
	 * 
	 * @return Liste aller Array-{@link Zelle}n.
	 */
	public LinkedList<Zelle> bestimmeAlleArrayZellen() {
		final LinkedList<Zelle> listeArrayZellen = new LinkedList<Zelle>();
		final Iterator<Zelle> it = bestimmeAlleZellen().iterator();
		while (it.hasNext()) {
			final Zelle zelle = it.next();
			if (zelle.isArray()) {
				listeArrayZellen.add(zelle);
			}
		}

		return listeArrayZellen;
	}

	/**
	 * Liefert eine Liste aller Listen-{@link Zelle}n zurck.
	 * 
	 * @return Liste aller Listen-{@link Zelle}n
	 */
	public LinkedList<Zelle> bestimmeAlleListenZellen() {
		final LinkedList<Zelle> listeListenZellen = new LinkedList<Zelle>();
		final Iterator<Zelle> it = bestimmeAlleZellen().iterator();
		while (it.hasNext()) {
			final Zelle zelle = it.next();
			if (zelle.isListe()) {
				listeListenZellen.add(zelle);
			}
		}

		return listeListenZellen;
	}

	/**
	 * Bestimmt alle Zellen.
	 * 
	 * @return Liste aller Zellen.
	 */
	public LinkedList<Zelle> bestimmeAlleZellen() {
		final LinkedList<Zelle> re = new LinkedList<Zelle>();

		final Zelle wurzel = getWurzel(this);
		re.add(wurzel);

		re.addAll(bestimmeNachfolgeZellenRekursiv(wurzel));

		return re;
	}

	/**
	 * Sucht in einer Liste von {@link Zelle}n nach denjenigen {@link Zelle}n,
	 * deren StartPosY der angegebenen StartPosY entspricht.
	 * 
	 * @param zellenListe
	 *            Liste von {@link Zelle}n
	 * @param startPosY
	 *            Start Pos Y
	 * @return Liste von Zellen, die mit gleicher StartPosY anfangen
	 */
	public LinkedList<Zelle> bestimmeZellenMitBestimmtenStartPosYAusZellenListe(
			final LinkedList<Zelle> zellenListe, final int startPosY) {
		final LinkedList<Zelle> zellenBestimmterTiefe = new LinkedList<Zelle>();

		final Iterator<Zelle> it = zellenListe.iterator();
		while (it.hasNext()) {
			final Zelle pruefling = it.next();
			final int startPosYPruefling = bestimmeStartPosY(pruefling);
			if (startPosYPruefling == startPosY) {
				zellenBestimmterTiefe.add(pruefling);
			}
		}

		return zellenBestimmterTiefe;
	}

	/**
	 * Bestimmt rekursive alle Nachfolgerzellen einer {@link Zelle}.
	 * 
	 * @param zelle
	 *            {@link Zelle}, deren Nachfolger bestimmt werden sollen.
	 * @return Liste der Nachfolger der angegbenen {@link Zelle}.
	 */
	private LinkedList<Zelle> bestimmeNachfolgeZellenRekursiv(final Zelle zelle) {
		final LinkedList<Zelle> re = new LinkedList<Zelle>();

		if (zelle.isListe()) {
			final Iterator<Zelle> it = zelle.getListe().iterator();
			while (it.hasNext()) {
				final Zelle zelleAusListe = it.next();
				re.add(zelleAusListe);
				re.addAll(bestimmeNachfolgeZellenRekursiv(zelleAusListe));
			}
		} else if (zelle.isArray()) {
			final Iterator<Zelle> it = zelle.getArray().iterator();
			while (it.hasNext()) {
				final Zelle zelleAusArray = it.next();
				re.add(zelleAusArray);
				re.addAll(bestimmeNachfolgeZellenRekursiv(zelleAusArray));
			}
		}

		return re;
	}

	/**
	 * Bestimmt die Ebene einer {@link Zelle}. Dazu werden nur die {@link Zelle}
	 * n in die Tiefe betrachtet.
	 * 
	 * @param zelle
	 *            {@link Zelle}, deren Ebene bestimmt werden soll.
	 * @return Tiefe der Zelle.
	 */
	public int bestimmeStartPosY(final Zelle zelle) {
		int tiefe = 0;
		final Zelle vorgaenger = zelle.getParent();

		if (vorgaenger == null) {
			tiefe = 0;
		} else if (vorgaenger.isArray()) {

			// System.out.println("Vorgnger Array (von: " +
			// zelle.getDarstellung()
			// + ")anz: " + vorgaenger.getArray().size() + " => "
			// + vorgaenger.getWert());

			/*
			 * Algorithmus: Wenn die Vorgnger Zelle ein Array ist, dann die
			 * einzelnen Zellen des Arrays durchlaufen. Die erste Zelle des
			 * Arrays hat die gleiche StartPosY wie das Array. Die Zweite Zelle
			 * des Array hat die StartPosY der ersten Array Zell + die Tiefe der
			 * ersten Array-Zelle + 1, usw... Mit der Tiefe ist die Anzahl der
			 * Plain-Zellen in der Tiefe des Arrays gemeint, denn nur diese sind
			 * sichtbar.
			 */
			final Iterator<Zelle> itArray = vorgaenger.getArray().iterator();
			int startPos = bestimmeStartPosY(vorgaenger);
			while (itArray.hasNext()) {
				final Zelle arrayElement = itArray.next();
				arrayElement.setStartY(startPos);
				if (arrayElement == zelle) {
					break;
				}
				startPos = startPos + bestimmeMaxTiefeInZelle(arrayElement);

			}

			tiefe = zelle.getStartY();

		} else {

			tiefe = tiefe + vorgaenger.bestimmeStartPosY(vorgaenger);
		}

		return tiefe;
	}

	/**
	 * Bestimmt die Tiefe einer Zelle in der Liste dieser Array-Zelle.
	 * 
	 * @param zelle
	 *            Zelle, deren Tiefe bestimmt werden soll.
	 * @return Die Tiefe oder <code>null</code>, wenn es sich nicht um eine
	 *         Array-Zelle handelt.
	 */
	protected int bestimmeTiefeInArray(final Zelle zelle) {
		int re = -1;

		if (isArray()) {
			re = array.indexOf(zelle);
		}

		return re;
	}

	public int bestimmeEndPosYAlt(final Zelle zelle) {
		int re = bestimmeStartPosY(zelle);

		final Zelle cellParent = zelle.getParent();

		if (cellParent == null) {
			re = bestimmeMaxTiefeInZelle(zelle);
		} else if (cellParent.isListe()) {

			re = (re + bestimmeMaxTiefeInZelle(cellParent)) - 1;
			// re = re +
			// bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(parent)
			// - 1;

			if ((re == 0) && (cellParent.getParent() != null)) {

				if (cellParent.getParent().isListe()) {
					re = bestimmeEndPosYAlt(cellParent.getParent()) - 1;

					/*
					 * Falls die obige Variante Scheitert: Listen auf gleicher
					 * Ebene (gleiches StartPosY) bestimmen Davon die Liste mit
					 * der Tiefsten Tiefe bestimmen
					 * 
					 * LinkedList<Zelle> listenZellen =
					 * bestimmeAlleListenZellen(); LinkedList<Zelle>
					 * listenAufGleicherEbene =
					 * bestimmeZellenMitBestimmtenStartPosYAusZellenListe
					 * (listenZellen, bestimmeStartPosY(parent));
					 * System.err.println("Anzahl Listen aus gleicher Ebene: " +
					 * listenAufGleicherEbene.size());
					 * 
					 * int tempTiefe = 0;
					 */

				}
			}
		} else if (cellParent.isArray()) {
			final int idx = cellParent.getArray().indexOf(zelle);

			if ((re == 0) && (cellParent.getParent() != null)
					&& (cellParent.getArray().size() > 0)) {
				re = bestimmeMaxTiefeInZelle(cellParent.getParent());
				// re =
				// bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(parent.getParent());
			} else {
				re = (re + bestimmeMaxTiefeInZelle(cellParent.getArray().get(
						idx))) - 1;
			}

		}

		return re;
	}

	public int bestimmeEndPosY(final Zelle zelle) {
		final Zelle wurzel = getWurzel(this);
		final int maxTiefeInZelle = wurzel
				.bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(wurzel);

		wurzel.setEndY(maxTiefeInZelle);

		final Iterator<Zelle> iterator = wurzel.getListe().iterator();
		while (iterator.hasNext()) {
			bestimmeEndPosYRekursiv(iterator.next(), maxTiefeInZelle);
		}

		return zelle.getStartY();
	}

	public void bestimmeEndPosYRekursiv(final Zelle zelle, final int maxTiefe) {
		zelle.setEndY(maxTiefe);

		/*
		 * Liste
		 */
		if (zelle.isListe()) {
			final Iterator<Zelle> it = zelle.getListe().iterator();
			while (it.hasNext()) {
				bestimmeEndPosYRekursiv(it.next(), maxTiefe);
			}
		}

		/*
		 * Plain
		 */
		if (zelle.isPlain()) {
			// nichts weiter zu tun
			Zelle.LOGGER.finest("Zelle ist Plain");
		}

		/*
		 * Array
		 */
		if (zelle.isArray()) {

			final LinkedList<Zelle> zellen = zelle.getArray();

			// if (array.size() == 0)
			// {
			// bestimmeEndPosYRekursiv(array.get(0), maxTiefe);
			// } else
			if (zellen.size() > 0) {
				final Iterator<Zelle> it = zellen.iterator();

				while (it.hasNext()) {
					final Zelle arreyElement = it.next();
					// bestimmeEndPosYRekursiv(arreyElement, 1);

					int summeTiefenNachfolgerzellen = 0;
					final int idx = zellen.indexOf(arreyElement);
					if (idx < (zellen.size() - 1)) {
						for (int i = idx + 1; i < zellen.size(); i++) {
							// int tiefe =
							// bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(array
							// .get(i));

							final int tiefe = bestimmeMaxTiefeInZelle(zellen
									.get(i));

							// System.out
							// .println("\t=>" + array.get(i).getDarstellung() +
							// tiefe);

							summeTiefenNachfolgerzellen = summeTiefenNachfolgerzellen
									+ tiefe;
						}

						// System.out.println("Arrray Nachfolger: "
						// + summeTiefenNachfolgerzellen);
						bestimmeEndPosYRekursiv(arreyElement, maxTiefe
								- summeTiefenNachfolgerzellen);
					} else if (idx == (zellen.size() - 1)) {
						// System.out.println("Array Max: " + maxTiefe);
						bestimmeEndPosYRekursiv(arreyElement, maxTiefe);
					}
				}
			}

		}

	}

	// public int bestimmeEndPosYAlt3(Zelle zelle)
	// {
	// int re = bestimmeStartPosY(zelle);
	//
	// Zelle parent = zelle.getParent();
	//
	// if (parent != null)
	// {
	// if (parent.isListe())
	// {
	// System.out.println("==>LISTE");
	// int tempMax = 0;
	// Iterator<Zelle> it = bestimmeZellenMitBestimmtenStartPosYAusZellenListe(
	// bestimmeAlleListenZellen(), bestimmeStartPosY(parent)).iterator();
	//
	// while (it.hasNext())
	// {
	// Zelle next = it.next();
	// int maxTiefeInZelle = bestimmeMaxTiefeInZelle(next) - 1;
	// if (maxTiefeInZelle > tempMax)
	// {
	// tempMax = maxTiefeInZelle;
	// re = tempMax;
	// }
	// }
	// } else if (parent.isArray())
	// {
	// System.out.println("==>ARRAY");
	// LinkedList<Zelle> array = parent.getArray();
	//
	// if (array.size() == 1)
	// {
	// // re = bestimmeEndPosY(parent) - 1;
	// re = re + bestimmeMaxTiefeInZelle(parent.getParent()) - 1;
	// } else
	// {
	// int idx = array.indexOf(zelle);
	// if (idx < array.size() - 1)
	// {
	// int startPosNachfolgerInArray = bestimmeStartPosY(array.get(idx++));
	// re = startPosNachfolgerInArray--;
	// } else
	// {
	// re = bestimmeMaxTiefeInZelle(parent) - 1;
	// }
	// }
	// }
	// }
	//
	// return re;
	// }

	// /**
	// * Bestimmt von Einer Zelle EndY.
	// *
	// * @param zelle
	// * Zelle, zu der EndY bestimmt werden soll.
	// * @return Liefert die Position EndY
	// */
	// public int bestimmeEndPosYAlt2(Zelle zelle)
	// {
	// int yStart = bestimmeStartPosY(zelle);
	//
	// int re = yStart;
	//
	// Zelle parent = zelle.getParent();
	//
	// // if (zelle.isArray() || zelle.isListe())
	// // {
	// // if (parent != null)
	// // {
	// // re = yStart +bestimmeEndPosY(parent);
	// // } else
	// // {
	// // re = yStart + bestimmeMaxTiefeInZelle(zelle) - 1;
	// // }
	// if (zelle.isArray())
	// {
	// re = yStart + bestimmeMaxTiefeInZelle(parent) - 1;
	// } else if (zelle.isListe())
	// {
	// re = yStart + bestimmeMaxTiefeInZelle(zelle) - 1;
	// } else if (zelle.isPlain())
	// {
	// if (parent == null)
	// {
	// re = yStart + 0;
	// System.out.println("->NULL");
	// } else if (parent.isListe())
	// {
	// re = bestimmeEndPosY(parent);
	// System.out.println("->LISTE");
	// } else
	// {
	// if (parent.isArray())
	// {
	// LinkedList<Zelle> array = parent.getArray();
	// int idx = array.indexOf(zelle);
	// if (idx < array.size() - 1)
	// {
	// int startPosNachfolgerInArray = bestimmeStartPosY(array.get(idx++));
	// re = startPosNachfolgerInArray--;
	// } else
	// {
	// re = bestimmeMaxTiefeInZelle(parent) - 1;
	// }
	// System.out.println("->Array");
	// }
	// }
	//
	// // int yStart = bestimmeStartPosY(zelle);
	// //
	// // // Bestimmen aller ArraKnoten auf gleicher Ebene
	// //
	// // LinkedList<Zelle> temp = new LinkedList<Zelle>();
	// //
	// // // LinkedList<Zelle> anz =
	// // bestimmeZellenMitBestimmtenStartPosYAusZellenListe(
	// // // bestimmeAlleArrayZellen(), yStart);
	// // //
	// // LinkedList<Zelle> anz =
	// // bestimmeZellenMitBestimmtenStartPosYAusZellenListe(
	// // bestimmeAlleListenZellen(), yStart);
	// // // System.err.println("ANZ: " + anz.size());
	// //
	// // Iterator<Zelle> it = anz.iterator();
	// // while (it.hasNext())
	// // {
	// // Zelle pruefling = it.next();
	// // int maxTiefeInZelle = bestimmeMaxTiefeInZelle(pruefling);
	// // if (maxTiefeInZelle > re)
	// // {
	// // re = maxTiefeInZelle;
	// // }
	// // }
	//
	// }
	//
	// // System.out.println("Anz Arrays: " + alleZellen.size() + " Gleiche
	// Tiefe:
	// // " + listeArrays.size());
	// //
	// //
	// // Iterator<Zelle> it = listeArrays.iterator();
	// // while(it.hasNext())
	// // {
	// // Zelle aZelle = it.next();
	// // int tiefstesElement =
	// // bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(aZelle);
	// //
	// // if(tiefstesElement > re)
	// // {
	// // re = tiefstesElement;
	// // }
	// // }
	// //
	// return re;
	// }

	// /**
	// * Bestimmt von Einer Zelle EndY.
	// *
	// * @param zelle
	// * Zelle, zu der EndY bestimmt werden soll.
	// * @return Liefert die Position EndY
	// */
	// public int bestimmeEndPosYAlt(Zelle zelle)
	// {
	// int re = 0;
	// if (istElementInArray(zelle))
	// {
	//
	// int tiefeInArray = zelle.getParent().getArray().indexOf(zelle);
	//
	// int listSize = zelle.getParent().getArray().size();
	//
	// if (tiefeInArray < zelle.getParent().getArray().size() - 1)
	// {
	// Zelle nachfolgeZelleInArray = zelle.getParent().getArray().get(
	// tiefeInArray + 1);
	// re = nachfolgeZelleInArray.bestimmeStartPosY(nachfolgeZelleInArray) - 1;
	// System.out.println("HIER");
	// } else
	// {
	// System.out.println("DA!");
	// re = tiefeInArray;
	// }
	//
	// } else
	// {
	// Zelle bestimmteVorgaengerListe = zelle.bestimmeVorgaengerListe(zelle);
	// re =
	// bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(bestimmteVorgaengerListe);
	// }
	//
	// return re;
	// }

	/**
	 * Bestimmt die direkte Vorgnger Listen-{@link Zelle}.
	 * 
	 * @param zelle
	 *            {@link Zelle}, deren direkte Vorgnger Listen-{@link Zelle}
	 *            bestimmt werden soll.
	 * @return Vorgnger Listen-{@link Zelle} oder <code>null</code>, wenn keine
	 *         solche existiert.
	 */
	protected Zelle bestimmeVorgaengerListe(final Zelle zelle) {
		Zelle vorgaengerListenZelle = null;

		final Zelle cellParent = zelle.getParent();

		if (cellParent != null) {
			if (cellParent.isListe()) {
				vorgaengerListenZelle = cellParent;
			} else {
				vorgaengerListenZelle = bestimmeVorgaengerListe(cellParent);
			}
		}

		return vorgaengerListenZelle;
	}

	/**
	 * Bestimmt die Maximale Tiefe innerhalb einer Zelle.
	 * 
	 * @param zelle
	 *            Zu prfende Zelle
	 * @return max Tiefe
	 */
	protected int bestimmeMaxTiefeInZelle(final Zelle zelle) {
		int re = 0;

		if (zelle.isArray()) {
			re = zelle.getArray().size();

			int tempMax = 0;
			final Iterator<Zelle> it = zelle.getArray().iterator();
			while (it.hasNext()) {
				final int wert = bestimmeMaxTiefeInZelle(it.next());
				if (wert > tempMax) {
					tempMax = wert;
				}
			}

			if (tempMax > 0) {
				re = (re + tempMax) - 1;
			}
		} else if (zelle.isListe()) {
			int tempMax = 0;

			final Iterator<Zelle> it = zelle.getListe().iterator();
			while (it.hasNext()) {
				final Zelle next = it.next();
				final int bestimmeMaxTiefeInZelle = bestimmeMaxTiefeInZelle(next);
				if (bestimmeMaxTiefeInZelle > tempMax) {
					tempMax = bestimmeMaxTiefeInZelle;
					// System.out.println("TEMPMAX: " + tempMax);
				}
			}

			re = tempMax;
		} else if (zelle.isPlain()) {
			re = 1;
		}

		return re;
	}

	/**
	 * Bestimmt von allen mglichen nachfolgenden Zellen das jeweils tiefste
	 * Array und summiert die Tiefen so auf, das eine Maximaltiefe entsteht. Es
	 * wird die Position des tiefsten Elementes im Array zurckgegeben.
	 * 
	 * @param zelle
	 *            Zelle, von deren nachfolgenden Zellen die Position des
	 *            tiefsten Elementes im tiefsten Array bestimmt werden soll.
	 * @return Die Position des tiefsten Elementes.
	 */
	protected int bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(
			final Zelle zelle) {
		int tiefe = 0;

		if (zelle.isListe()) {
			final Iterator<Zelle> it = zelle.getListe().iterator();
			while (it.hasNext()) {
				final int tempTiefe = bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(it
						.next());
				if (tempTiefe > tiefe) {
					tiefe = tempTiefe;
				}
			}
		} else if (zelle.isArray()) {
			// int tempTiefe = zelle.bestimmeStartPosY(zelle) +
			// zelle.getArray().size()
			// - 1;
			int tempTiefe = zelle.bestimmeStartPosY(zelle)
					+ zelle.getArray().size();
			final Iterator<Zelle> it = zelle.getArray().iterator();
			while (it.hasNext()) {
				final Zelle tiefereZelle = it.next();
				final int nachfolgerTiefe = bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(tiefereZelle);
				if (nachfolgerTiefe > tempTiefe) {
					tempTiefe = nachfolgerTiefe;
				}
			}
			if (tempTiefe > tiefe) {
				tiefe = tempTiefe;
			}
		} else if (zelle.isPlain()) {
			// int tempTiefe = zelle.getStartY();
			final int tempTiefe = 1;
			if (tempTiefe > tiefe) {
				tiefe = tempTiefe;
			}
		}

		return tiefe;
	}

	/**
	 * Liefert die Spalte, in der sich die {@link Zelle} befindet.
	 * 
	 * @param zelle
	 *            {@link Zelle}, deren Spalte berechnet werden soll.
	 * @return Spalte der Zelle.
	 */
	public int getSpalte(final Zelle zelle) {
		int spalte = -1;

		final Zelle vorgaenger = zelle.getParent();
		if (vorgaenger == null) {
			spalte = 0;
		} else {
			if (vorgaenger.isListe()) {
				spalte = vorgaenger.bestimmePositionInListe(this);
				spalte = spalte + getSpalte(vorgaenger);
			} else {
				spalte = vorgaenger.getSpalte(vorgaenger);
			}
		}

		return spalte;
	}

	/**
	 * Bestimmt die Position einer {@link Zelle} in der Liste der Eltern-
	 * {@link Zelle}, wenn diese eine Liste ist.
	 * 
	 * @param zelle
	 *            {@link Zelle}, deren Position in der Liste der Eltern-
	 *            {@link Zelle} bestimmt werden soll.
	 * @return Position in der Liste oder <code>-1</code>, wenn die Eltern-
	 *         {@link Zelle} keine Liste ist oder diese {@link Zelle} sich nicht
	 *         in der Liste der Eltern-{@link Zelle} befindet.
	 */
	protected int bestimmePositionInListe(final Zelle zelle) {
		int re = -1;
		if (isListe()) {
			re = liste.indexOf(zelle);
		}
		return re;
	}

	/**
	 * Setzt die Breite dieser Zelle.
	 * 
	 * @param breite
	 *            Breite der Zelle.
	 */
	public void setBreite(final int breite) {
		this.breite = breite;
	}

	/**
	 * Berechnet die Start-Position X der {@link Zelle} innerhalb der Tabelle.
	 * 
	 * @param zelle
	 *            Zelle, deren Start-Position X bestimmt werden soll.
	 * @return Start-Position der Zelle.
	 */
	public int bestimmeStartPosX(final Zelle zelle) {
		int re = 0;

		if (zelle.getParent() != null) {
			final Zelle cellParent = zelle.getParent();
			if (cellParent.isListe()) {
				int summe = 0;
				final int positionInListe = cellParent
						.bestimmePositionInListe(zelle);
				// System.out.println("POS: " + positionInListe);
				final LinkedList<Zelle> listeVonParent = cellParent.getListe();
				for (int i = 0; i < positionInListe; i++) {
					final int zellenBreite = getBreite(listeVonParent.get(i));
					summe = summe + zellenBreite;
				}

				summe = summe + bestimmeStartPosX(cellParent);

				re = summe;
			} else if (cellParent.isArray()) {
				re = bestimmeStartPosX(cellParent);
			}
		}

		return re;
	}

	/**
	 * Berechnet die End-Position X der {@link Zelle} innerhalb der Tabelle.
	 * 
	 * @param zelle
	 *            {@link Zelle}, deren End-Position X bestimmt werden soll.
	 * @return End-Position der Zelle.
	 */
	public int bestimmeEndPosX(final Zelle zelle) {
		int re = 0;

		re = (bestimmeStartPosX(zelle) + getBreite(zelle)) - 1;

		return re;
	}

	/**
	 * Berechnet die Breite der {@link Zelle}.
	 * 
	 * @param zelle
	 *            {@link Zelle} deren Breite berechnet werden soll.
	 * @return Breite der {@link Zelle}.
	 */
	public int getBreite(final Zelle zelle) {
		int re = -1;

		if (breite > 1) {
			// In diesem Fall wurde die Breite vorhergehend
			// festgelegt
			// - z.B. bei leeren Plain Zellen
			re = breite;
		} else {
			if (zelle.isPlain()) {
				re = 1;
			} else if (zelle.isListe()) {
				int summe = 0;
				final Iterator<Zelle> it = zelle.getListe().iterator();
				while (it.hasNext()) {
					final Zelle zelleAusListe = it.next();
					summe = summe + getBreite(zelleAusListe);
				}
				re = summe;
			} else if (zelle.isArray()) {
				int max = 0;
				final Iterator<Zelle> it = zelle.getArray().iterator();
				while (it.hasNext()) {
					final Zelle zelleAusArray = it.next();
					final int tempWert = zelleAusArray.getBreite(zelleAusArray);
					if (tempWert > max) {
						max = tempWert;
					}
				}
				re = max;
			}
		}

		return re;
	}

	/**
	 * Setzt diese {@link Zelle} auf Plain, d.h. sie ist atomar und beinhaltet
	 * genau einen Wert.
	 * 
	 * @param wert
	 *            <code>true</code>, wenn diese {@link Zelle} Plain sein soll,
	 *            sonst <code>false</code>.
	 */
	public void setPlain(final boolean wert) {
		isPlain = wert;
	}

	/**
	 * Prft, ob die {@link Zelle} Plain ist.
	 * 
	 * @return <b>true</b> wenn die {@link Zelle} Plain ist, sonst
	 *         <code>false</code>.
	 */
	public boolean isPlain() {
		return isPlain;
	}

	/**
	 * Setzt diese {@link Zelle} auf Liste, wenn sie in der Breite weitere
	 * {@link Zelle}n beinhalten soll.
	 * 
	 * @param wert
	 *            <code>true</code>, wenn die Zelle eine Liste ist, sonst
	 *            <code>false</code>.
	 */
	public void setListe(final boolean wert) {
		isListe = wert;
	}

	/**
	 * Prft, ob die {@link Zelle} eine Liste ist.
	 * 
	 * @return <code>true</code> wenn die Zelle eine Liste ist, sonst
	 *         <code>false</code>.
	 */
	public boolean isListe() {
		return isListe;
	}

	/**
	 * Setzt diese {@link Zelle} auf Array, wenn sie in der Tiefe weitere
	 * {@link Zelle}n beinhalten soll.
	 * 
	 * @param wert
	 *            <code>true</code>, wenn diese {@link Zelle} ein Array ist,
	 *            sonst <code>false</code>.
	 */
	public void setArray(final boolean wert) {
		isArray = wert;
	}

	/**
	 * Prft, ob die {@link Zelle} ein Array ist.
	 * 
	 * @return <code>true</code> wenn die Zelle ein Array ist, sonst
	 *         <code>false</code>.
	 */
	public boolean isArray() {
		return isArray;
	}

	/**
	 * Liefert den Wert der {@link Zelle}.
	 * 
	 * @return Wert der {@link Zelle}.
	 */
	public Object getWert() {
		return value;
	}

	/**
	 * Setzt den Wert der Zelle.
	 * 
	 * @param wert
	 *            Wert der Zelle als {@link Object}.
	 */
	protected void setWert(final Object wert) {
		this.value = wert;
	}

	public Object getDarstellung() {
		String re = "unbekannt";
		if (value instanceof Data) {
			final Data data = (Data) value;
			re = data.valueToString();
			if ("undefiniert".equals(re)) {
				Zelle.LOGGER.warning("getDarstellung(): unbekannt");
			}

		} else if (value instanceof String) {
			re = (String) value;
		}

		return re;
	}

	/**
	 * Liefert eine String-Reprsentation dieser Zelle.
	 */
	@Override
	public String toString() {
		String re = "Zelle ";

		if (isPlain()) {
			re = re + "(PLAIN) ";
		} else if (isListe()) {
			re = re + "(LISTE) ";
		} else if (isArray()) {
			re = re + "(ARRAY) ";
		}

		re = re + value.toString() + ": SX: " + getStartX() + " SY: "
				+ getStartY() + " EX: " + getEndX() + " EY: " + getEndY();

		return re;
	}

	/**
	 * Liefert StartX.
	 * 
	 * @return StartX.
	 */
	public int getStartX() {
		return startX;
	}

	/**
	 * Setzt StartX.
	 * 
	 * @param startX
	 */
	public void setStartX(final int startX) {
		this.startX = startX;
	}

	/**
	 * Liefert StartY.
	 * 
	 * @return StartY
	 */
	public int getStartY() {
		return startY;
	}

	/**
	 * Setzt StartY.
	 * 
	 * @param startY
	 */
	public void setStartY(final int startY) {
		this.startY = startY;
	}

	/**
	 * Liefert EndX.
	 * 
	 * @return EndX
	 */
	public int getEndX() {
		return endX;
	}

	/**
	 * Setzt EndX.
	 * 
	 * @param endX
	 */
	public void setEndX(final int endX) {
		this.endX = endX;
	}

	/**
	 * Liefert EndY.
	 * 
	 * @return EndY
	 */
	public int getEndY() {
		return endY;
	}

	/**
	 * Setzt EndY.
	 * 
	 * @param endY
	 *            EndY
	 */
	public void setEndY(final int endY) {
		this.endY = endY;
	}

	/**
	 * Prft, ob sich eine Zelle im Bereich befindet.
	 * 
	 * @param row
	 *            Zeile
	 * @param col
	 *            Spalte
	 * @return Liefert <code>true</code>, wenn sich die Zelle im Bereich
	 *         befindet, sonst <code>false</code>.
	 */
	public boolean istInBereich(final int row, final int col) {
		boolean re = false;

		if ((row >= startY) && (row <= endY)) {
			if ((col >= startX) && (col <= endX)) {
				re = true;
			}
		}

		return re;
	}

}
