/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.login;

import java.util.EventObject;

/**
 * Diese Klasse ist der Event, der vom UrlasserDialogListener verarbeitet wird.
 * 
 * Es sind verschiedene Konstruktoren vorhanden, die Fehler- bzw.
 * ErfolgsmeldungsEvents generieren knnen.
 * 
 * Die Auswertung erfolgt durch den UrlasserDialogListener.
 * 
 * @author HofmannJ
 * 
 */
public class UrlasserDialogEvent extends EventObject {
	/**
	 * UID.
	 */
	private static final long serialVersionUID = -1919361584370073161L;

	/**
	 * Aufzhlungstyp fr die Fehler beim Aufbau der Datenverteilerverbindung.<br>
	 */
	public enum FehlerDavVerbindung {
		/**
		 * Fehler bei der Kommunikation mit dem Datenverteiler aufgetreten.<br>
		 * 
		 * siehe ClientDavConnection CommunicationError.<br>
		 */
		KOMMUNIKATIONS_FEHLER,
		/**
		 * Ausnahme bei der Kommunikation mit dem Datenverteiler aufgetreten.<br>
		 * 
		 * siehe ClientDavConnection ConnectionException.<br>
		 */
		KOMMUNIKATIONS_AUSNAHME,
		/**
		 * Benutzeranmeldung am Datenverteiler ist fehlgeschlagen.<br>
		 * 
		 * siehe ClientDavConnection InconsistentLoginException.<br>
		 */
		BENUTZERANMELDUNG_FEHLGESCHLAGEN,
	}

	/**
	 * Aufzhlungstyp fr die Fehler beim Versand der Daten zum Datenverteiler.<br>
	 */
	public enum FehlerDatenVersand {
		/**
		 * Fehler bei der Kommunikation mit dem Datenverteiler aufgetreten.<br>
		 * 
		 * siehe ClientDavConnection CommunicationError.<br>
		 */
		KOMMUNIKATIONS_FEHLER,
		/**
		 * Ausnahme bei der Kommunikation mit dem Datenverteiler aufgetreten.<br>
		 * 
		 * siehe ClientDavConnection ConnectionException.<br>
		 */
		KOMMUNIKATIONS_AUSNAHME,
		/**
		 * Keine Berechtigung fr den Versand der Daten an den Datenverteiler
		 * (fehlende Zugriffsrechte Datenverteiler).<br>
		 * 
		 * Implementierungsinformation:<br>
		 * Diese Information wird vom Datenverteiler ber das
		 * ClientSenderInterface in der Methode dataRequest mit dem Status
		 * (state) STOP_SENDING_NO_RIGHTS mitgeteilt und wird auch gemeldet,
		 * wenn keine Sendesteuerung gewnscht wurde.<br>
		 */
		KEINE_BERECHTIGUNG,
	}

	/**
	 * Konstruktor fr einen UrlasserDialogEvent. Er wird benutzt wenn angezeigt
	 * werden soll, dass die Verbindung zum Datenverteiler fr den eingegebenen
	 * Benutzer aufgebaut wurde.
	 * 
	 * @param source
	 *            Objekt, dass den Event ausgelst hat.
	 * @param benutzer
	 *            Benutzer, fr den die Verbindung aufgebaut wurde.
	 */
	public UrlasserDialogEvent(final Object source, final String benutzer) {
		super(source);
	}

	/**
	 * Konstruktor fr einen UrlasserDialogEvent. Er wird benutzt wenn angezeigt
	 * werden soll, dass der Verbindungsaufbau zum Datenverteiler fr den
	 * eingegebenen Benutzer fehlgeschlagen ist.
	 * 
	 * @param source
	 *            Objekt, dass den Event ausgelst hat.
	 * @param benutzer
	 *            Benutzer, fr den die DaV-Verbindung aufgebaut werden sollte.
	 * @param fehlerDavVerbindung
	 *            Fehler, der beim DaV-Verbindungsaufbau aufgetreten ist.
	 */
	public UrlasserDialogEvent(final Object source, final String benutzer,
			final FehlerDavVerbindung fehlerDavVerbindung) {
		super(source);
	}

	/**
	 * Konstruktor fr den UrlasserDialogEvent. Er wird benutzt wenn angezeigt
	 * werden soll, dass alle Daten erfolgreich zum Datenverteiler gesendet
	 * wurden.
	 * 
	 * @param source
	 *            Objekt, dass den Event ausgelst hat.
	 * @param benutzer
	 *            Benutzer, fr den die Daten erfolgreich versand wurden.
	 * @param datenversand
	 *            Switch der anzeigt, dass die Daten erfolgreich versand wurden.
	 */
	public UrlasserDialogEvent(final Object source, final String benutzer,
			final boolean datenversand) {
		super(source);
	}

	/**
	 * Konstruktor fr den UrlasserDialogEvent Er wird benutzt wenn angezeigt
	 * werden soll, dass beim Versenden der Daten zum DaV ein Fehler aufgetreten
	 * ist.
	 * 
	 * @param source
	 *            Objekt, dass den Event ausgelst hat.
	 * @param fehlerDatenVersand
	 *            Fehler, der beim Datenversand aufgetreten ist.
	 * @param datenversand
	 *            Switch der anzeigt, dass die Daten erfolgreich versand wurden.
	 */
	public UrlasserDialogEvent(final Object source,
			final FehlerDatenVersand fehlerDatenVersand,
			final boolean datenversand) {
		super(source);
	}

}
