/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort;

import java.util.List;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.EnumTmcRichtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.dav.daf.main.ClientDavConnection;

public class FilterAsbOrtsangabeWorking implements Cloneable {

	/** Verbindung zum DaV. */
	private final ClientDavConnection davConnection;

	/** PID der selektierten Strae. */
	private String pidStrasse;

	/** Anfangsknoten der ASB-Stationierung. */
	private String anfangsKnoten;

	/**
	 * TmcRichtung des Anfangsknotens auf dem jeweiligen ueren Straensegment.
	 */
	private EnumTmcRichtung tmcRichtungAnfangsKnoten;

	/** Endknoten der ASB-Stationierung. */
	private String endKnoten;

	/** TmcRichtung des Endknotens auf dem jeweiligen ueren Straensegment. */
	private EnumTmcRichtung tmcRichtungEndKnoten;

	/** Stationierung. */
	private long stationierung = -1;

	/** {@link AsbOrtsangabe}. */
	private AsbOrtsangabe asbOrtsangabe;

	/**
	 * Konstruktor. bernimmt die Verbindung zum DaV und fhrt die
	 * Initialisierung von {@link AsbOrtsangabe} durch.
	 * 
	 * @param connection
	 *            DaV-Verbindung
	 */
	public FilterAsbOrtsangabeWorking(final ClientDavConnection connection) {
		this.davConnection = connection;
		if (davConnection != null) {
			init();
		}
	}

	/**
	 * Fhrt die Initialisierung durch.
	 */
	private void init() {
		asbOrtsangabe = new AsbOrtsangabe(davConnection);
	}

	/**
	 * Liefert die Listen der auswhlbaren {@link Strasse}n zurck.
	 * 
	 * @return Liste der auswhlbaren {@link Strasse}n
	 */
	public List<Strasse> getStrassen() {
		final List<Strasse> strassen = asbOrtsangabe.getStrassen();
		return strassen;
	}

	/**
	 * Selektiert die gewhlte {@link Strasse} an Hand ihrerer PID. Fhrt die
	 * Initialisierung der {@link AsbOrtsangabe} an Hand der Pid der Strae
	 * durch.
	 * 
	 * @param pid
	 *            PID der zu selektierenden {@link Strasse}
	 */
	public void setSelektierteStrasse(final String pid) {
		this.pidStrasse = pid;
		if (asbOrtsangabe != null) {
			asbOrtsangabe.initialisiereStrukturNachStrasse(pidStrasse);
		}
	}

	/**
	 * Liefert die {@link AsbOrtsangabe}, mit der sich die bentigten Knoten
	 * bestimmen lassen und die Stationierung berprfen lt.
	 * 
	 * @return {@link AsbOrtsangabe}
	 */
	public AsbOrtsangabe getAsbOrtsangabe() {
		return asbOrtsangabe;
	}

	/**
	 * Liefert die PID der selektierten {@link Strasse}.
	 * 
	 * @return PID der selektierten {@link Strasse}
	 */
	public String getPidSelektierteStrasse() {
		return pidStrasse;
	}

	/**
	 * Liefert den AngangsKnoten.
	 * 
	 * @return Anfangsknoten.
	 */
	public String getAnfangsKnoten() {
		return this.anfangsKnoten;
	}

	/**
	 * Setzt den Anfangsknoten.
	 * 
	 * @param anfangsKnoten
	 *            Anfangsknoten.
	 */
	public void setAnfangsKnoten(final String anfangsKnoten) {
		this.anfangsKnoten = anfangsKnoten;
	}

	/**
	 * Liefert die Tmc-Richtung des AnfangsKnotens.
	 * 
	 * @return Tmc-Richtung des AnfangsKnotens
	 */
	public EnumTmcRichtung getAnfangsKnotenTmcRichtung() {
		return tmcRichtungAnfangsKnoten;
	}

	/**
	 * Setzt die Tmc-Richtung des AnfangsKnotens.
	 * 
	 * @param richtung
	 *            Tmc-Richtung des AnfangsKnotens
	 */
	public void setAnfangsKnotenTmcRichtung(final EnumTmcRichtung richtung) {
		this.tmcRichtungAnfangsKnoten = richtung;
	}

	/**
	 * Liefert den Endknoten.
	 * 
	 * @return Endknoten
	 */
	public String getEndKnoten() {
		return endKnoten;
	}

	/**
	 * Setzt den Endknoten.
	 * 
	 * @param endKnoten
	 */
	public void setEndKnoten(final String endKnoten) {
		this.endKnoten = endKnoten;
	}

	/**
	 * Liefert die Tmc-Richtung des EndKnotens.
	 * 
	 * @return Tmc-Richtung des EndKnotens
	 */
	public EnumTmcRichtung getEndKnotenTmcRichtung() {
		return tmcRichtungEndKnoten;
	}

	/**
	 * Setzt die Tmc-Richtung des EndKnotens.
	 * 
	 * @param richtung
	 *            Tmc-Richtung des EndKnotens
	 */
	public void setEndKnotenTmcRichtung(final EnumTmcRichtung richtung) {
		this.tmcRichtungEndKnoten = richtung;
	}

	/**
	 * Setzt die Stationierung.
	 * 
	 * @param stationierungswert
	 */
	public void setStationierung(final long stationierungswert) {
		this.stationierung = stationierungswert;
	}

	/**
	 * Liefert die Stationierung zurck.
	 * 
	 * @return Stationierung
	 */
	public long getStationierung() {
		return stationierung;
	}

	@Override
	public String toString() {
		String re = "===" + getClass().getName() + "===\n";
		re = re + "Strasse (PID):    " + pidStrasse + "\n";
		re = re + "VON (PID):        " + anfangsKnoten + "\n";
		re = re + "VON TmcRichtung:  " + tmcRichtungAnfangsKnoten + "\n";
		re = re + "NACH (PID):       " + endKnoten + "\n";
		re = re + "Nach TmcRichtung: " + tmcRichtungEndKnoten + "\n";
		re = re + "Stationierung:    " + stationierung + "\n";

		for (int i = 0; i < (6 + getClass().getName().length()); i++) {
			re = re + "=";
		}

		re = re + "\n";
		return re;
	}

	@Override
	public Object clone() {
		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new InternalError();
		}
		final FilterAsbOrtsangabeWorking re = new FilterAsbOrtsangabeWorking(
				davConnection);
		re.pidStrasse = pidStrasse;
		re.anfangsKnoten = anfangsKnoten;
		re.tmcRichtungAnfangsKnoten = tmcRichtungAnfangsKnoten;
		re.endKnoten = endKnoten;
		re.tmcRichtungEndKnoten = tmcRichtungEndKnoten;
		re.stationierung = stationierung;
		re.asbOrtsangabe = asbOrtsangabe;
		return re;
	}

}
