/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.KonfigurationNeu;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenKnoten;
import de.bsvrz.dav.daf.main.ClientDavConnection;

/**
 * Filter fr den KnotenOffset gem TBuV-73 (4.ter Dialog).
 * 
 * @author HofmannJ
 * 
 */
public class FilterKnoten implements Cloneable {

	/**
	 * Verbindung zum DaV.
	 */
	private final ClientDavConnection davConnection;

	/**
	 * Speichert die Pid der selektierten Strae.
	 */
	private String pidStrasse;

	/**
	 * Speichert die Pid des selektierten Knotens.
	 */
	private String pidKnoten;

	/**
	 * Speichert die eingegebenen Kilometer als float.
	 */
	private float km = -1.0f;
	// Sans 20091124

	/**
	 * Speichert die Richtung.
	 */
	private Richtung richtung = Richtung.positiv;

	/**
	 * Liefert die Konfiguration.
	 */
	private final KonfigurationNeu konf;

	/**
	 * Konstruktor, als Parameter muss eine Verbindung zum DaV bergeben werden.
	 * 
	 * @param connection
	 *            Verbindung zum DaV
	 */
	public FilterKnoten(final ClientDavConnection connection) {
		this.davConnection = connection;
		konf = KonfigurationNeu.getInstanz();
		konf.bestimmeObjekte(davConnection);
	}

	/**
	 * Liefert eine Liste aller {@link Strasse}en zurck.
	 * 
	 * @return Liste aller {@link Strasse}en
	 */
	public LinkedList<Strasse> getStrassen() {
		final LinkedList<Strasse> strassen = new LinkedList<Strasse>();

		final TreeMap<String, Strasse> tmSortiert = new TreeMap<String, Strasse>();
		tmSortiert.putAll(konf.getStrassen());

		final Iterator<Entry<String, Strasse>> it = tmSortiert.entrySet()
				.iterator();
		while (it.hasNext()) {
			final Entry<String, Strasse> entry = it.next();

			if (!"null".equals(entry.getValue().getName())) {
				strassen.add(entry.getValue());
			}

		}
		return strassen;
	}

	/**
	 * Liefert die Liste der gefilterten Knoten zugehrig zur jeweils
	 * ausgewhlten {@link Strasse}. Gleichzeitig wird die PID der ausgewhlten
	 * {@link Strasse} gesetzt.
	 * 
	 * @param pidSelektierteStrasse
	 *            PID der ausgewhlten {@link Strasse}.
	 * 
	 * @return Liste aller {@link StrassenKnoten}, die zu einer {@link Strasse}
	 *         gehren.
	 */
	public List<StrassenKnoten> getKnoten(final String pidSelektierteStrasse) {
		this.pidStrasse = pidSelektierteStrasse;

		konf.filtereNachStrasse(pidSelektierteStrasse);
		final Map<String, StrassenKnoten> hmStrasseKnotenGefiltert = konf
				.getStrasseKnotenGefiltert();

		final Map<String, StrassenKnoten> tmStrassenKnotenSortiert = new TreeMap<String, StrassenKnoten>();
		tmStrassenKnotenSortiert.putAll(hmStrasseKnotenGefiltert);

		final LinkedList<StrassenKnoten> re = new LinkedList<StrassenKnoten>();
		final Iterator<Entry<String, StrassenKnoten>> it = tmStrassenKnotenSortiert
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, StrassenKnoten> entry = it.next();
			re.add(entry.getValue());
		}

		return re;
	}

	/**
	 * liefert die Pid der selektierten Strae.
	 * 
	 * @return Pid der selektierten Strae
	 */
	public String getPidStrasse() {
		return pidStrasse;
	}

	/**
	 * Setzt die selektierte Pid der Strasse aus dem Dialog, sofern eine Strae
	 * mit der angegebenen Pid existiert.
	 * 
	 * @param pidStrasse
	 *            Zu setzende PID der Strae
	 */
	public void setPidStrasse(final String pidStrasse) {
		this.pidStrasse = pidStrasse;
	}

	/**
	 * Liefert die Pid des selektierten Knotens.
	 * 
	 * @return Pid des selektierten Knotens
	 */
	public String getPidKnoten() {
		return pidKnoten;
	}

	/**
	 * Setzt die selektierte Pid des StraenKnotens aus dem Dialog, sofern ein
	 * StraenKnoten mit der angegebenen Pid existiert.
	 * 
	 * @param pidKnoten
	 *            Zu setzende Pid
	 */
	public void setPidKnoten(final String pidKnoten) {
		this.pidKnoten = pidKnoten;
	}

	/**
	 * Setzt die Kilometer als Offset.
	 * 
	 * @param wert
	 *            Kilometer-Offset
	 */
	public void setKm(final float wert) {
		km = wert;
	}

	/**
	 * Liefert die Kilometer als Offset.
	 * 
	 * @return Kilometer-Offset
	 */
	public float getKm() {
		return km;
	}

	/**
	 * Liefert die Richtung.
	 * 
	 * @return Richtung
	 */
	public Richtung getRichtung() {
		return richtung;
	}

	/**
	 * Setzt die Richtung.
	 * 
	 * @param richtung
	 *            Richtung
	 */
	public void setRichtung(final Richtung richtung) {
		this.richtung = richtung;
	}

	// Ende der Methoden fr die Vorinitialisierung
	// ////

	@Override
	public String toString() {
		String re = "";
		re = re + "===" + getClass().getName() + "===\n";
		re = re + "PID Strae: " + pidStrasse + "\n";

		if (pidKnoten != null) {
			re = re + "PID Knoten: " + pidKnoten + "\n";
		}

		re = re + "Kilometer:  " + km + "\n";
		re = re + "Richtung:   ";

		switch (richtung) {
		case negativ:
			re = re + "negativ";
			break;
		case positiv:
			re = re + "positiv";
			break;
		default:
			break;
		}

		re = re + "\n";

		for (int i = 0; i < (6 + getClass().getName().length()); i++) {
			re = re + "=";
		}

		re = re + "\n";

		return re;
	}

	/**
	 * Enumerator fr die Richtung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum Richtung {
		/**
		 * Positive Richtung.
		 */
		positiv,

		/**
		 * Negative Richtung.
		 */
		negativ
	}

	@Override
	public Object clone() {

		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new InternalError();
		}

		final FilterKnoten re = new FilterKnoten(davConnection);
		re.pidStrasse = pidStrasse;
		re.pidKnoten = pidKnoten;
		re.km = km;
		re.richtung = richtung;
		return re;
	}

}
