/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse realisiet ein inneres Straenteilsegment.
 * 
 * @author Dambach Werke GmbH
 */
public class InneresStrassenSegment extends StrassenSegment {
	/**
	 * zum Straensegment gehrendes Vorgngersegment.
	 */
	private AeusseresStrassenSegment vonStrassenSegment;

	/**
	 * zum Straensegment gehrendes Nachfolgersegment.
	 */
	private AeusseresStrassenSegment nachStrassenSegment;

	/**
	 * Konstruktor der Klasse.
	 * 
	 * @param dataModel
	 *            Datenmodell
	 * @param objekt
	 *            Objekt vom Typ "typ.inneresStraenSegment"
	 */
	public InneresStrassenSegment(final DataModel dataModel,
			final SystemObject objekt) {
		super(dataModel, objekt);

		final AttributeGroup atg = dataModel
				.getAttributeGroup("atg.inneresStraenSegment");

		if (atg != null) {
			final Data confData = objekt.getConfigurationData(atg);

			if (confData != null) {
				final SystemObject s1 = confData.getReferenceValue(
						"nachStraenSegment").getSystemObject();
				if (s1 == null) {
					nachStrassenSegment = null;
				} else {
					nachStrassenSegment = KonfigurationNeu.getInstanz()
							.getAeusseresStrassenSegment(s1.getPid());
				}

				final SystemObject s2 = confData.getReferenceValue(
						"vonStraenSegment").getSystemObject();
				if (s2 == null) {
					vonStrassenSegment = null;
				} else {
					vonStrassenSegment = KonfigurationNeu.getInstanz()
							.getAeusseresStrassenSegment(s2.getPid());
				}
			}
		}
	}

	/**
	 * @return liefert die Variable nachStraenSegment zurck
	 */
	public AeusseresStrassenSegment getNachStrassenSegment() {
		return nachStrassenSegment;
	}

	/**
	 * @return liefert die Variable vonStraenSegmemt zurck
	 */
	public AeusseresStrassenSegment getVonStrassenSegmemt() {
		return vonStrassenSegment;
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um ein inneres
	 * Straensegment handelt.
	 * 
	 * @return bei innerem Straensegment: true, sonst: false
	 */
	@Override
	public boolean isInneresStrassenSegment() {
		return true;
	}

	/**
	 * Methode prft das Straensegmenten in einem Autobahnkreuz oder
	 * Autobahndreieck liegt.
	 * 
	 * @return Autobahnkreuz, Autobahndreick: true, sonst false
	 */
	@Override
	public boolean beginntInAutobahnKreuzOderDreieck() {
		// Ein inneres Straensegment hat keinen direkten Bezug zum
		// Straenknoten, daher erfolgt der
		// Zugriff ber die durch das innere Straensegment verbundenen ueren
		// Straenknoten

		if (nachStrassenSegment != null) {
			return nachStrassenSegment.beginntInAutobahnKreuzOderDreieck();
		}

		if (vonStrassenSegment != null) {
			return vonStrassenSegment.endetInAutobahnKreuzOderDreieck();
		}

		return false;
	}

	/**
	 * Methode prft das Straensegmenten in einem Autobahnkreuz oder
	 * Autobahndreieck endet.
	 * 
	 * @return Autobahnkreuz, Autobahndreick: true, sonst false
	 */
	@Override
	public boolean endetInAutobahnKreuzOderDreieck() {
		// Da ein inneres Straensegment innerhalb eines Straenknotens liegt,
		// braucht nicht zwischen
		// beginnt und endet unterschieden werden.

		return beginntInAutobahnKreuzOderDreieck();
	}

	/**
	 * Methode prft ob der Straenknoten in dem das Straensegment liegt ein
	 * Autobahnanfang ist.
	 * 
	 * @return Autobahnanfang: true, sonst false
	 */
	@Override
	public boolean beginntAmAutobahnAnfang() {
		// Ein inneres Straensegment hat keinen direkten Bezug zum
		// Straenknoten, daher erfolgt der
		// Zugriff ber die durch das innere Straensegment verbundenen ueren
		// Straenknoten

		if (nachStrassenSegment != null) {
			return nachStrassenSegment.beginntAmAutobahnAnfang();
		}

		return false;
	}

	/**
	 * Methode prft ob der Straenknoten in dem das Straensegment liegt ein
	 * Autobahnende ist.
	 * 
	 * @return Autobahnende true, sonst false
	 */
	@Override
	public boolean endetAmAutobahnEnde() {
		// Ein inneres Straensegment hat keinen direkten Bezug zum
		// Straenknoten, daher erfolgt der
		// Zugriff ber die durch das innere Straensegment verbundenen ueren
		// Straenknoten

		if (vonStrassenSegment != null) {
			return vonStrassenSegment.endetAmAutobahnEnde();
		}

		return false;
	}

	@Override
	public String toString() {
		String re = "";
		re = re + getSystemObject().getName() + " (PID: "
				+ getSystemObject().getPid() + ")";

		return re;
	}

	public String toStringLang() {
		String re = "";
		re = re + "innerseStraenSegment: " + getSystemObject().getPid() + "\n";

		re = re + "Von StraenSegment:    ";
		if (vonStrassenSegment != null) {
			re = re + vonStrassenSegment.getPid() + "\n";
		} else {
			re = re + "null\n";
		}

		re = re + "Nach Straensegment:   ";
		if (nachStrassenSegment != null) {
			re = re + nachStrassenSegment.getPid() + "\n";
		} else {
			re = re + "null\n";
		}

		return re;
	}

}
