/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.Iterator;
import java.util.LinkedList;

import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Klasse zur Reprsentation eins XML-Filterelements.
 * 
 * @author HofmannJ
 * 
 */
public class ElementFilter implements Comparable<ElementFilter>, Cloneable {
	/**
	 * ID des Elementes.
	 */
	private int id = -1;

	/**
	 * Bezeichnung.
	 */
	private String bezeichnung;

	/**
	 * Das Element selbst.
	 */
	private Element xmlElement;

	/**
	 * Liste der regulren Ausdrcke des Elementes.
	 */
	private LinkedList<ElementRegEx> regEx = new LinkedList<ElementRegEx>();

	/**
	 * Konstuktor, initialisiert einen {@link ElementFilter} mit ID und
	 * Bezeichnung.
	 * 
	 * @param id
	 *            ID des Elementes
	 * @param bezeichnung
	 *            Bezeichnung
	 */
	public ElementFilter(final int id, final String bezeichnung) {
		initialisiere(id, bezeichnung);
	}

	/**
	 * Konstruktor, initialisiert einen {@link ElementFilter} an Hand des JDOM-
	 * {@link Element}es.
	 * 
	 * @param element
	 */
	public ElementFilter(final Element element) {
		initialisiere(element);
	}

	/**
	 * Fhrt die Initialisierung durch.
	 * 
	 * @param initialId
	 *            ID zur Initialisierung
	 * @param initialBezeichnung
	 *            Bezeichnung zur Initialisierung
	 */
	public void initialisiere(final int initialId,
			final String initialBezeichnung) {
		this.id = initialId;
		this.bezeichnung = initialBezeichnung;
		xmlElement = new Element(XmlFilterVerwaltung.XML_ELEMENT_FILTER);
		final Attribute attId = new Attribute(XmlFilterVerwaltung.XML_ATT_ID,
				"" + initialId);
		xmlElement.setAttribute(attId);
		final Attribute attBez = new Attribute(
				XmlFilterVerwaltung.XML_ATT_BEZEICHNUNG, initialBezeichnung);
		xmlElement.setAttribute(attBez);
	}

	/**
	 * Initialisiert das Element an Hand eines JDOM-{@link Element}es.
	 * 
	 * @param element
	 *            JDOM-{@link Element}
	 */
	public void initialisiere(final Element element) {
		if (element.getName().equals(XmlFilterVerwaltung.XML_ELEMENT_FILTER)) {
			xmlElement = element;

			/*
			 * ID Holen
			 */
			try {
				this.id = Integer.valueOf(xmlElement
						.getAttributeValue(XmlFilterVerwaltung.XML_ATT_ID));
			} catch (final NumberFormatException e) {
				e.printStackTrace();
			}

			/*
			 * Bezeichnung Holen
			 */
			bezeichnung = xmlElement
					.getAttributeValue(XmlFilterVerwaltung.XML_ATT_BEZEICHNUNG);

			final Iterator<Element> it = xmlElement.getChildren(
					XmlFilterVerwaltung.XML_ELEMENT_AUSDRUCK).iterator();
			while (it.hasNext()) {
				final Element ele = it.next();
				final ElementRegEx eleRegEx = new ElementRegEx(ele);
				regEx.add(eleRegEx);
			}

		}
	}

	/**
	 * Liefert diesen {@link ElementFilter} als JDOM-{@link Element} zurck.
	 * 
	 * @return {@link ElementFilter} als JDOM-{@link Element}
	 */
	public Element getElement() {
		/*
		 * Initialisieren
		 */
		initialisiere(id, bezeichnung);

		/*
		 * Liste von RegEx
		 */
		if (regEx != null) {
			xmlElement.removeChildren(XmlFilterVerwaltung.XML_ELEMENT_AUSDRUCK);

			if (regEx.size() > 0) {
				final Iterator<ElementRegEx> it = regEx.iterator();
				while (it.hasNext()) {
					final ElementRegEx elementRegEx = it.next();
					xmlElement.addContent(elementRegEx.getElement());
				}
			}

		}

		return xmlElement;
	}

	/**
	 * Liefert die ID dieses Elementes.
	 * 
	 * @return ID
	 */
	public int getId() {
		return id;
	}

	/**
	 * Setzt die ID dieses Elementes.
	 * 
	 * @param id
	 *            ID
	 */
	public void setId(final int id) {
		this.id = id;
	}

	/**
	 * Liefert die Bezeichnung dieses Elementes.
	 * 
	 * @return Bezeichnung
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * Setzt die Bezeichnung dieses Elementes.
	 * 
	 * @param bezeichnung
	 *            Zu setzende Bezeichnung
	 */
	public void setBezeichnung(final String bezeichnung) {
		this.bezeichnung = bezeichnung;
	}

	/**
	 * Fgt einen regulren Ausdruck zur Liste der regulren Ausdrcke hinzu.
	 * Existiert ein regulrer Ausdruck mit der gleichen ID, so wird dieser
	 * gelscht und durch den neuen ersetzt.
	 * 
	 * @param newRegEx
	 *            Hinzuzufgender regulrer Ausdruck
	 */
	public void addRegEx(final ElementRegEx newRegEx) {

		// Liste zu lschender Elemente
		final LinkedList<ElementRegEx> loeschListe = new LinkedList<ElementRegEx>();

		/**
		 * Prfen, welche Elemente gelscht werden
		 */
		final Iterator<ElementRegEx> itLoeschElementSuchen = regEx.iterator();
		while (itLoeschElementSuchen.hasNext()) {
			final ElementRegEx pruefElement = itLoeschElementSuchen.next();
			if (pruefElement.getId() == newRegEx.getId()) {
				loeschListe.add(pruefElement);
			}
		}

		/*
		 * Lschen
		 */
		final Iterator<ElementRegEx> itLoeschen = loeschListe.iterator();
		while (itLoeschen.hasNext()) {
			final ElementRegEx loeschElement = itLoeschen.next();
			regEx.remove(loeschElement);
		}

		/*
		 * Hinzufgen
		 */
		regEx.add(newRegEx);

	}

	/**
	 * Liefert die Liste der regulren Ausdrcke dieses Elementes.
	 * 
	 * @return Liste der regulren Ausdrcke.
	 */
	public LinkedList<ElementRegEx> getRegex() {
		return regEx;
	}

	/**
	 * Ersetzt die Liste von regulren Ausdrcken mit einer neuen Liste von
	 * regulren Ausdrcken.
	 * 
	 * @param regExes
	 *            Ersatzliste von regulren Ausdrcken.
	 */
	public void setRegEx(final LinkedList<ElementRegEx> regExes) {
		this.regEx = regExes;
	}

	/**
	 * Bestimmt die nchste, freie ID fr Elemente vom Typ {@link ElementRegEx}.
	 * 
	 * @return ID
	 */
	public int bestimmeNaechsteFreieIdRegEx() {
		int re = -1;

		final Iterator<ElementRegEx> it = regEx.iterator();
		while (it.hasNext()) {
			final ElementRegEx regExElement = it.next();
			if (regExElement.getId() > re) {
				re = regExElement.getId();
			}
		}

		re = re + 1;

		return re;
	}

	@Override
	public ElementFilter clone() {
		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new IllegalStateException(e);
		}
		final ElementFilter elementFilterClone = new ElementFilter(id,
				bezeichnung);

		final Iterator<ElementRegEx> it = regEx.iterator();
		while (it.hasNext()) {
			final ElementRegEx regExClone = it.next().clone();
			elementFilterClone.addRegEx(regExClone);
		}

		return elementFilterClone;
	}

	@Override
	public String toString() {
		final String re = "Filter " + id + ": " + bezeichnung;
		return re;
	}

	@Override
	public int compareTo(final ElementFilter o) {
		int re = 0;

		if (this.getId() < o.getId()) {
			re = -1;
		} else if (this.getId() == o.getId()) {
			re = 0;
		} else if (this.getId() > o.getId()) {
			re = 1;
		}

		return re;
	}
}
