/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;

/**
 * Liefert eine Datenstruktur fr eine Tabelle.
 * 
 * @author HofmannJ
 * 
 */
public class TabellenDatenStruktur {
	/**
	 * Speichert die Anzahl der Attribute einer Tabellenzeile.
	 */
	private final int tdsAnzahlSpalten;

	/**
	 * Speichert die Liste der Tabellenzeilen.
	 */
	private final LinkedList<ITabellenZeile> zeilenDaten = new LinkedList<ITabellenZeile>();

	/**
	 * Speichert Modifizierungen, wie z.B. Filterungen.
	 */
	private LinkedList<ITabellenZeile> zeilenModifiziert = new LinkedList<ITabellenZeile>();

	/**
	 * Speichert die Titel der Spalten. Initialisierung des Array erfolgt im
	 * Konstruktor dieser Klasse.
	 */
	private SpaltenElement[] tdsSpaltenTitel;

	/**
	 * Gibt die Sortierung an Hand der Nummern der Spalten an.
	 */
	private Sortierung[] tdsSortierung;

	/**
	 * Array, speichert die Breiten der Spalten in Pixeln.
	 */
	private int[] tdsSpaltenBreiten;

	/**
	 * Zeigt an, aus welcher DatenQuelle die Daten geliefert werden sollen.
	 * Default ist ungefiltert.
	 */
	private DatenQuelle datenQuelle = DatenQuelle.UNGEFILTERT;

	/**
	 * Der zu dieser Tabelle zugehrige Filter.
	 */
	private TabellenFilter tabellenFilter;

	/**
	 * Speicher fr den DND-Transfer einer {@link ITabellenZeile}.
	 */
	private ITabellenZeile dndSpeicherZeile;

	/**
	 * Flag, zeigt an ob Sortiert werden soll oder nicht.
	 */
	private boolean sortieren;

	/**
	 * Flag, zeigt an ob gefiltert werden soll oder nicht.
	 */
	private boolean filtern;

	/**
	 * Flag, zeigt an ob die nderungsdarstellung verwendet werden soll oder
	 * nicht.
	 */
	private boolean aenderungsDarstellung;

	/**
	 * Konstruktor, bernimmt die Anzahl der Spalten der Tabelle.
	 * 
	 * @param anzSpalten
	 *            Anzahlt der Attribute pro Tabellenzeile
	 */
	public TabellenDatenStruktur(final int anzSpalten) {
		tdsAnzahlSpalten = anzSpalten;
		tdsSpaltenTitel = new SpaltenElement[tdsAnzahlSpalten];
		tabellenFilter = new TabellenFilter(tdsAnzahlSpalten, this);
	}

	/**
	 * Setzt die DatenQuelle.
	 * 
	 * @param {@link DatenQuelle}
	 */
	public void setDatenQuelle(final DatenQuelle datenQuelle) {
		this.datenQuelle = datenQuelle;
	}

	/**
	 * Liefert die aktuelle {@link DatenQuelle}.
	 * 
	 * @return {@link DatenQuelle}
	 */
	public DatenQuelle getDatenQuelle() {
		if (datenQuelle.equals(DatenQuelle.UNGEFILTERT)) {
			return DatenQuelle.UNGEFILTERT;
		}
		return DatenQuelle.GEFILTERT;
	}

	/**
	 * Fgt eine {@link ITabellenZeile} zur Liste der Zeilen hinzu. Die
	 * Attribute einer {@link ITabellenZeile} <b>mssen</b> das Interface
	 * {@link Comparable} zwingend erfllen!
	 * 
	 * @param zeile
	 *            Hinzuzufgende {@link ITabellenZeile}.
	 * @throws IllegalArgumentException
	 *             Wird geworfen wenn...
	 */
	public void addZeile(final ITabellenZeile zeile) {
		if (zeile != null) {
			if (zeile.getAnzahlAttribute() == tdsAnzahlSpalten) {
				boolean implementierenComparable = true;
				for (int i = 0; i < zeile.getAnzahlAttribute(); i++) {
					final Object attribut = zeile.getAttribut(i);
					if (!(attribut instanceof Comparable)) {
						implementierenComparable = false;
					}
				}

				if (!implementierenComparable) {
					throw new IllegalArgumentException(
							"Kann die bergebene ITabellenZeile nicht zur Liste der "
									+ "Tabellenzeilen hinzufgen. Mindestens eines der "
									+ "Attribute implementiert nicht das zwingend notwendige "
									+ "Interface Comparable!");
				}
				zeilenDaten.add(zeile);
			} else {
				throw new IllegalArgumentException(
						"Die bergebene ITabellenZeile hat nur "
								+ zeile.getAnzahlAttribute()
								+ " Attribute. Es sind jedoch mindestens "
								+ tdsAnzahlSpalten
								+ " notwendig. Kann die bergbene ITabellenZeile nicht zur Datenstruktur hinzufgen!");
			}
		} else {
			throw new IllegalArgumentException(
					"Die bergebene ITabellenZeile ist null und wird nicht hinzugefgt!");
		}
	}

	/**
	 * Liefert die Anzahl der Zeilen zurck. An Hand der gewhlten
	 * {@link DatenQuelle} werden die Daten aus der Liste
	 * {@link TabellenDatenStruktur#zeilenDaten} oder
	 * {@link TabellenDatenStruktur#_zeilenDatenGefiltert} geliefert.
	 * 
	 * @return Anzahl der Zeilen
	 */
	public int getAnzahlZeilen() {
		int re = -1;

		if (getZeilen() != null) {
			re = getZeilen().size();
		}

		return re;
	}

	/**
	 * Liefert die Zeilen zurck. Je nach gewhlter {@link DatenQuelle} wird
	 * entweder {@link TabellenDatenStruktur#zeilenDaten} oder
	 * {@link TabellenDatenStruktur#_zeilenDatenGefiltert} zurckgeliefert.
	 * 
	 * @return Zeilen der Tabelle
	 */
	public LinkedList<ITabellenZeile> getZeilen() {
		LinkedList<ITabellenZeile> re = null;

		// /*
		// * Holen der Daten aus der jeweiligen DatenQuelle
		// */
		// if (_datenQuelle.equals(DatenQuelle.UNGEFILTERT))
		// {
		// re = _zeilenDaten;
		// } else if (_datenQuelle.equals(DatenQuelle.GEFILTERT))
		// {
		// re = _zeilenDatenGefiltert;
		// }

		re = zeilenDaten;

		/*
		 * nderungsdarstellung berechen wenn gewnscht
		 */
		if (aenderungsDarstellung) {
			re = berechneAenderungsDarstellung(re);
		}

		/*
		 * Sortierung
		 */
		if (sortieren && (tdsSortierung != null)) {
			final LinkedList<ITabellenZeile> clone = (LinkedList<ITabellenZeile>) re
					.clone();
			re = sortiereListe(clone, tdsSortierung);
		}

		/*
		 * Filtern
		 */
		if (filtern && (tabellenFilter != null)) {
			re = filtereTabelle(re);
		}

		/*
		 * Modifizierung sichern
		 */
		zeilenModifiziert = re;

		return re;
	}

	/**
	 * Liefert die durch Filter, Sortierung und nderungsdarstellung
	 * modifizierte Zeilenstruktur zurck.
	 * 
	 * @return Modifizierte {@link ITabellenZeile}n-Struktur dieser
	 *         TabellenDatenStruktur.
	 */
	public LinkedList<ITabellenZeile> getZeilenModifiziert() {
		return zeilenModifiziert;
	}

	/**
	 * Liefert die Anzahl der Attribute pro Zeile.
	 * 
	 * @return Anzahl Attribute
	 */
	public int getAnzahlAttribute() {
		return tdsAnzahlSpalten;
	}

	/**
	 * bernimmt ein Array mit den Titeln der Spalten. Muss exakt der Anzahl der
	 * dem Konstruktor bergebenen Anzahl von Attributen entsprechen.
	 * 
	 * @param spaltenTitel
	 *            Spaltentitel
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn die Gre des bergebenen Arrays von
	 *             Spaltentiteln nicht der Anzahl der Attribute entspricht.
	 */
	public void setSpaltenTitel(final SpaltenElement[] spaltenTitel) {
		if (spaltenTitel.length == tdsSpaltenTitel.length) {
			tdsSpaltenTitel = spaltenTitel;
		} else {
			throw new IllegalArgumentException(
					"Das bergebene Array verfgt ber " + spaltenTitel.length
							+ " Spaltentitel. Es sind jedoch exakt "
							+ tdsSpaltenTitel.length
							+ " Spaltentitel notwendig!");
		}
	}

	/**
	 * Liefert den Titel einer Spalte zurck.
	 * 
	 * @param idxSpalte
	 * @return
	 */
	public SpaltenElement getSpaltenTitel(final int idxSpalte) {
		SpaltenElement re = new SpaltenElement("Keine Bezeichnung");

		if (tdsSpaltenTitel != null) {
			if (idxSpalte < tdsSpaltenTitel.length) {
				re = tdsSpaltenTitel[idxSpalte];
			}
		}

		return re;
	}

	/**
	 * Liefert alle Spaltentitel als Array vom Typ {@link SpaltenElement}.
	 * 
	 * @return Spaltentitel
	 */
	public SpaltenElement[] getAlleSpaltenTitel() {
		return tdsSpaltenTitel;
	}

	public LinkedList<SpaltenElement> getAlleSpaltenTitelAlsListe() {
		final LinkedList<SpaltenElement> re = new LinkedList<SpaltenElement>();
		for (final SpaltenElement spaltenElement : tdsSpaltenTitel) {
			re.add(spaltenElement);
		}

		return re;
	}

	/**
	 * Setzt Eine neue Sortierung an Hand einer Liste von {@link SpaltenElement}
	 * en. Diese wird mit den SpaltenTiteln verglichen und die Indexposition
	 * bestimmt.
	 * 
	 * @param spaltenElemente
	 */
	public void setSortierung(final LinkedList<SpaltenElement> spaltenElemente) {
		final LinkedList<SpaltenElement> alleSpaltenTitelAlsListe = getAlleSpaltenTitelAlsListe();
		final Sortierung[] neueSortierung = new Sortierung[spaltenElemente
				.size()];

		final Iterator<SpaltenElement> it = spaltenElemente.iterator();
		int i = 0;
		while (it.hasNext()) {
			final SpaltenElement vergleichsElement = it.next();
			final int indexOf = alleSpaltenTitelAlsListe
					.indexOf(vergleichsElement);
			neueSortierung[i] = new Sortierung(indexOf);
			neueSortierung[i]
					.setAufsteigend(vergleichsElement.istAufsteigend());
			i++;
		}

		tdsSortierung = neueSortierung;
	}

	/**
	 * Setzt die Sortierung an Hand des Index der Spalten.
	 * 
	 * @param sortierung
	 */
	public void setSortierung(final Sortierung[] sortierung) {
		if (sortierung.length <= tdsAnzahlSpalten) {
			boolean ok = true;
			for (final Sortierung wert : sortierung) {
				final Integer integer = wert.getSpalte();
				if ((integer < 0) || (integer >= tdsAnzahlSpalten)) {
					ok = false;
					break;
				}
			}

			if (ok) {
				this.tdsSortierung = sortierung;
			} else {
				throw new IllegalArgumentException(
						"Ein Wert innerhalb der Sortierung liegt ausserhalb des gltigen Bereiches von 0 bis "
								+ (tdsAnzahlSpalten - 1) + "!");
			}
		} else {
			throw new IllegalArgumentException(
					"Die Anzahl der Elemente in der bergbenen Sortierung ist hher "
							+ "als die Anzahl der vorhandenen Attribute + ("
							+ sortierung.length + " > " + tdsAnzahlSpalten
							+ ").");
		}

	}

	/**
	 * Liefert die aktuelle Sortierung zurck.
	 * 
	 * @return Aktuelle Sortierung.
	 */
	public Sortierung[] getSortierung() {
		return tdsSortierung;
	}

	/**
	 * Sortiert eine Liste aus {@link ITabellenZeile} an Hand einer Sortierfolge
	 * bestehend aus SpaltenIndexen. Sind mehrere Werte in einer Spalte gleich,
	 * so wird die nchste Spalte, welche in der Sortierfolge angegeben ist, zur
	 * weiteren Sortierung des Zeilenbereichs dieser Werte verwendet. Diese
	 * Methode ruft sich rekursiv auf.
	 * 
	 * @param liste
	 *            Liste der zu sortierenden {@link ITabellenZeile}n.
	 * @param sortierFolge
	 *            Sortierfolge bestehend aus den Indexen des Spalten, nach denen
	 *            sortiert werden soll.
	 * @return Sortierte Listes
	 */
	public LinkedList<ITabellenZeile> sortiereListe(
			final LinkedList<ITabellenZeile> listeUnsortiert,
			final Sortierung[] sortierFolge) {
		final LinkedList<ITabellenZeile> liste = listeUnsortiert;

		final Integer idxSortierSpalte = sortierFolge[0].getSpalte();
		// Index der Spalte, nach der sortiert werden soll.
		final boolean aufsteigend = sortierFolge[0].istAufsteigend();

		final Vergleich vgl = new Vergleich(idxSortierSpalte, aufsteigend);

		Collections.sort(liste, vgl);

		/*
		 * Soll nach weiteren Spalten sortiert werden? Richtet sich nach Anzahl
		 * der Elemente in der Sortierreihenfolge.
		 */
		if (sortierFolge.length > 1) {
			/*
			 * Entfernen des 1.sten Elementes in der Sortierreihenfolge
			 */
			final Sortierung[] sortierfolgeNeu = new Sortierung[sortierFolge.length - 1];
			for (int i = 1; i < sortierFolge.length; i++) {
				sortierfolgeNeu[i - 1] = sortierFolge[i];
			}

			/*
			 * Bestimmen der Zeilen-Positionen gleicher Eintrge in einer Spalte
			 */
			for (int i = 0; i < liste.size(); i++) {
				final Object ausgangsAttribut = liste.get(i).getAttribut(
						idxSortierSpalte);

				final int startZeile = i;
				// Startposition fr gleichen Inhalt
				int endeZeile = i;
				// Endposition fr gleichen Inhalt
				for (int j = i; j < liste.size(); j++) {
					final Object vergleichsAttribut = liste.get(j).getAttribut(
							idxSortierSpalte);

					/*
					 * Vergleichen der Attribute der Start und der Potentiellen
					 * Endzeile
					 */
					if (ausgangsAttribut.equals(vergleichsAttribut)) {
						endeZeile = j;
						i = j;
					}
				}

				/*
				 * Wenn die Startzeile < Endzeile ist, dann diesen Bereich
				 * sortieren.
				 */
				if (startZeile < endeZeile) {
					/*
					 * Auszugsliste zusammenstellen
					 */
					final LinkedList<ITabellenZeile> tempListe = new LinkedList<ITabellenZeile>();
					for (int zus = startZeile; zus <= endeZeile; zus++) {
						tempListe.add(liste.get(zus));

					}

					/*
					 * Sortieren der Auszugsliste
					 */
					sortiereListe(tempListe, sortierfolgeNeu);
					// sortiereListe(tempListe, new Integer[]{new Integer(2)});

					/*
					 * Entfernen der alten Eintrge
					 */
					liste.removeAll(tempListe);

					/*
					 * Einfgen der Sortierten Liste
					 */
					liste.addAll(startZeile, tempListe);
				}

			}

		}

		return liste;
	}

	/**
	 * Setzt die Breiten aller Spalten. Die Angaben der Breiten werden in einem
	 * Array von int in Pixeln bergeben.<br>
	 * <b>Achtung:</b> Die Anzahl der Elemente in diesem Array muss der Anzahl
	 * der Attribute entsprechen!
	 * 
	 * @param spaltenBreiten
	 *            Array von Spaltenbreiten in Pixeln
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn die Gre des Arrays nicht der Anzahl der
	 *             Attribute entspricht!
	 */
	public void setSpaltenBreiten(final int[] spaltenBreiten) {
		if ((spaltenBreiten.length >= 0)
				|| (spaltenBreiten.length <= tdsAnzahlSpalten)) {
			tdsSpaltenBreiten = spaltenBreiten;
		} else {
			throw new IllegalArgumentException(
					"Die Anzahl der Spaltenbreiten muss zwischen 0 und "
							+ tdsAnzahlSpalten + " liegen! Sie haben "
							+ spaltenBreiten.length + " bergeben!");
		}
	}

	/**
	 * Liefert die Breite fr eine Spalte zurck. Wurde keine Angaben zur Breite
	 * der Spalten gemacht, wird 50 zurckgeliefert.
	 * 
	 * @param idxSpalte
	 *            Breite der Spalte in Pixeln oder 50, wenn keine Breitenangaben
	 *            gemacht wurden.
	 * @return
	 */
	public int getSpaltenBreite(final int idxSpalte) {
		int re = 50;
		if ((tdsSpaltenBreiten != null)
				&& (idxSpalte < tdsSpaltenBreiten.length)) {
			re = tdsSpaltenBreiten[idxSpalte];
		}
		return re;
	}

	/**
	 * Liefert den {@link TabellenFilter} dieser Klasse, welcher sich z.B.
	 * mittels eines Dialoges bearbeiten lsst.
	 * 
	 * @return {@link TabellenFilter} dieser Klasse.
	 */
	public TabellenFilter getTabellenFilter() {
		return tabellenFilter;
	}

	public void setTabellenFilter(final TabellenFilter filter) {
		this.tabellenFilter = filter;
	}

	/**
	 * Filtert die Tabelle an Hand des eingestellten Filters.
	 * 
	 * @param zeilen
	 *            Liste der {@link ITabellenZeile}n der zu filternden Tabelle.
	 * @return Gefilterte Liste der {@link ITabellenZeile} der Tabelle.
	 */
	public LinkedList<ITabellenZeile> filtereTabelle(
			final LinkedList<ITabellenZeile> zeilen) {
		final LinkedList<ITabellenZeile> re = new LinkedList<ITabellenZeile>();

		/*
		 * ITabellenZeilen aus der ungefilterten Liste holen
		 */
		final Iterator<ITabellenZeile> it = zeilen.iterator();
		while (it.hasNext()) {
			final ITabellenZeile tabellenZeile = it.next();
			re.add(tabellenZeile);
		}

		/*
		 * datenFiltern, wird in den TabellenFilter verlagert
		 */
		if (tabellenFilter != null) {
			try {
				tabellenFilter.filtereTabelle(re);
			} catch (final IllegalArgumentException iae) {
				iae.printStackTrace();
			}
			// _zeilenDatenGefiltert = re;
		}

		return re;
	}

	/**
	 * Erzeugt die Kopfzeilen diester {@link TabellenDatenStruktur} an hand des
	 * Parameters {@link TabellenDatenStruktur#tdsSpaltenTitel} und
	 * {@link TabellenDatenStruktur#tdsSpaltenBreiten}.
	 * 
	 * @param tableViewer
	 *            {@link TableViewer}, fr den die Spalten (als
	 *            {@link TableColumn}s, zu erzeugen sind.
	 * @param verschiebbareSpalten
	 *            Wenn <code>true</code>, dann sind die Spalten verschiebbar,
	 *            bei <code>false</code> nicht.
	 */
	public void erzeugeTabellenKopfzeile(final TableViewer tableViewer,
			final boolean verschiebbareSpalten) {
		final Table table = tableViewer.getTable();

		final int anzAttribute = getAnzahlAttribute();
		for (int i = 0; i < anzAttribute; i++) {
			final TableColumn tc = new TableColumn(table, SWT.CENTER);
			tc.setText(getSpaltenTitel(i).toString());
			tc.setWidth(getSpaltenBreite(i));
			if (verschiebbareSpalten) {
				tc.setMoveable(true);
			}
		}

		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}

	@Override
	public String toString() {
		String re = "===TabellenDatenStruktur===\n";
		re = re + "Anzahl Attribute: " + getAnzahlAttribute() + "\n";
		re = re + "Anzahl Zeilen:    " + getAnzahlZeilen() + "\n";

		if ((tdsSpaltenTitel != null) && (tdsSpaltenTitel.length > 0)) {
			re = re + "Spaltentitel: ";

			for (final SpaltenElement titel : tdsSpaltenTitel) {
				re = re + titel + "; ";
			}

			re = re.substring(0, re.length() - 2);

			re = re + "\n";
		}

		final Iterator<ITabellenZeile> itZeilen = getZeilen().iterator();
		int i = 0;
		while (itZeilen.hasNext()) {
			re = re + "Zeile " + i + ": ";
			final ITabellenZeile zeile = itZeilen.next();
			for (int j = 0; j < tdsAnzahlSpalten; j++) {
				re = re + zeile.getAttribut(j) + "; ";
			}
			re = re.substring(0, re.length() - 2);
			re = re + "\n";

			re = re + "  Idx Attribute zur nderungsdarstellung: ";
			final int[] zuBeruecksichtigendeAttribute = zeile
					.getZuBeruecksichtigendeAttribute();
			for (final int attIdx : zuBeruecksichtigendeAttribute) {
				re = re + attIdx + "; ";
			}
			re = re.substring(0, re.length() - 2);
			re = re + "\n";

			i++;
		}

		re = re + "Spaltensorierungsreihenfolge: ";
		if (tdsSortierung != null) {

			for (final Sortierung wert : tdsSortierung) {
				re = re + wert.getSpalte() + "; ";
			}

			re = re.substring(0, re.length() - 2);
			re = re + "\n";
		} else {
			re = re + " keine Angabe\n";
		}

		re = re + "===========================\n";
		return re;
	}

	/**
	 * Setzt eine {@link ITabellenZeile} in den Speicher fr den DND-Transfer.
	 * 
	 * @param dndZeile
	 *            Zu speichernde {@link ITabellenZeile}.
	 */
	public void setDNDSpeicherZeile(final ITabellenZeile dndZeile) {
		this.dndSpeicherZeile = dndZeile;
	}

	/**
	 * Holt eine {@link ITabellenZeile} aus den Speicher fr den DND-Transfer.
	 * 
	 * @return Gespeicherte {@link ITabellenZeile} oder <code>null</code>, wenn
	 *         keine gespeichert wurde.
	 */
	public ITabellenZeile getDNDSpeicherZeile() {
		return dndSpeicherZeile;
	}

	public boolean dropZeile(final ITabellenZeile ziel) {
		boolean erfolgreich = false;

		if ((ziel != null) && (dndSpeicherZeile != null)) {
			final LinkedList<ITabellenZeile> zeilen = getZeilen();
			if ((zeilen.contains(dndSpeicherZeile)) && (zeilen.contains(ziel))) {
				final int idxQuelle = zeilen.indexOf(dndSpeicherZeile);
				final int idxZiel = zeilen.indexOf(ziel);

				if (zeilen.remove(dndSpeicherZeile)) {
					int idx = zeilen.indexOf(ziel);

					if (idxQuelle < idxZiel) {
						idx = idx + 1;
					}

					zeilen.add(idx, dndSpeicherZeile);

					dndSpeicherZeile = null;
					erfolgreich = true;
				}

			}
		} else if ((ziel == null) && (dndSpeicherZeile != null)) {
			final LinkedList<ITabellenZeile> zeilen = getZeilen();
			if (zeilen.contains(dndSpeicherZeile)) {
				zeilen.remove(dndSpeicherZeile);
			}
			zeilen.add(dndSpeicherZeile);
			dndSpeicherZeile = null;
			erfolgreich = true;
		}

		return erfolgreich;
	}

	/**
	 * Prft, ob die Tabelle sortiert wird oder nicht.
	 * 
	 * @return Liefert <code>true</code>, wenn die Tabelle sortiert wird, sonst
	 *         <code>false</code>.
	 */
	public boolean istSortiert() {
		return sortieren;
	}

	/**
	 * Setzt das Flag zum Sortieren der Tabelle.
	 * 
	 * @param sortieren
	 *            Muss <code>true</code> sein, wenn die Tabelle sortiert werden
	 *            soll, sonst <code>false</code>.
	 */
	public void setSortieren(final boolean sortieren) {
		this.sortieren = sortieren;
	}

	/**
	 * Prft, ob die Tabelle gefiltert wird.
	 * 
	 * @return Liefert <code>true</code>, wenn die Tabelle gefiltert wird bzw.
	 *         <code>false</code>, wenn die Tabelle nicht gefiltert wird.
	 */
	public boolean istGefiltert() {
		return filtern;
	}

	/**
	 * Setzt das Flag zum Filtern der Tabelle.
	 * 
	 * @param gefiltert
	 *            Muss <code>true</code> sein, wenn die Tabelle gefiltert werden
	 *            soll, sonst <code>false</code>.
	 */
	public void setGefiltert(final boolean gefiltert) {
		this.filtern = gefiltert;
	}

	/**
	 * Prft, ob die nderungsdarstellung gewhlt wurde.
	 * 
	 * @return Liefert <code>true</code> bei nderungsdarstellung, sonst
	 *         <code>false</code>.
	 */
	public boolean istAenderungsDarstellung() {
		return aenderungsDarstellung;
	}

	/**
	 * Setzt die nderungsdarstellung.
	 * 
	 * @param aenderungsDarstellung
	 *            Bei <code>true</code> wird die nderungsdarstellung gesetzt,
	 *            bei <code>false</code> die Zustandsdarstellung.
	 */
	public void setAenderungsDarstellung(final boolean aenderungsDarstellung) {
		this.aenderungsDarstellung = aenderungsDarstellung;
	}

	/**
	 * Berechnet aus einer Liste von {@link ITabellenZeile} eine
	 * nderungsdarstellung. Der Algorithmus arbeitet wie folgt: Zeige nur die
	 * Zeilen an, in denen sich eine Spalte im Vergleich zur letzten,
	 * dargestellten Zeile, mit gleichem ZeilenObjekt (==), gendert hat.
	 * 
	 * @param zeilen
	 *            Liste von {@link ITabellenZeile}n, aus denen eine
	 *            nderungsdarstellung berechnet werden soll.
	 * @return Liste mit den {@link ITabellenZeile}n der nderungsdarstellung
	 */
	private LinkedList<ITabellenZeile> berechneAenderungsDarstellung(
			final LinkedList<ITabellenZeile> zeilen) {
		final LinkedList<ITabellenZeile> re = new LinkedList<ITabellenZeile>();

		final Iterator<ITabellenZeile> it = zeilen.iterator();
		while (it.hasNext()) {
			final ITabellenZeile tabellenZeile = it.next();

			/*
			 * Gre == 0 => Zeile hinzufgen
			 */
			if (re.size() == 0) {
				re.add(tabellenZeile);
			} else {
				/*
				 * Letzte Zeile mit gleichem Objekt bestimmen
				 */
				final ITabellenZeile letzteZeileMitGleichemObjekt = bestimmeLetzteZeileMitZeilenObjektInListe(
						tabellenZeile.getZeilenObject(), re);
				if (letzteZeileMitGleichemObjekt == null) {
					re.add(tabellenZeile);
				} else {
					/*
					 * Vergleichen der Attribute
					 */

					/*
					 * Test 1 - sind die zu bercksichtigenden Attribute gleich?
					 */
					final int laengeBeruecksichtigendeAttTabellenZeile = tabellenZeile
							.getZuBeruecksichtigendeAttribute().length;

					final int laengeLetzteZeileBeruecksichtigendeAtt = letzteZeileMitGleichemObjekt
							.getZuBeruecksichtigendeAttribute().length;

					// Gleiche Lnge?
					if (laengeBeruecksichtigendeAttTabellenZeile == laengeLetzteZeileBeruecksichtigendeAtt) {
						// Gleiche zu bercksichtigende Attribute (gleiche
						// Indexe)?
						final int[] attTabbellenZeile = tabellenZeile
								.getZuBeruecksichtigendeAttribute();
						final int[] attLetzteZeile = letzteZeileMitGleichemObjekt
								.getZuBeruecksichtigendeAttribute();

						boolean gleicheZuBeruecksigtigendeAttribute = true;
						for (int i = 0; i < attLetzteZeile.length; i++) {
							if (attTabbellenZeile[i] != attLetzteZeile[i]) {
								gleicheZuBeruecksigtigendeAttribute = false;
								break;
							}
						}

						if (!gleicheZuBeruecksigtigendeAttribute) {
							final String err = "Die zu bercksichtigenden Attribute von "
									+ tabellenZeile
									+ " und "
									+ letzteZeileMitGleichemObjekt
									+ " stimmen nicht berein!";
							RwCompatActivator
									.getDefault()
									.getLog()
									.log(new Status(IStatus.INFO,
											RwCompatActivator.PLUGIN_ID,
											IStatus.OK, err, null));
						} else {
							/*
							 * Vergleichen der eigentlichen Inhalte der
							 * Attribute der jeweiligen ITabellenZeilen
							 */
							boolean gleich = true;
							for (int i = 0; i < attTabbellenZeile.length; i++) {
								if (!tabellenZeile
										.getAttribut(attTabbellenZeile[i])
										.equals(letzteZeileMitGleichemObjekt
												.getAttribut(attTabbellenZeile[i]))) {
									gleich = false;
									break;
								}
							}

							if (!gleich) {
								/*
								 * Hinzufgen der Zeile, wenn die zu
								 * vergleichenden Attribute nicht gleich sind.
								 */
								re.add(tabellenZeile);
							}

						}

					} else {
						final String err = "Lnge der zu bercksichtigenden Attribute fr die "
								+ "nderungsdarstellung der aufgebauten nderungsdarstellungsliste ("
								+ laengeLetzteZeileBeruecksichtigendeAtt
								+ ") stimmt nicht mit der Lnge der zu prfenden Zeile ("
								+ laengeBeruecksichtigendeAttTabellenZeile
								+ ") berein!";

						RwCompatActivator
								.getDefault()
								.getLog()
								.log(new Status(IStatus.INFO,
										RwCompatActivator.PLUGIN_ID,
										IStatus.OK, err, null));
					}

				}

			}
		}

		return re;
	}

	/**
	 * Bestimmt in einer Liste {@link ITabellenZeile} die letzte Zeile, welche
	 * ein bestimmtes Objekt als ZeilenObjekt enthlt.
	 * 
	 * @param zeilenObjekt
	 *            ZeilenObjekt, zu dem der letzte Eintrag in der Liste gesucht
	 *            werden soll.
	 * @param eilen
	 *            Liste von {@link ITabellenZeile}n, in welche der letzte
	 *            Eintrag mit dem entsprechenden ZeilenObjekt gesucht werden
	 *            soll.
	 * @return Liefert die letzte {@link ITabellenZeile}, welche das gegebene
	 *         Objekt als ZeilenObjekt enthlt oder <code>null</code>, falls
	 *         keine solche Zeile existiert.
	 */
	private ITabellenZeile bestimmeLetzteZeileMitZeilenObjektInListe(
			final Object zeilenObjekt,
			final LinkedList<ITabellenZeile> listeZeilen) {
		ITabellenZeile re = null;

		final Iterator<ITabellenZeile> it = listeZeilen.iterator();
		while (it.hasNext()) {
			final ITabellenZeile zeile = it.next();
			if (zeile.getZeilenObject() == zeilenObjekt) {
				re = zeile;
			}
		}

		return re;
	}

	/**
	 * {@link Comparator} zur Anwendung auf {@link Collections} vom Typ
	 * {@link ITabellenZeile}.<br>
	 * <b>Achtung:</b> Nur auf {@link Collections} des Typs
	 * {@link ITabellenZeile} anwendbar!!!
	 * 
	 * @author HofmannJ
	 * 
	 */
	@SuppressWarnings("unchecked")
	private static class Vergleich implements Comparator<ITabellenZeile> {
		private int idxSpalte = -1;
		private boolean aufsteigend = true;

		public Vergleich(final int idxSpalte, final boolean aufsteigend) {
			this.idxSpalte = idxSpalte;
			this.aufsteigend = aufsteigend;
		}

		@Override
		public int compare(final ITabellenZeile zeileA,
				final ITabellenZeile zeileB) {
			int re = 0;

			final Object attA = zeileA.getAttribut(idxSpalte);
			final Object attB = zeileB.getAttribut(idxSpalte);

			if ((attA instanceof Comparable) && (attB instanceof Comparable)) {
				final Comparable<Object> compA = (Comparable<Object>) attA;
				final Comparable<Object> compB = (Comparable<Object>) attB;

				if (aufsteigend) {
					re = compA.compareTo(compB);
				} else {
					re = compB.compareTo(compA);
				}
			}

			return re;
		}

	}

	/**
	 * Enumerator zur Auswahl der {@link DatenQuelle}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum DatenQuelle {
		GEFILTERT("Gefiltert", "Gefilterte ITabellenZeilen"), UNGEFILTERT(
				"Ungefiltert", "Ungefilterte ITabellenZeilen");

		/**
		 * Bezeichnung der {@link DatenQuelle}.
		 */
		private String quelleBezeichnung;

		/**
		 * Info zur {@link DatenQuelle}.
		 */
		private String quelleInfo;

		private DatenQuelle(final String bezeichnung, final String info) {
			quelleBezeichnung = bezeichnung;
			quelleInfo = info;
		}

		/**
		 * Liefert die Bezeichnung der {@link DatenQuelle}.
		 * 
		 * @return Bezeichnung der {@link DatenQuelle}
		 */
		public String getBezeichnung() {
			return quelleBezeichnung;
		}

		/**
		 * Liefert Infos zur {@link DatenQuelle}.
		 * 
		 * @return Infos zur {@link DatenQuelle}
		 */
		public String getInfo() {
			return quelleInfo;
		}

		@Override
		public String toString() {
			return quelleBezeichnung;
		}

	}

	/**
	 * Diese Klasse speichert die Filter zu einer Tabelle. Die Filter bestehen
	 * aus Regulren Ausdrcken und werden in einer Liste von {@link String}s
	 * gespeichert. Die Vorbelegung dieser Klasse ber den Parameter im
	 * Konstruktor hat mit der Anzahl der Spalten der Tabelle zu erfolgen.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public class TabellenFilter implements Cloneable {
		/**
		 * Anzahl der Spalten des TabellenFilters.
		 */
		private int filterAnzahlSpalten = -1;

		/**
		 * Liste der Filterausdrcke.
		 */
		private final TreeMap<Integer, FilterAusdruck> filterAusdruecke;

		/**
		 * Referenz auf die {@link TabellenDatenStruktur}.
		 */
		private final TabellenDatenStruktur tabellenDatenStruktur;

		/**
		 * Erzeugt einen TabellenFilter mit X Spalten. Die Filterausdrcke
		 * werden mit der RegEx <code>.*</code> vorbelegt und lassen damit jede
		 * Zeile durch.
		 * 
		 * @param anzahlSpalten
		 *            Anzahl der Spalten, fr welche Filter eingerichtet werden
		 *            sollen.
		 * @param tds
		 *            Verweis auf die {@link TabellenDatenStruktur}, auf die
		 *            sich der {@link TabellenFilter} bezieht.
		 */
		public TabellenFilter(final int anzahlSpalten,
				final TabellenDatenStruktur tds) {
			/**
			 * Anzahl der Spalten, fr die Filter bereitgestellt werden.
			 */
			filterAnzahlSpalten = anzahlSpalten;

			/**
			 * Verweis auf die TabellenDatenStruktur, zu der dieser Filter
			 * gehrt.
			 */
			tabellenDatenStruktur = tds;

			/**
			 * Liste von Filterausdrcken
			 */
			filterAusdruecke = new TreeMap<Integer, FilterAusdruck>();

			/**
			 * Initialisierung der Filterausdrcke fr die angegebene Anzahl von
			 * Spalten
			 */
			for (int i = 0; i < filterAnzahlSpalten; i++) {
				filterAusdruecke.put(i, new FilterAusdruck(i, ".*",
						tabellenDatenStruktur));
			}

		}

		/**
		 * Liefert eine Liste mit den Filterausdrcken zurck.
		 * 
		 * @return Liste mit Filterausdrcken
		 */
		public LinkedList<FilterAusdruck> getFilterAusdruecke() {
			final LinkedList<FilterAusdruck> liste = new LinkedList<FilterAusdruck>();
			final Iterator<Entry<Integer, FilterAusdruck>> it = filterAusdruecke
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<Integer, FilterAusdruck> entry = it.next();
				liste.add(entry.getValue());
			}

			// return _filterAusdruecke;
			return liste;
		}

		/**
		 * Liefert den Filterausdruck an Position X der Liste von
		 * Filteraudrcken zurck, entsprechend der X.ten Spalte der Tabelle.
		 * 
		 * @param spalte
		 *            Position bzw. Spalte des Filterausdrucks, der geliefert
		 *            werden soll.
		 * @return Liefert den jeweiligen Filterausdruck als Sting oder
		 *         <code>null</code>, wenn zu der angegebenen Index kein
		 *         Filterausdruck existiert.
		 */
		public FilterAusdruck getFilterAusdruck(final int spalte) {
			FilterAusdruck re = null;
			if ((spalte >= 0) && (spalte < filterAusdruecke.size())) {
				re = filterAusdruecke.get(spalte);
			}
			return re;
		}

		/**
		 * Setzt einen Filterausdruck (String mit einem Regulren Ausdruck) fr
		 * eine bestimmte Spalte der Tabelle.
		 * 
		 * @param spalte
		 *            Spalte der Tabelle, fr welche der Filterausdruck gesetzt
		 *            werden soll.
		 * @param filterAusdruckNeu
		 *            Zu setzender Filterausdruck.
		 * @throws IllegalArgumentException
		 *             Wird geworfen, wenn der Index der angegebenen Spalte
		 *             ungltig ist oder der Filterausdruck <code>null</code>
		 *             oder seine Lnge == 0 ist.
		 */
		public void setFilterAusdruck(final int spalte,
				final FilterAusdruck filterAusdruckNeu) {
			if ((filterAusdruckNeu != null)
					&& (filterAusdruckNeu.getFilter().length() > 0)) {
				if ((spalte >= 0) && (spalte < filterAusdruecke.size())) {
					filterAusdruecke.put(spalte, filterAusdruckNeu);
				} else {
					throw new IllegalArgumentException(
							"Die bergebene Spaltenindex ist ungltig. Nur der Bereich von 0 bis "
									+ (filterAusdruecke.size() - 1)
									+ " wird akzeptiert!");
				}
			} else {
				throw new IllegalArgumentException(
						"Der bergebene FilterAusdruck ist null oder ein String der Lnge 0!");
			}
		}

		/**
		 * Filtert mit den Einstellungen dieses {@link TabellenFilter}s eine
		 * Liste von {@link ITabellenZeile}n, welche die Struktur der Tabelle
		 * ausmacht. Ausgefilterte {@link ITabellenZeile}n werden aus der Liste
		 * entfernt, von daher muss eine Kopie der ursprnglichen Liste
		 * bergeben werden.
		 * 
		 * @param tabelle
		 *            Liste der {@link ITabellenZeile}n der zu filternden
		 *            Tabelle.
		 * @throws IllegalArgumentException
		 *             Wird geworfen, wenn eine der in der bergebenen Listen
		 *             vorhandenen {@link ITabellenZeile}n nicht ber die
		 *             notwendige Anzahl von Spalten verfgt.
		 */
		public void filtereTabelle(final LinkedList<ITabellenZeile> tabelle) {
			/*
			 * Prfung, ob alle Zeilen der zu prfenden Tabelle ber die
			 * Richtige Anzahl von Spalten verfgen.
			 */
			boolean anzahlSpaltenKorrekt = true;
			final Iterator<ITabellenZeile> itAnzSpaltenPruefung = tabelle
					.iterator();
			int posListe = 0;
			while (itAnzSpaltenPruefung.hasNext()) {
				final ITabellenZeile tabellenZeile = itAnzSpaltenPruefung
						.next();
				if (tabellenZeile.getAnzahlAttribute() != filterAnzahlSpalten) {
					anzahlSpaltenKorrekt = false;
					break;
				}
				posListe++;
			}

			/*
			 * Wenn die Anzahl der Spalten korrekt ist, dann mit dem Filtern
			 * beginnen.
			 */
			if (anzahlSpaltenKorrekt) {

				final LinkedList<ITabellenZeile> loeschListe = new LinkedList<ITabellenZeile>();

				/*
				 * Bestimmen der Zeilen, die gelscht werden sollen, wenn der
				 * Regulre Ausdruck von einer Spalte nicht erfllt wird.
				 */
				final Iterator<ITabellenZeile> itZeilen = tabelle.iterator();
				while (itZeilen.hasNext()) {
					boolean loeschen = false;
					final ITabellenZeile tabellenZeile = itZeilen.next();
					final int anzSpaltenTabellenZeile = tabellenZeile
							.getAnzahlAttribute();
					if (anzSpaltenTabellenZeile == filterAnzahlSpalten) {
						for (int i = 0; i < filterAnzahlSpalten; i++) {
							final String filterAusdruck = getFilterAusdruck(i)
									.getFilter();
							final Pattern pattern = Pattern
									.compile(filterAusdruck);
							final String wert = tabellenZeile.getAttribut(i)
									.toString();
							final Matcher matcher = pattern.matcher(wert);
							final boolean behalten = matcher.matches();

							// System.out.println("Wert: \"" + wert +
							// "\" Filter RegEx: "
							// + filterAusdruck + " = \"" + behalten + "\"");

							if (!behalten) {
								loeschen = true;
								break;
							}
						}
					}

					if (loeschen) {
						loeschListe.add(tabellenZeile);
					}

				}

				/*
				 * Lschen der Zeilen nach Vorgabe der loeschListe
				 */
				final Iterator<ITabellenZeile> itLoeschen = loeschListe
						.iterator();
				while (itLoeschen.hasNext()) {
					final ITabellenZeile loeschZeile = itLoeschen.next();
					if (tabelle.contains(loeschZeile)) {
						tabelle.remove(loeschZeile);
					}
				}
			} else {
				throw new IllegalArgumentException(
						"Die ITabellenZeile an Position "
								+ posListe
								+ " der bergebenen Liste verfgt nicht ber die notwendige Anzahl von "
								+ filterAnzahlSpalten + " Spalten!");
			}

		}

		@Override
		public String toString() {
			String re = "===TabellenFilter===\n";

			final Iterator<Entry<Integer, FilterAusdruck>> it = filterAusdruecke
					.entrySet().iterator();
			int i = 0;

			while (it.hasNext()) {
				final String filter = it.next().getValue().getFilter();
				re = re + "Filter " + i + ": " + filter + "\n";
				i++;
			}

			re = re + "====================\n";

			return re;
		}

		@Override
		public Object clone() {
			try {
				super.clone();
			} catch (final CloneNotSupportedException e) {
				throw new IllegalStateException(e);
			}

			Object re = null;
			final TabellenFilter tfNeu = new TabellenFilter(
					this.filterAnzahlSpalten, this.tabellenDatenStruktur);

			final Iterator<Entry<Integer, FilterAusdruck>> it = filterAusdruecke
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<Integer, FilterAusdruck> entry = it.next();
				final FilterAusdruck filter = entry.getValue();
				final FilterAusdruck clone = filter.clone();

				tfNeu.filterAusdruecke.put(entry.getKey(), clone);
			}

			re = tfNeu;
			return re;
		}
	}

	/**
	 * Diese Klasse implementiert einen Filterausdruck bestehend aus Spalte und
	 * Regulrem Ausdruck des aktuellen Filters.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public class FilterAusdruck implements Cloneable {
		/**
		 * Spalte zu der dieser FilterAusdruck zugeordnet ist.
		 */
		private int spalte = -1;

		/**
		 * Regulrer Ausdruck des Filters.
		 */
		private String filter = ".*";

		/**
		 * {@link TabellenDatenStruktur} auf die sich dieser Filter bezieht.
		 * Wird zur Ermittlung der Spaltenbezeichnung bentigt.
		 */
		private final TabellenDatenStruktur tds;

		/**
		 * Konstruktor.
		 * 
		 * @param spalte
		 *            Index der Spalte in der Tabelle, auf die sich dieser
		 *            {@link FilterAusdruck} bezieht.
		 * @param filter
		 *            Regulrer Ausdruck des Filters.
		 * @param tds
		 *            {@link TabellenDatenStruktur} auf die sich dieser
		 *            {@link FilterAusdruck} bezieht.
		 */
		public FilterAusdruck(final int spalte, final String filter,
				final TabellenDatenStruktur tds) {
			this.spalte = spalte;
			this.filter = filter;
			this.tds = tds;
		}

		/**
		 * Liefert die Spalte, auf die sich dieser {@link FilterAusdruck}
		 * bezieht.
		 * 
		 * @return Spalte des {@link FilterAusdruck}s
		 */
		public int getSpalte() {
			return spalte;
		}

		/**
		 * Liefert die Bezeichnung der Spalte, auf die sich dieser
		 * {@link FilterAusdruck} bezieht. Wird an Hand der zugehrigen
		 * {@link TabellenDatenStruktur} ermittelt.
		 * 
		 * @return Bezeichnung der Spalte auf die sich dieser
		 *         {@link FilterAusdruck} bezieht.
		 */
		public String getBezeichnung() {
			return tds.getSpaltenTitel(spalte).toString();
		}

		/**
		 * Liefert den regulren Ausdruck dieses {@link FilterAusdruck}s.
		 * 
		 * @return Regulrer Ausdruck
		 */
		public String getFilter() {
			return filter;
		}

		/**
		 * Setzt den regulren Ausdruck dieses {@link FilterAusdruck}s.
		 * 
		 * @param filter
		 *            Zu setzender regulrer Ausdruck
		 */
		public void setFilter(final String filter) {
			this.filter = filter;
		}

		@Override
		public FilterAusdruck clone() {
			try {
				super.clone();
			} catch (final CloneNotSupportedException e) {
				throw new IllegalStateException(e);
			}

			// System.out.println("==>" + toString() + "<==");
			final FilterAusdruck fa = new FilterAusdruck(getSpalte(),
					getFilter(), tds);
			return fa;

		}

		@Override
		public String toString() {
			final String re = "Filterausdruck - Spalte: " + spalte
					+ " Filter: " + filter;
			return re;
		}
	}

}
