/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.util;

import org.eclipse.core.runtime.Platform;

import de.bsvrz.buv.rw.basislib.internal.RahmenwerkService;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Definitionen der Platzhalter fr die Anzeige von Informationen ber die
 * Anwendungsumgebung.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwInfoDef.java 50446 2014-05-16 11:35:00Z hoesel $
 */
public enum RwInfoDef {
	/** Name des angemeldeten Nutzers. */
	USER("{user}"),
	/** PID des angemeldeten Benutzers. */
	USERPID("{userpid}"),
	/** Hostname aus der Datenverteileranmeldung. */
	HOST("{host}"),
	/** Port aus der Datenverteileranmeldung. */
	PORT("{port}"),
	/** Standardsimulationsvariante. */
	SIMVAR("{simvar}"),
	/** Name der verwendeten AOE. */
	AOE("{aoe}"),
	/** PID der verwendeten AOE. */
	AOEPID("{aoepid}"),
	/** Rahmenwerkversion. */
	VERSION("{version}"),
	/** Nutzerklasse(n). */
	USERCLASS("{userclass}");

	private final String id;

	private RwInfoDef(final String id) {
		this.id = id;
	}

	/**
	 * liefert den formatierten Informationsstring, die enthaltenen Platzhalter
	 * werden entsprechend ersetzt.
	 * 
	 * @param formatStr
	 *            der Formatstring
	 * @return der resultierende String
	 */
	public static String format(final String formatStr) {
		return VERSION.berechneString(formatStr);
	}

	private String berechneString(final String formatStr) {
		String result = formatStr;

		if (formatStr == null) {
			result = "";
		}

		for (final RwInfoDef item : RwInfoDef.values()) {
			if (result.contains(item.id)) {
				final String replacement;
				switch (item) {
				case AOE:
					replacement = getAoe();
					break;
				case AOEPID:
					replacement = getAoePid();
					break;
				case HOST:
					replacement = getDavHostName();
					break;
				case PORT:
					replacement = getDavPort();
					break;
				case SIMVAR:
					replacement = getSimulationsVariante();
					break;
				case USER:
					replacement = getBenutzerName();
					break;
				case USERPID:
					replacement = getBenutzerPid();
					break;
				case VERSION:
					replacement = getRahmenwerkVersion();
					break;
				case USERCLASS:
					replacement = getNutzerklasse();
					break;
				default:
					replacement = null;
					break;
				}

				if (replacement != null) {
					while (result.contains(item.id)) {
						result = result.replace(item.id, replacement);
					}
				}
			}
		}

		return result;
	}

	private String getRahmenwerkVersion() {
		return Platform.getBundle("de.bsvrz.buv.rw.rw").getVersion().toString();
	}

	private String getAoePid() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			return RahmenwerkService.getService().getRahmenWerk()
					.getDavVerbindung().getDataModel()
					.getConfigurationAuthority().getPid();
		}
		return "";
	}

	private String getAoe() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			return RahmenwerkService.getService().getRahmenWerk()
					.getDavVerbindung().getDataModel()
					.getConfigurationAuthority().getPid();
		}
		return "";
	}

	private String getSimulationsVariante() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			return Short.toString(RahmenwerkService.getService()
					.getRahmenWerk().getDavVerbindung()
					.getClientDavParameters().getSimulationVariant());
		}
		return "";
	}

	private String getDavPort() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			return Integer.toString(RahmenwerkService.getService()
					.getRahmenWerk().getDavVerbindung()
					.getClientDavParameters().getDavCommunicationSubAddress());
		}
		return "";
	}

	private String getDavHostName() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			return RahmenwerkService.getService().getRahmenWerk()
					.getDavVerbindung().getClientDavParameters()
					.getDavCommunicationAddress();
		}
		return "";
	}

	private String getBenutzerPid() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			return RahmenwerkService.getService().getRahmenWerk().getBenutzer()
					.getPid();
		}
		return "";
	}

	private String getBenutzerName() {
		String result = RahmenwerkService.getService().getRahmenWerk()
				.getBenutzerName();
		if (result == null) {
			result = "";
		}
		return result;
	}

	private String getNutzerklasse() {
		String result = "";
		final ClientDavInterface davVerbindung = RahmenwerkService.getService()
				.getRahmenWerk().getDavVerbindung();
		final SystemObject benutzer = davVerbindung.getLocalUser();
		final Aspect asp = davVerbindung.getDataModel().getAspect(
				"asp.parameterSoll");
		if (RahmenwerkService.getService().getRahmenWerk()
				.usesBerechtigungenNeu()) {
			//neue Berechtigungsklassen
			final AttributeGroup atg = davVerbindung.getDataModel()
					.getAttributeGroup("atg.berechtigungsklassen");
			final DataDescription desc = new DataDescription(atg, asp);
			final ResultData resultData = davVerbindung.getData(benutzer, desc,
					60000);
			if (resultData != null && resultData.hasData()) {
				final Data data = resultData.getData();
				final ReferenceArray referenceArray = data
						.getReferenceArray("Berechtigungsklassen");
				for (int i = 0; i < referenceArray.getLength(); i++) {
					final SystemObject systemObject = referenceArray
							.getSystemObject(i);
					if (!result.isEmpty()) {
						result = result + ", ";
					}
					result = result + systemObject.getName();
				}
			}
		} else {
			//alte Berechtigungsklassen
			final AttributeGroup atg = davVerbindung.getDataModel()
					.getAttributeGroup("atg.benutzerParameter");
			final DataDescription desc = new DataDescription(atg, asp);
			final ResultData resultData = davVerbindung.getData(benutzer, desc,
					60000);
			if (resultData != null && resultData.hasData()) {
				final Data data = resultData.getData();
				final ReferenceValue referenceValue = data
						.getReferenceValue("berechtigungsklasse");
				final SystemObject objNutzerklasse = referenceValue
						.getSystemObject();
				if (objNutzerklasse != null) {
					result = objNutzerklasse.getName();
				}
			}
		}
		return result;
	}
}
