/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.einstellungen;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungAvailabilityListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungChangeListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungLocation;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungOwnerType;
import de.bsvrz.buv.rw.basislib.einstellungen.Einstellungen;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsAdresse;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsFactory;
import de.bsvrz.buv.rw.basislib.einstellungen.SpeicherKey;
import de.bsvrz.buv.rw.basislib.einstellungen.UrlasserDialogAbgebrochenException;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserDatenDialog;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserDatenSender;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;
import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Implementierung einer Komponente fr die Schnittstelle {@link Einstellungen},
 * wird als OSGI-Service in der Applikation zur Verfgung gestellt.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: EinstellungenImpl.java 51069 2014-07-30 13:31:37Z peuker $
 */
public class EinstellungenImpl implements Einstellungen {

	/**
	 * Exception zum Abfangen des Abbruchs des Urlasserdialogs aus dem
	 * UI-Thread.
	 */
	public class CancelException extends RuntimeException {

		/** ID zur Serialisierung. */
		private static final long serialVersionUID = 1L;
	}

	private static EinstellungenImpl instance;
	private EinstellungenSpeichermanager speicherManager;
	private final Map<String, EinstellungsFactory> factories = new HashMap<String, EinstellungsFactory>();

	/** aktiviert den Service. */
	protected void activate() {
		EinstellungenImpl.instance = this;
	}

	/** deaktiviert den Service. */
	protected void deactivate() {
		EinstellungenImpl.instance = null;
	}

	/**
	 * die globale Instanz der Komponente.
	 *
	 * @return die Instanz
	 */
	public static EinstellungenImpl getInstance() {
		return EinstellungenImpl.instance;
	}

	/**
	 * bindet eine Instanz des Service {@link Rahmenwerk} an die Komponente.
	 *
	 * @param newRahmenwerk
	 *            der Service
	 */
	protected void bindRahmenwerk(final Rahmenwerk newRahmenwerk) {
		this.speicherManager = new EinstellungenSpeichermanager(this,
				newRahmenwerk);
	}

	/**
	 * entfernt eine Instanz des Service {@link Rahmenwerk} von der Komponente.
	 *
	 * @param oldRahmenwerk
	 *            der Service
	 */
	protected void unbindRahmenwerk(final Rahmenwerk oldRahmenwerk) {
		this.speicherManager = null;
	}

	/**
	 * fgt eine Factory zur Umsetzung von Einstellungen hinzu.
	 *
	 * @param factory
	 *            die Factory
	 */
	protected void addEinstellungsFactory(final EinstellungsFactory factory) {
		this.factories.put(factory.getTyp(), factory);
	}

	/**
	 * entfernt eine Factory zur Umsetzung von Einstellungen hinzu.
	 *
	 * @param factory
	 *            die Factory
	 */
	protected void removeEinstellungsFactory(final EinstellungsFactory factory) {
		this.factories.remove(factory.getTyp());
	}

	@Override
	public Object getValue(final EinstellungsAdresse adresse)
			throws IOException {
		return speicherManager.getValue(adresse);
	}

	@Override
	public void setValue(final EinstellungsAdresse adresse,
			final Object einstellung) throws IOException {
		try {
			if ((adresse.getLocation() == EinstellungLocation.NETZWERKWEIT)
					&& (adresse.getOwnerType() == EinstellungOwnerType.SYSTEM)) {

				Display.getDefault().syncExec(new Runnable() {
					@Override
					public void run() {
						final UrlasserDatenDialog dialog = new UrlasserDatenDialog(
								PlatformUI.getWorkbench()
										.getActiveWorkbenchWindow().getShell(),
								new UrlasserDatenSender() {

									@Override
									public void execute(
											final ClientDavInterface verbindung,
											final UrlasserInfo urlasser) {
										try {
											// TODO Daten ber tatschliche
											// Urlasserverbindung senden
											setValue(adresse, einstellung,
													urlasser);
										} catch (final IOException e) {
											ErrorDialog
													.openError(
															PlatformUI
																	.getWorkbench()
																	.getActiveWorkbenchWindow()
																	.getShell(),
															"Fehler",
															"Einstellung konnte nicht gespeichert werden",
															new Status(
																	IStatus.ERROR,
																	RwEinstellungenActivator.PLUGIN_ID,
																	e.getLocalizedMessage(),
																	e));
										}
									}
								});
						if (dialog.open() != Window.OK) {
							throw new CancelException();
						}
					}
				});

			} else {
				setValue(adresse, einstellung, null);
			}
		} catch (final CancelException e) {
			throw new UrlasserDialogAbgebrochenException();
		}
	}

	@Override
	public void setValue(final EinstellungsAdresse adresse,
			final Object einstellung, final UrlasserInfo urlasser)
			throws IOException {
		UrlasserInfo info = urlasser;
		if (urlasser == null) {
			final Rahmenwerk rahmenwerk = speicherManager.getRahmenwerk();
			if (rahmenwerk.isOnline()) {
				info = new UrlasserInfo(rahmenwerk.getBenutzer(),
						rahmenwerk.getPasswort(), "Einstellung fr "
								+ adresse.getId() + " aktualisiert",
						rahmenwerk.getBenutzerName());
			} else {
				info = new UrlasserInfo(null, rahmenwerk.getPasswort(),
						"Einstellung fr " + adresse.getId() + " aktualisiert",
						rahmenwerk.getBenutzerName());
			}
		}
		try {
			speicherManager.setValue(adresse, einstellung, info);
		} catch (final EinstellungsSpeicherException e) {
			throw new IOException(e);
		}
	}

	@Override
	public void addEinstellungsListener(final EinstellungChangeListener listener) {
		speicherManager.addChangeListener(listener, null);
	}

	@Override
	public void addEinstellungsListener(
			final EinstellungChangeListener listener, final String category) {
		speicherManager.addChangeListener(listener, category);
	}

	@Override
	public void removeEinstellungsListener(
			final EinstellungChangeListener listener) {
		speicherManager.removeChangeListener(listener, null);
	}

	@Override
	public void removeEinstellungsListener(
			final EinstellungChangeListener listener, final String category) {
		speicherManager.removeChangeListener(listener, category);
	}

	@Override
	public void addEinstellungsAvailabilityListener(
			final EinstellungAvailabilityListener listener) {
		speicherManager.addAvailabilityListener(listener);
	}

	@Override
	public void removeEinstellungsAvailabilityListener(
			final EinstellungAvailabilityListener listener) {
		speicherManager.removeAvailabilityListener(listener);
	}

	EinstellungsFactory getFactory(final String typ) {
		return factories.get(typ);
	}

	@Override
	public void removeValue(final EinstellungsAdresse adresse) {
		try {
			setValue(adresse, null);
		} catch (final IOException e) {
			// sollte beim Entfernen nicht auftreten
		}
	}

	@Override
	public Map<String, String> getEinstellungsId(final SpeicherKey key)
			throws IOException {
		return speicherManager.getEinstellungsId(key);
	}
}
