/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.einstellungen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungAvailabilityListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungChangeListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsAdresse;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsEvent;

/**
 * Manager zur Verwaltung und Benachrichtigungen von Einstellungslistenern.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: EinstellungsListenerManager.java 45420 2013-08-09 12:12:12Z peuker $
 */
class EinstellungsListenerManager {

	private final List<EinstellungAvailabilityListener> avaliabilityListeners = new ArrayList<EinstellungAvailabilityListener>();

	private final List<EinstellungChangeListener> globalListeners = new ArrayList<EinstellungChangeListener>();
	private final Map<String, List<EinstellungChangeListener>> categoryListeners = new HashMap<String, List<EinstellungChangeListener>>();

	public void fireChanges(final SpeicherErgebnis ergebnis,
			final EinstellungsAdresse adresse, final Object einstellung) {

		final List<EinstellungChangeListener> listeners = new ArrayList<EinstellungChangeListener>();
		synchronized (globalListeners) {
			listeners.addAll(globalListeners);
		}
		synchronized (categoryListeners) {
			final List<EinstellungChangeListener> liste = categoryListeners
					.get(adresse.getTyp());
			if (liste != null) {
				listeners.addAll(liste);
			}
		}

		if (!listeners.isEmpty()) {
			final EinstellungsEvent event = new EinstellungsEvent(this,
					adresse, einstellung);
			for (final EinstellungChangeListener listener : listeners) {
				switch (ergebnis) {
				case CHANGED:
					listener.einstellungAktualisiert(event);
					break;
				case NEW:
					listener.einstellungAngelegt(event);
					break;
				case REMOVED:
					listener.einstellungEntfernt(event);
					break;
				case UNCHANGED:
				default:
					break;
				}
			}
		}
	}

	public void fireAvailability(final boolean status) {
		synchronized (avaliabilityListeners) {
			for (final EinstellungAvailabilityListener listener : avaliabilityListeners) {
				if (status) {
					listener.available();
				} else {
					listener.disabled();
				}
			}
		}
	}

	public void add(final EinstellungChangeListener listener,
			final String category) {
		if (category == null) {
			synchronized (globalListeners) {
				globalListeners.add(listener);
			}
		} else {
			synchronized (categoryListeners) {
				List<EinstellungChangeListener> liste = categoryListeners
						.get(category);
				if (liste == null) {
					liste = new ArrayList<EinstellungChangeListener>();
					categoryListeners.put(category, liste);
				}
				liste.add(listener);
			}
		}
	}

	public void remove(final EinstellungChangeListener listener,
			final String category) {
		if (category == null) {
			synchronized (globalListeners) {
				globalListeners.remove(listener);
			}
		} else {
			synchronized (categoryListeners) {
				final List<EinstellungChangeListener> liste = categoryListeners
						.get(category);
				if (liste != null) {
					liste.remove(listener);
					if (liste.isEmpty()) {
						categoryListeners.remove(category);
					}
				}
			}
		}
	}

	public void add(final EinstellungAvailabilityListener listener) {
		synchronized (avaliabilityListeners) {
			avaliabilityListeners.add(listener);
		}
	}

	public void remove(final EinstellungAvailabilityListener listener) {
		synchronized (avaliabilityListeners) {
			avaliabilityListeners.remove(listener);
		}
	}
}
