/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.internal;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobManager;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.SafeRunnable;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.dav.DavConnectionException;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent.Type;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.login.LoginDialog.AnmeldeDaten;
import de.bsvrz.buv.rw.rw.menu.RwToolbarElementManagerImpl;
import de.bsvrz.dav.daf.main.ApplicationCloseActionHandler;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.DavConnectionListener;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList.Argument;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;

/**
 * Implementierung des Rahmenwerk-Interfaces, der allen Plugins als OSGI-Service
 * zur Verfgung steht.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkImpl.java 51039 2014-07-23 12:10:12Z peuker $
 */
public final class RahmenwerkImpl implements Rahmenwerk {

	/**
	 * interner Hintergrundjob, der auf Listener wartet, die ein Veto zur
	 * Trennung der Datenverteilerverbindung eingelegt haben.
	 */
	private class VetoListenerJob extends Job {

		private final Set<DavVerbindungsListener> vetoListeners = new HashSet<DavVerbindungsListener>();
		private final Rahmenwerk rahmenwerk;
		private final Object lock;
		private boolean running;

		public VetoListenerJob(final RahmenwerkImpl rahmenwerk,
				final Object lock) {
			super("VetoListenerHandler");
			this.rahmenwerk = rahmenwerk;
			this.lock = lock;
		}

		@Override
		protected IStatus run(final IProgressMonitor monitor) {

			monitor.setTaskName("Warte auf zu schlieende Datenverteilerverbindungen ....");

			synchronized (lock) {
				running = true;
				lock.notifyAll();
			}

			final DavVerbindungsEvent event = new DavVerbindungsEvent(
					rahmenwerk, DavVerbindungsEvent.Type.DISCONNECT,
					rahmenwerk.getDavVerbindung());

			while (running) {
				synchronized (vetoListeners) {
					final Set<DavVerbindungsListener> keinVeto = new HashSet<DavVerbindungsListener>();
					for (final DavVerbindungsListener listener : vetoListeners) {
						if (!listener.verbindungHalten(event)) {
							keinVeto.add(listener);
						}
					}
					vetoListeners.removeAll(keinVeto);
					if (vetoListeners.isEmpty()) {
						running = false;
					}
				}

				if (running) {
					try {
						Thread.sleep(2000);
					} catch (final InterruptedException e) {
						RahmenwerkImpl.LOGGER.warning(e.getLocalizedMessage());
					}
				}
			}

			return Status.OK_STATUS;
		}

		public void addVetoListener(final DavVerbindungsListener listener) {
			synchronized (vetoListeners) {
				vetoListeners.add(listener);
			}
		}

		@Override
		public boolean belongsTo(final Object family) {
			return Rahmenwerk.JOB_FAMILY.equals(family);
		}

		public boolean isRunning() {
			return running;
		}
	}

	private static RahmenwerkImpl instanz;

	/**
	 * liefert die Instanz des Services.
	 *
	 * @return die Instanz
	 */
	public static RahmenwerkImpl getInstanz() {
		return RahmenwerkImpl.instanz;
	}

	private final String[] arguments;

	private ClientDavConnection davVerbindung;

	private String benutzerName;

	private String passwort;

	private final List<DavVerbindungsListener> verbindungsListenerList = new ArrayList<DavVerbindungsListener>();

	private static final Debug LOGGER = Debug.getLogger();

	private final RwToolbarElementManagerImpl toolbarElementManager;

	/** Standardkonstruktor. */
	public RahmenwerkImpl() {
		arguments = Platform.getApplicationArgs();
		Debug.init(RahmenwerkActivator.PLUGIN_ID, getStartParameter());
		RahmenwerkImpl.instanz = this;

		/*
		 * Der Thread dient einzig und allein der Initialisierung der Klasse
		 * ClientDavParameters, da hier eine erhebliche Verzgerung beim Laden
		 * der BundleProperties der Klasse entsteht, die sich sonst erst beim
		 * Herstellen der Datenverteilerverbindung nach der Besttigung des
		 * Login-Dialogs bemerkbar machen wrde.
		 */
		final Thread initialisationThread = new Thread("DavInitialiser") {
			@Override
			public void run() {
				try {
					final ClientDavParameters parameters = new ClientDavParameters();
					RahmenwerkImpl.LOGGER
							.finer("ClientDavParameter initialisiert"
									+ parameters);
				} catch (final MissingParameterException e) {
					RahmenwerkImpl.LOGGER
							.finer("ClientDavParameterinitialisierung fehlgeschalgen: "
									+ e.getLocalizedMessage());
				}
			}
		};
		initialisationThread.start();

		toolbarElementManager = new RwToolbarElementManagerImpl();
	}

	@Override
	public void addDavVerbindungsListener(final DavVerbindungsListener listener) {
		synchronized (verbindungsListenerList) {
			verbindungsListenerList.add(listener);
		}
	}

	/** beendet die bestehende Datenverteilerverbindung. */
	public void davVerbindungBeenden() {
		if (davVerbindung != null) {

			final DavVerbindungsEvent event = new DavVerbindungsEvent(this,
					DavVerbindungsEvent.Type.DISCONNECT, davVerbindung);

			final Object vetoLock = new Object();
			final VetoListenerJob vetoListenerJob = new VetoListenerJob(this,
					vetoLock);

			for (final DavVerbindungsListener listener : verbindungsListenerList) {
				if (listener.verbindungHalten(event)) {
					vetoListenerJob.addVetoListener(listener);
				}
			}

			vetoListenerJob.schedule();
			synchronized (vetoLock) {
				try {
					if (!vetoListenerJob.isRunning()) {
						vetoLock.wait(5000);
					}
				} catch (final InterruptedException e) {
					RahmenwerkImpl.LOGGER.warning(e.getLocalizedMessage());
				}
			}

			final IJobManager jobManager = Job.getJobManager();
			jobManager.cancel(Rahmenwerk.JOB_FAMILY);
			try {
				jobManager.join(Rahmenwerk.JOB_FAMILY, null);
			} catch (final OperationCanceledException e) {
				RahmenwerkImpl.LOGGER.warning(e.getLocalizedMessage(), e);
			} catch (final InterruptedException e) {
				RahmenwerkImpl.LOGGER.warning(e.getLocalizedMessage(), e);
			}

			davVerbindung.disconnect(false, null);
		}
	}

	/**
	 * stellt die Datenverteilerverbindung mit den bergebenen Daten her.
	 *
	 * @param name
	 *            der Benutzername
	 * @param pass
	 *            das Passwort des Benutzers
	 * @param host
	 *            der Hostname (oder IP-Adresse) des Datenverteilers
	 * @param port
	 *            der Port fr die Verbindungsaufnahme
	 * @throws DavConnectionException
	 *             die Verbindung konnte nicht hergestellt werden
	 */
	public void davVerbindungHerstellen(final String name, final String pass,
			final String host, final int port) throws DavConnectionException {

		if (davVerbindung != null) {
			throw new IllegalStateException(
					"Es besteht bereits eine Datenverteilerverbindung");
		}

		benutzerName = name;
		this.passwort = pass;

		final ArgumentList argumentList = getStartParameter();

		try {

			final ClientDavParameters parameter = new ClientDavParameters(
					argumentList);
			parameter.setDavCommunicationAddress(host);
			parameter.setDavCommunicationSubAddress(port);

			if (argumentList.hasArgument("-ConfigurationPathBenutzer")) {
				parameter.setConfigurationPath(argumentList.fetchArgument(
						"-ConfigurationPathBenutzer").getValue());
			}

			if (argumentList.hasArgument("-applicationTypePid")) {
				parameter.setApplicationTypePid(argumentList.fetchArgument(
						"-applicationTypePid").getValue());
			}

			final StringBuilder appName = new StringBuilder();
			appName.append("Rahmenwerk_auf_");
			appName.append(InetAddress.getLocalHost().getHostAddress());
			appName.append("_Benutzer_");
			appName.append(benutzerName);
			parameter.setApplicationName(appName.toString());

			davVerbindung = new ClientDavConnection(parameter);

			davVerbindung.connect();
			davVerbindung.addConnectionListener(new DavConnectionListener() {

				@Override
				public void connectionClosed(final ClientDavInterface dav) {
					if (dav.equals(davVerbindung)) {
						final DavVerbindungsEvent event = new DavVerbindungsEvent(
								this, Type.DISCONNECT, davVerbindung);
						davVerbindung = null;
						fireVerbindungsListeners(event);
					}
				}
			});

			davVerbindung.setCloseHandler(new ApplicationCloseActionHandler() {
				@Override
				public void close(final String message) {
					if (davVerbindung != null) {
						davVerbindung.disconnect(true, message);
						final DavVerbindungsEvent event = new DavVerbindungsEvent(
								this, Type.DISCONNECT, davVerbindung);
						davVerbindung = null;
						fireVerbindungsListeners(event);
					}
				}
			});

			davVerbindung.login(name, passwort);

			MessageSender.getInstance().init(davVerbindung, "Rahmenwerk",
					appName.toString());

			final DavVerbindungsEvent event = new DavVerbindungsEvent(this,
					Type.CONNECT, davVerbindung);
			fireVerbindungsListeners(event);

		} catch (final UnknownHostException e) {
			handleConnectionError(e);
		} catch (final MissingParameterException e) {
			handleConnectionError(e);
		} catch (final CommunicationError e) {
			handleConnectionError(e);
		} catch (final ConnectionException e) {
			handleConnectionError(e);
		} catch (final InconsistentLoginException e) {
			handleConnectionError(e);
		}
	}

	private void fireVerbindungsListeners(final DavVerbindungsEvent event) {
		final Type type = event.getType();
		final Collection<DavVerbindungsListener> listeners = new HashSet<DavVerbindungsListener>();
		synchronized (verbindungsListenerList) {
			listeners.addAll(verbindungsListenerList);
		}
		switch (type) {
		case CONNECT:
			for (final DavVerbindungsListener listener : listeners) {
				listener.verbindungHergestellt(event);
			}
			break;
		case DISCONNECT:
			for (final DavVerbindungsListener listener : listeners) {
				SafeRunnable.run(new ISafeRunnable() {

					@Override
					public void run() throws Exception {
						listener.verbindungGetrennt(event);
					}

					@Override
					public void handleException(final Throwable exception) {
						final ILog log = RahmenwerkActivator.getDefault()
								.getLog();
						log.log(new Status(
								IStatus.ERROR,
								RahmenwerkActivator.PLUGIN_ID,
								"Fehler beim unterbrechen der Datenverteiler-Verbindung...",
								exception));
					}
				});

			}
			break;
		default:
			break;
		}
	}

	@Override
	public String getBenutzerName() {
		return benutzerName;
	}

	@Override
	public SystemObject getBenutzer() {
		if (davVerbindung != null) {
			return davVerbindung.getLocalUser();
		}
		return null;
	}

	@Override
	public ClientDavInterface getDavVerbindung() {
		return davVerbindung;
	}

	@Override
	public String getPasswort() {
		return passwort;
	}

	@Override
	public ArgumentList getStartParameter() {
		final String[] argCopy = new String[arguments.length];
		System.arraycopy(arguments, 0, argCopy, 0, arguments.length);
		return new ArgumentList(argCopy);
	}

	private void handleConnectionError(final Exception e)
			throws DavConnectionException {
		if (davVerbindung != null) {
			davVerbindung.disconnect(false, e.getLocalizedMessage());
			davVerbindung = null;
		}

		throw new DavConnectionException(
				"Datenverteilerverbindung konnte nicht hergestellt werden", e);
	}

	@Override
	public boolean isOnline() {
		return davVerbindung != null;
	}

	/**
	 * fhrt die Anmeldung bim Datenverteiler mit den angegebenen Parametern
	 * aus.
	 *
	 * @param daten
	 *            die Anmeldedaten
	 * @throws DavConnectionException
	 *             die Anmeldung/Login konnte nicht ordnungsgem ausgefhrt
	 *             werden
	 */
	public void login(final AnmeldeDaten daten) throws DavConnectionException {

		try {
			davVerbindungHerstellen(daten.getName(), daten.getPasswort(),
					daten.getHost(), daten.getPort());
		} catch (final IllegalArgumentException e) {
			MessageDialog.openError(null, "FEHLER", e.getLocalizedMessage()
					+ "\nDie Applikation wird beendet");
			throw new IllegalArgumentException(
					"Oberflaechenobjekt konnte nicht ermittelt werden", e);
		}
	}

	@Override
	public void removeDavVerbindungsListener(
			final DavVerbindungsListener listener) {
		synchronized (verbindungsListenerList) {
			verbindungsListenerList.remove(listener);
		}
	}

	/**
	 * setzt den aktuellen Benutzername fr das Rahmenwerk.
	 *
	 * @param benutzerName
	 *            der Name
	 */
	public void setBenutzerName(final String benutzerName) {
		this.benutzerName = benutzerName;
	}

	@Override
	public boolean usesBerechtigungenNeu() {
		final ArgumentList argumente = getStartParameter();
		if (argumente.hasArgument("-rechtePruefung")) {
			final Argument argument = argumente
					.fetchArgument("-rechtePruefung");
			if ("neu".equals(argument.getValue())) {
				return true;
			}
		}
		return false;
	}

	@Override
	public SystemObject getOberflaechenObject() {
		SystemObject result = null;

		if (davVerbindung != null) {
			final ArgumentList argumente = getStartParameter();
			if (argumente.hasArgument("-oberflaechenObjekt")) {
				final Argument argument = argumente
						.fetchArgument("-oberflaechenObjekt");
				result = davVerbindung.getDataModel().getObject(
						argument.getValue());
				if (result == null) {
					throw new IllegalArgumentException(
							"Das geforderte Oberflchenobjekt mit der PID \""
									+ argument.getValue()
									+ "\" ist in der verwendeten Konfiguration nicht verfgbar!");
				}
			} else {
				result = davVerbindung.getLocalConfigurationAuthority();
			}
		}

		return result;
	}

	@Override
	public RwToolbarElementManagerImpl getRwToolBarManager() {
		return toolbarElementManager;
	}
}
