/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.ui.services.IServiceLocator;
import org.osgi.framework.Bundle;

/**
 * Abstrakte Implementierung der Definition eines Elements, das in eine Toolbar
 * oder das Men der Rahmenwerksapplikation eingebunden werden kann.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwToolbarElement.java 45420 2013-08-09 12:12:12Z peuker $
 */
public abstract class RwToolbarElement {

	static final String ATT_ID = "id";

	static final String ATT_NAME = "name";

	static final String ATT_LABEL = "label";

	private final IConfigurationElement configuration;

	private final Map<String, RwToolbarElementParameter> parameters = new HashMap<String, RwToolbarElementParameter>();

	/**
	 * Konstruktor, erzeugt eine Elementdefinition auf Basis der bergebenen
	 * Konfigurationsdaten aus der entsprechenden Plugin-Definition.
	 * 
	 * @param configuration
	 *            die Konfigurationsdaten
	 */
	public RwToolbarElement(final IConfigurationElement configuration) {
		this.configuration = configuration;
	}

	/**
	 * fgt der Definition einen Parameter hinzu.
	 * 
	 * @param parameter
	 *            der Parameter
	 */
	protected void addParameter(final RwToolbarElementParameter parameter) {
		parameters.put(parameter.getId(), parameter);
	}

	/**
	 * erzeugt die Contribution, die als Instanz der Definition in einer Toolbar
	 * oder einem Men eingebunden wird.
	 * 
	 * @param locator
	 *            der Servicelocator (in der Regel die Workbench)
	 * @param element
	 *            das Element, das die Einbindung in das Men oder eine Toolbar
	 *            beschreibt und die konkreten Parameter enthlt
	 * @return das Contribution-Elemente
	 * @throws CoreException
	 *             das Element konnte nicht erzeugt werden.
	 */
	public abstract IContributionItem createContributionItem(
			final IServiceLocator locator, MenueAction element)
			throws CoreException;

	/**
	 * liefert den beschreibenden Text des Elements.
	 * 
	 * @return den Text
	 */
	public String getBeschreibung() {
		return getName();
	}

	/**
	 * liefert den Name des Bundles, in dem die Definition vorgenommen wurde.
	 * 
	 * @return der Name
	 */
	public String getBundleName() {
		final Bundle bundle = Platform.getBundle(getPluginId());
		return bundle.getHeaders().get("Bundle-Name");
	}

	/**
	 * liefert die Konfiguration (plugin.xml) aus der die Definition erstellt
	 * wurde.
	 * 
	 * @return die Konfiguration
	 */
	protected IConfigurationElement getConfiguration() {
		return configuration;
	}

	/**
	 * liefert die ID des Elements.
	 * 
	 * @return die ID
	 */
	public String getId() {
		return configuration.getAttribute(RwToolbarElement.ATT_ID);
	}

	/**
	 * liefert den Name des Elements.
	 * 
	 * @return den Name
	 */
	public String getName() {
		String result = configuration.getAttribute(RwToolbarElement.ATT_LABEL);

		if (result == null) {
			result = configuration.getAttribute(RwToolbarElement.ATT_NAME);
		}
		if (result == null) {
			return getId();
		}

		return result;
	}

	/**
	 * liefert die Parameter der Definition.
	 * 
	 * @return die Parameter
	 */
	public Map<String, RwToolbarElementParameter> getParameters() {
		return Collections.unmodifiableMap(parameters);
	}

	/**
	 * liefert die ID des Plug-ins, in dem die Definition erfolgte.
	 * 
	 * @return die ID
	 */
	public String getPluginId() {
		return configuration.getContributor().getName();
	}

	@Override
	public String toString() {
		return getName() + "(" + getId() + ")";
	}
}
