/*
 * Copyright (C) 2004 by Friederich Kupzog Elektronik & Software
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package de.kupzog.ktable;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.HTMLTransfer;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.ActionFactory;

/**
 * 
 * @author Lorenz Maierhofer <lorenz.maierhofer@logicmindguide.com>
 */
public class KTableActionHandler {

	static final char TAB = '\t';

	static final String PlatformLineDelimiter = System
			.getProperty("line.separator");

	public KTableCopyAction m_CopyAction;

	public KTableCopyAllAction m_CopyAllAction;

	public KTableCutAction m_CutAction;

	public KTablePasteAction m_PasteAction;

	public KTableSelectAllAction m_SelectAllAction;

	protected KTable m_table;

	protected MenuManager m_contextMenuManager;

	/**
   * 
   */
	public KTableActionHandler(final KTable table) {
		m_table = table;
		createActions();
		registerActionUpdater();

		// add actions to context menu:
		m_contextMenuManager = new MenuManager("#PopupMenu");
		m_contextMenuManager.setRemoveAllWhenShown(true);
		m_contextMenuManager.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(final IMenuManager manager) {
				fillContextMenu(manager);
			}
		});
		final Menu menu = m_contextMenuManager.createContextMenu(m_table);
		m_table.setMenu(menu);
	}

	/**
	 * @return Returns the menu manager used to build the context menu of the
	 *         table.
	 *         <p>
	 *         The purpose for this is normally the registering of context menus
	 *         in the workbench.
	 * @see org.eclipse.ui.IWorkbenchPartSite#registerContextMenu(org.eclipse.jface.action.MenuManager,
	 *      org.eclipse.jface.viewers.ISelectionProvider)
	 */
	public MenuManager getMenuManager() {
		return m_contextMenuManager;
	}

	protected void createActions() {
		m_CopyAction = new KTableCopyAction();
		m_CopyAllAction = new KTableCopyAllAction();
		m_PasteAction = new KTablePasteAction();
		m_CutAction = new KTableCutAction();
		m_SelectAllAction = new KTableSelectAllAction();
	}

	protected void fillContextMenu(final IMenuManager menumanager) {
		menumanager.add(m_CopyAction);
		menumanager.add(m_CutAction);
		menumanager.add(m_PasteAction);
		menumanager.add(new Separator());
		menumanager.add(m_CopyAllAction);
		menumanager.add(m_SelectAllAction);
		menumanager.add(new Separator());
		// Other plug-ins can contribute their actions here
		menumanager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/**
	 * Registers the cut, copy, paste and select_all actions for global use at
	 * the IActionBar given.
	 * <p>
	 * Currently does not set up the UNDO and REDO actions because they will be
	 * implemented in another way.
	 * 
	 * @param actionBar
	 *            The IActionBars that allows global action registration.
	 *            Normally you can get that with getViewerSite().getActionBars()
	 *            or getEditorSite().getActionBars().
	 */
	public void registerGlobalActions(final IActionBars actionBar) {
		actionBar.setGlobalActionHandler(ActionFactory.CUT.getId(),
				this.m_CutAction);
		actionBar.setGlobalActionHandler(ActionFactory.COPY.getId(),
				this.m_CopyAction);
		actionBar.setGlobalActionHandler(ActionFactory.PASTE.getId(),
				this.m_PasteAction);
		actionBar.setGlobalActionHandler(ActionFactory.SELECT_ALL.getId(),
				this.m_SelectAllAction);
		actionBar.updateActionBars();
	}

	protected class KTableCopyAction extends Action {
		protected KTableCopyAction() {
			setId("KTableCopyActionHandler");//$NON-NLS-1$
			setEnabled(false);
			setText("Kopieren");
		}

		@Override
		public void run() {
			if ((m_table != null) && !m_table.isDisposed()) {
				setClipboardContent(m_table.getCellSelection());
			}
		}

		public void updateEnabledState() {
			if ((m_table != null) && !m_table.isDisposed()) {
				final Point[] selection = m_table.getCellSelection();
				setEnabled((selection != null) && (selection.length > 0));
			} else {
				setEnabled(false);
			}
		}
	}

	protected class KTableCopyAllAction extends Action {
		protected KTableCopyAllAction() {
			setId("KTableCopyAllActionHandler");//$NON-NLS-1$
			setEnabled(false);
			setText("Ganze Tabelle kopieren");
		}

		@Override
		public void run() {
			if ((m_table != null) && !m_table.isDisposed()) {
				setClipboardContent(getAllTableCells());
			}
		}

		public void updateEnabledState() {
			if ((m_table != null) && !m_table.isDisposed()) {
				setEnabled(true);
			} else {
				setEnabled(false);
			}
		}

		private Point[] getAllTableCells() {
			final KTableModel model = m_table.getModel();
			if (model == null) {
				return new Point[] {};
			}
			final Vector cells = new Vector(model.getColumnCount()
					* model.getRowCount());
			for (int row = 0; row < model.getRowCount(); row++) {
				for (int col = 0; col < model.getColumnCount(); col++) {
					final Point valid = model.belongsToCell(col, row);
					if ((valid.y == row) && (valid.x == col)) {
						cells.add(valid);
					}
				}
			}
			return (Point[]) cells.toArray(new Point[] {});
		}
	}

	protected class KTableCutAction extends Action {
		protected KTableCutAction() {
			setId("KTableCutActionHandler");//$NON-NLS-1$
			setEnabled(false);
			setText("Ausschneiden");
		}

		@Override
		public void run() {
			if ((m_table != null) && !m_table.isDisposed()) {
				final Point[] selection = m_table.getCellSelection();
				setClipboardContent(selection);
				removeContentAt(selection);
			}
		}

		public void updateEnabledState() {
			if ((m_table != null) && !m_table.isDisposed()) {
				final Point[] selection = m_table.getCellSelection();
				setEnabled((selection != null) && (selection.length > 0));
			} else {
				setEnabled(false);
			}
		}

		protected void removeContentAt(final Point[] selection) {
			final KTableModel model = m_table.getModel();
			if (model == null) {
				return;
			}
			final boolean updateSeperateCells = selection.length > 4 ? false
					: true;
			try {
				if (!updateSeperateCells) {
					m_table.setRedraw(false);
				}
				for (final Point element : selection) {
					model.setContentAt(element.x, element.y, "");
					if (updateSeperateCells) {
						m_table.updateCell(element.x, element.y);
					}
				}
			} finally {
				if (!updateSeperateCells) {
					m_table.setRedraw(true);
				}
			}
		}
	}

	protected class KTableSelectAllAction extends Action {
		protected KTableSelectAllAction() {
			setId("KTableSelectAllActionHandler");//$NON-NLS-1$
			setEnabled(false);
			setText("Alles Markieren");
		}

		@Override
		public void run() {
			if ((m_table != null) && !m_table.isDisposed()) {
				final KTableModel model = m_table.getModel();
				if (model != null) {
					selectAll(model);
				}
			}
		}

		public void updateEnabledState() {
			if ((m_table != null) && !m_table.isDisposed()
					&& m_table.isMultiSelectMode()) {
				setEnabled(true);
			} else {
				setEnabled(false);
			}
		}

		protected void selectAll(final KTableModel model) {
			final Vector sel = new Vector();
			for (int row = model.getFixedHeaderRowCount(); row < model
					.getRowCount(); row++) {
				for (int col = model.getFixedHeaderColumnCount(); col < model
						.getColumnCount(); col++) {
					final Point cell = model.belongsToCell(col, row);
					if ((cell.x == col) && (cell.y == row)) {
						sel.add(cell);
					}
				}
			}
			try {
				m_table.setRedraw(false);
				m_table.setSelection(new Point[] {}, false);
				m_table.setSelection((Point[]) sel.toArray(new Point[] {}),
						false);
			} finally {
				m_table.setRedraw(true);
			}
		}
	}

	protected class KTablePasteAction extends Action {
		protected KTablePasteAction() {
			setId("KTablePasteActionHandler");//$NON-NLS-1$
			setEnabled(false);
			setText("Einfgen");
		}

		@Override
		public void run() {
			if ((m_table != null) && !m_table.isDisposed()) {
				pasteToSelection(getTextFromClipboard(),
						m_table.getCellSelection());
			}
		}

		protected String getTextFromClipboard() {
			final Clipboard clipboard = new Clipboard(m_table.getDisplay());
			try {
				return clipboard.getContents(TextTransfer.getInstance())
						.toString();
			} catch (final Exception ex) {
				return "";
			} finally {
				clipboard.dispose();
			}
		}

		protected void pasteToSelection(final String text,
				final Point[] selection) {
			if ((selection == null) || (selection.length == 0)) {
				return;
			}
			final KTableModel model = m_table.getModel();
			if (model == null) {
				return;
			}

			try {
				m_table.setRedraw(false);
				m_table.setSelection(new Point[] {}, false);
				final Vector sel = new Vector();

				final String[][] cellTexts = parseCellTexts(text);
				for (int row = 0; row < cellTexts.length; row++) {
					for (int col = 0; col < cellTexts[row].length; col++) {
						model.setContentAt(col + selection[0].x, row
								+ selection[0].y, cellTexts[row][col]);
						sel.add(new Point(col + selection[0].x, row
								+ selection[0].y));
					}
				}
				m_table.setSelection((Point[]) sel.toArray(new Point[] {}),
						false);
			} finally {
				m_table.setRedraw(true);
			}
		}

		protected String[][] parseCellTexts(final String text) {
			if (!m_table.isMultiSelectMode()) {
				return new String[][] { { text } };
			} else {
				final String[] lines = text
						.split(KTableActionHandler.PlatformLineDelimiter);
				final String[][] cellText = new String[lines.length][];
				for (int line = 0; line < lines.length; line++) {
					cellText[line] = lines[line].split(KTableActionHandler.TAB
							+ "");
				}
				return cellText;
			}
		}

		public void updateEnabledState() {
			if ((m_table != null) && !m_table.isDisposed()) {
				final Point[] selection = m_table.getCellSelection();
				if (selection == null) {
					setEnabled(false);
				} else if (selection.length > 1) {
					setEnabled(false);
				} else {
					setEnabled(true);
				}
			} else {
				setEnabled(false);
			}
		}
	}

	/**
	 * Copies the specified text range to the clipboard. The table will be
	 * placed in the clipboard in plain text format and RTF format.
	 * 
	 * @param selection
	 *            The list of cell indices thats content should be set to the
	 *            clipboard.
	 * 
	 * @exception SWTError
	 *                , see Clipboard.setContents
	 * @see org.eclipse.swt.dnd.Clipboard.setContents
	 */
	protected void setClipboardContent(final Point[] selection) throws SWTError {
		// RTFTransfer rtfTransfer = RTFTransfer.getInstance();
		final TextTransfer plainTextTransfer = TextTransfer.getInstance();
		final HTMLTransfer htmlTransfer = HTMLTransfer.getInstance();

		// String rtfText = getRTFForSelection(selection);
		final String plainText = getTextForSelection(selection);
		final String htmlText = getHTMLForSelection(selection);

		final Clipboard clipboard = new Clipboard(m_table.getDisplay());
		try {
			clipboard.setContents(new String[] { plainText, htmlText }, // rtfText
					new Transfer[] { plainTextTransfer, htmlTransfer }); // rtfTransfer
		} catch (final SWTError error) {
			// Copy to clipboard failed. This happens when another application
			// is accessing the clipboard while we copy. Ignore the error.
			// Rethrow all other errors.
			if (error.code != DND.ERROR_CANNOT_SET_CLIPBOARD) {
				throw error;
			}
		} finally {
			clipboard.dispose();
		}
	}

	private Point[] findTableDimensions(final Point[] selection) {
		final Point topLeft = new Point(-1, -1);
		final Point bottomRight = new Point(-1, -1);

		for (final Point cell : selection) {
			if (topLeft.x < 0) {
				topLeft.x = cell.x;
			} else if (topLeft.x > cell.x) {
				topLeft.x = cell.x;
			}
			if (bottomRight.x < 0) {
				bottomRight.x = cell.x;
			} else if (bottomRight.x < cell.x) {
				bottomRight.x = cell.x;
			}

			if (topLeft.y < 0) {
				topLeft.y = cell.y;
			} else if (topLeft.y > cell.y) {
				topLeft.y = cell.y;
			}
			if (bottomRight.y < 0) {
				bottomRight.y = cell.y;
			} else if (bottomRight.y < cell.y) {
				bottomRight.y = cell.y;
			}
		}
		return new Point[] { topLeft, bottomRight };
	}

	private Point findCellSpanning(final int col, final int row,
			final KTableModel model) {
		final Point spanning = new Point(1, 1);
		final Point cell = new Point(col, row);
		while (model.belongsToCell(col + spanning.x, row).equals(cell)) {
			spanning.x++;
		}

		while (model.belongsToCell(col, row + spanning.y).equals(cell)) {
			spanning.y++;
		}

		return spanning;
	}

	protected String getHTMLForSelection(final Point[] selection) {
		final StringBuffer html = new StringBuffer();
		sortSelectedCells(selection);

		final Point[] dimensions = findTableDimensions(selection);
		final Point topLeft = dimensions[0];
		final Point bottomRight = dimensions[1];

		final KTableModel model = m_table.getModel();
		if (model == null) {
			return "";
		}
		// add header:
		html.append("Version:1.0\n");
		html.append("StartHTML:0000000000\n");
		html.append("EndHTML:0000000000\n");
		html.append("StartFragment:0000000000\n");
		html.append("EndFragment:0000000000\n");
		html.append("<html><body><table>");

		Point nextValidCell = selection[0];
		int selCounter = 1;
		for (int row = topLeft.y; row <= bottomRight.y; row++) {
			html.append("<tr>");
			for (int col = topLeft.x; col <= bottomRight.x; col++) {
				// may skip the cell when it is spanned by another one.
				if (model.belongsToCell(col, row).equals(new Point(col, row))) {

					if ((nextValidCell.x == col) && (nextValidCell.y == row)) {
						html.append("<td");
						final Point spanning = findCellSpanning(col, row, model);
						if (spanning.x > 1) {
							html.append(" colspan=\"" + spanning.x + "\"");
						}
						if (spanning.y > 1) {
							html.append(" rowspan=\"" + spanning.y + "\"");
						}
						html.append(">");

						final Object content = model.getContentAt(col, row);
						html.append(maskHtmlChars(content.toString()));
						if (selCounter < selection.length) {
							nextValidCell = selection[selCounter];
							selCounter++;
						}
					} else {
						html.append("<td>");
					}

					html.append("</td>");
				}
			}
			html.append("</tr>");
		}
		html.append("</table></body></html>");

		return html.toString();
	}

	private String maskHtmlChars(String text) {
		text = text.replaceAll("&", "&amp;");
		text = text.replaceAll("", "&auml;");
		text = text.replaceAll("", "&Auml;");
		text = text.replaceAll("", "&ouml;");
		text = text.replaceAll("", "&Ouml;");
		text = text.replaceAll("", "&uuml;");
		text = text.replaceAll("", "&Uuml;");
		text = text.replaceAll("", "&szlig;");
		text = text.replaceAll("\"", "&quot;");
		text = text.replaceAll("<", "&lt");
		text = text.replaceAll(">", "&gt");
		text = text.replaceAll("", "&euro;");
		return text;
	}

	protected String getTextForSelection(final Point[] selection) {
		final StringBuffer text = new StringBuffer();
		final Point topLeft = sortSelectedCells(selection);
		final KTableModel model = m_table.getModel();
		if (model == null) {
			return "";
		}

		int currentCol = topLeft.x;
		for (int i = 0; i < selection.length; i++) {
			for (; currentCol < selection[i].x; currentCol++) {
				text.append(KTableActionHandler.TAB);
			}

			final Object content = model.getContentAt(selection[i].x,
					selection[i].y);
			text.append(content.toString());

			if ((i + 1) < selection.length) {
				for (int row = selection[i].y; row < selection[i + 1].y; row++) {
					text.append(KTableActionHandler.PlatformLineDelimiter);
				}
				if (selection[i].y != selection[i + 1].y) {
					currentCol = topLeft.x;
				}
			}
		}
		return text.toString();
	}

	protected Point sortSelectedCells(final Point[] selection) {
		Arrays.sort(selection, new Comparator() {

			@Override
			public int compare(final Object o1, final Object o2) {
				final Point p1 = (Point) o1;
				final Point p2 = (Point) o2;
				if (p1.y < p2.y) {
					return -1;
				}
				if (p1.y > p2.y) {
					return +1;
				}
				if (p1.x < p2.x) {
					return -1;
				}
				if (p1.x > p2.x) {
					return +1;
				}
				return 0;
			}

		});

		int minCol = selection[0].x;
		for (int i = 1; i < selection.length; i++) {
			if (selection[i].x < minCol) {
				minCol = selection[i].x;
			}
		}
		return new Point(minCol, selection[0].y);
	}

	protected String getRTFForSelection(final Point[] selection) {
		return getTextForSelection(selection);
	}

	protected void registerActionUpdater() {
		m_table.addCellSelectionListener(new KTableCellSelectionListener() {

			@Override
			public void cellSelected(final int col, final int row,
					final int statemask) {
				updateActions();
			}

			@Override
			public void fixedCellSelected(final int col, final int row,
					final int statemask) {
				updateActions();
			}
		});
	}

	protected void updateActions() {
		m_CopyAction.updateEnabledState();
		m_CopyAllAction.updateEnabledState();
		m_CutAction.updateEnabledState();
		m_PasteAction.updateEnabledState();
		m_SelectAllAction.updateEnabledState();
	}
}
